@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,24 +12,24 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 
 #include "libgimpwidgets/gimpwidgets.h"
 
 #include "actions-types.h"
 
-#include "config/gimpguiconfig.h"
-
 #include "core/gimp.h"
 #include "core/gimpcontainer.h"
 #include "core/gimpcontext.h"
 #include "core/gimpimage.h"
+#include "core/gimptooloptions.h"
+#include "core/gimptoolinfo.h"
 
 #include "widgets/gimpactionfactory.h"
 #include "widgets/gimpactiongroup.h"
@@ -37,12 +37,15 @@
 #include "widgets/gimpcontainerview.h"
 #include "widgets/gimpdock.h"
 #include "widgets/gimpdockable.h"
+#include "widgets/gimpdockwindow.h"
 #include "widgets/gimpimageeditor.h"
 #include "widgets/gimpitemtreeview.h"
 
 #include "display/gimpdisplay.h"
 #include "display/gimpdisplayshell.h"
+#include "display/gimpimagewindow.h"
 #include "display/gimpnavigationeditor.h"
+#include "display/gimpstatusbar.h"
 
 #include "dialogs/dialogs.h"
 
@@ -52,18 +55,21 @@
 #include "buffers-actions.h"
 #include "channels-actions.h"
 #include "colormap-actions.h"
-#include "config-actions.h"
 #include "context-actions.h"
 #include "cursor-info-actions.h"
+#include "dashboard-actions.h"
 #include "debug-actions.h"
 #include "dialogs-actions.h"
 #include "dock-actions.h"
 #include "dockable-actions.h"
 #include "documents-actions.h"
 #include "drawable-actions.h"
+#include "dynamics-actions.h"
+#include "dynamics-editor-actions.h"
 #include "edit-actions.h"
 #include "error-console-actions.h"
 #include "file-actions.h"
+#include "filters-actions.h"
 #include "fonts-actions.h"
 #include "gradient-editor-actions.h"
 #include "gradients-actions.h"
@@ -71,6 +77,7 @@
 #include "image-actions.h"
 #include "images-actions.h"
 #include "layers-actions.h"
+#include "mypaint-brushes-actions.h"
 #include "palette-editor-actions.h"
 #include "palettes-actions.h"
 #include "patterns-actions.h"
@@ -80,8 +87,12 @@
 #include "select-actions.h"
 #include "templates-actions.h"
 #include "text-editor-actions.h"
+#include "text-tool-actions.h"
 #include "tool-options-actions.h"
+#include "tool-presets-actions.h"
+#include "tool-preset-editor-actions.h"
 #include "tools-actions.h"
+#include "vector-toolpath-actions.h"
 #include "vectors-actions.h"
 #include "view-actions.h"
 #include "windows-actions.h"
@@ -96,32 +107,32 @@ GimpActionFactory *global_action_factory = NULL;
 
 /*  private variables  */
 
-static GimpActionFactoryEntry action_groups[] =
+static const GimpActionFactoryEntry action_groups[] =
 {
-  { "brush-editor", N_("Brush Editor"), GIMP_STOCK_BRUSH,
+  { "brush-editor", N_("Brush Editor"), GIMP_ICON_BRUSH,
     brush_editor_actions_setup,
     brush_editor_actions_update },
-  { "brushes", N_("Brushes"), GIMP_STOCK_BRUSH,
+  { "brushes", N_("Brushes"), GIMP_ICON_BRUSH,
     brushes_actions_setup,
     brushes_actions_update },
-  { "buffers", N_("Buffers"), GIMP_STOCK_BUFFER,
+  { "buffers", N_("Buffers"), GIMP_ICON_BUFFER,
     buffers_actions_setup,
     buffers_actions_update },
-  { "channels", N_("Channels"), GIMP_STOCK_CHANNEL,
+  { "channels", N_("Channels"), GIMP_ICON_CHANNEL,
     channels_actions_setup,
     channels_actions_update },
-  { "colormap", N_("Colormap"), GIMP_STOCK_COLORMAP,
+  { "colormap", N_("Colormap"), GIMP_ICON_COLORMAP,
     colormap_actions_setup,
     colormap_actions_update },
-  { "config", N_("Configuration"), GTK_STOCK_PREFERENCES,
-    config_actions_setup,
-    config_actions_update },
-  { "context", N_("Context"), GIMP_STOCK_TOOL_OPTIONS /* well... */,
+  { "context", N_("Context"), GIMP_ICON_DIALOG_TOOL_OPTIONS /* well... */,
     context_actions_setup,
     context_actions_update },
   { "cursor-info", N_("Pointer Information"), NULL,
     cursor_info_actions_setup,
     cursor_info_actions_update },
+  { "dashboard", N_("Dashboard"), GIMP_ICON_DIALOG_DASHBOARD,
+    dashboard_actions_setup,
+    dashboard_actions_update },
   { "debug", N_("Debug"), NULL,
     debug_actions_setup,
     debug_actions_update },
@@ -137,76 +148,100 @@ static GimpActionFactoryEntry action_groups[] =
   { "documents", N_("Document History"), NULL,
     documents_actions_setup,
     documents_actions_update },
-  { "drawable", N_("Drawable"), GIMP_STOCK_LAYER,
+  { "drawable", N_("Drawable"), GIMP_ICON_LAYER,
     drawable_actions_setup,
     drawable_actions_update },
-  { "edit", N_("Edit"), GTK_STOCK_EDIT,
+  { "dynamics", N_("Paint Dynamics"), GIMP_ICON_DYNAMICS,
+    dynamics_actions_setup,
+    dynamics_actions_update },
+  { "dynamics-editor", N_("Paint Dynamics Editor"), GIMP_ICON_DYNAMICS,
+    dynamics_editor_actions_setup,
+    dynamics_editor_actions_update },
+  { "edit", N_("Edit"), GIMP_ICON_EDIT,
     edit_actions_setup,
     edit_actions_update },
-  { "error-console", N_("Error Console"), GIMP_STOCK_WARNING,
+  { "error-console", N_("Error Console"), GIMP_ICON_DIALOG_WARNING,
     error_console_actions_setup,
     error_console_actions_update },
-  { "file", N_("File"), GTK_STOCK_FILE,
+  { "file", N_("File"), "text-x-generic",
     file_actions_setup,
     file_actions_update },
-  { "fonts", N_("Fonts"), GIMP_STOCK_FONT,
+  { "filters", N_("Filters"), GIMP_ICON_GEGL,
+    filters_actions_setup,
+    filters_actions_update },
+  { "fonts", N_("Fonts"), GIMP_ICON_FONT,
     fonts_actions_setup,
     fonts_actions_update },
-  { "gradient-editor", N_("Gradient Editor"), GIMP_STOCK_GRADIENT,
+  { "gradient-editor", N_("Gradient Editor"), GIMP_ICON_GRADIENT,
     gradient_editor_actions_setup,
     gradient_editor_actions_update },
-  { "gradients", N_("Gradients"), GIMP_STOCK_GRADIENT,
+  { "gradients", N_("Gradients"), GIMP_ICON_GRADIENT,
     gradients_actions_setup,
     gradients_actions_update },
-  { "help", N_("Help"), GTK_STOCK_HELP,
+  { "tool-presets", N_("Tool Presets"), GIMP_ICON_TOOL_PRESET,
+    tool_presets_actions_setup,
+    tool_presets_actions_update },
+  { "tool-preset-editor", N_("Tool Preset Editor"), GIMP_ICON_TOOL_PRESET,
+    tool_preset_editor_actions_setup,
+    tool_preset_editor_actions_update },
+  { "help", N_("Help"), "help-browser",
     help_actions_setup,
     help_actions_update },
-  { "image", N_("Image"), GIMP_STOCK_IMAGE,
+  { "image", N_("Image"), GIMP_ICON_IMAGE,
     image_actions_setup,
     image_actions_update },
-  { "images", N_("Images"), GIMP_STOCK_IMAGE,
+  { "images", N_("Images"), GIMP_ICON_IMAGE,
     images_actions_setup,
     images_actions_update },
-  { "layers", N_("Layers"), GIMP_STOCK_LAYER,
+  { "layers", N_("Layers"), GIMP_ICON_LAYER,
     layers_actions_setup,
     layers_actions_update },
-  { "palette-editor", N_("Palette Editor"), GIMP_STOCK_PALETTE,
+  { "mypaint-brushes", N_("MyPaint Brushes"), GIMP_ICON_MYPAINT_BRUSH,
+    mypaint_brushes_actions_setup,
+    mypaint_brushes_actions_update },
+  { "palette-editor", N_("Palette Editor"), GIMP_ICON_PALETTE,
     palette_editor_actions_setup,
     palette_editor_actions_update },
-  { "palettes", N_("Palettes"), GIMP_STOCK_PALETTE,
+  { "palettes", N_("Palettes"), GIMP_ICON_PALETTE,
     palettes_actions_setup,
     palettes_actions_update },
-  { "patterns", N_("Patterns"), GIMP_STOCK_PATTERN,
+  { "patterns", N_("Patterns"), GIMP_ICON_PATTERN,
     patterns_actions_setup,
     patterns_actions_update },
-  { "plug-in", N_("Plug-Ins"), GIMP_STOCK_PLUGIN,
+  { "plug-in", N_("Plug-ins"), GIMP_ICON_PLUGIN,
     plug_in_actions_setup,
     plug_in_actions_update },
-  { "quick-mask", N_("Quick Mask"), GIMP_STOCK_QUICK_MASK_ON,
+  { "quick-mask", N_("Quick Mask"), GIMP_ICON_QUICK_MASK_ON,
     quick_mask_actions_setup,
     quick_mask_actions_update },
-  { "sample-points", N_("Sample Points"), GIMP_STOCK_SAMPLE_POINT,
+  { "sample-points", N_("Sample Points"), GIMP_ICON_SAMPLE_POINT,
     sample_points_actions_setup,
     sample_points_actions_update },
-  { "select", N_("Select"), GIMP_STOCK_SELECTION,
+  { "select", N_("Select"), GIMP_ICON_SELECTION,
     select_actions_setup,
     select_actions_update },
-  { "templates", N_("Templates"), GIMP_STOCK_TEMPLATE,
+  { "templates", N_("Templates"), GIMP_ICON_TEMPLATE,
     templates_actions_setup,
     templates_actions_update },
-  { "text-editor", N_("Text Editor"), GTK_STOCK_EDIT,
+  { "text-tool", N_("Text Tool"), GIMP_ICON_EDIT,
+    text_tool_actions_setup,
+    text_tool_actions_update },
+  { "text-editor", N_("Text Editor"), GIMP_ICON_EDIT,
     text_editor_actions_setup,
     text_editor_actions_update },
-  { "tool-options", N_("Tool Options"), GIMP_STOCK_TOOL_OPTIONS,
+  { "tool-options", N_("Tool Options"), GIMP_ICON_DIALOG_TOOL_OPTIONS,
     tool_options_actions_setup,
     tool_options_actions_update },
-  { "tools", N_("Tools"), GIMP_STOCK_TOOLS,
+  { "tools", N_("Tools"), GIMP_ICON_DIALOG_TOOLS,
     tools_actions_setup,
     tools_actions_update },
-  { "vectors", N_("Paths"), GIMP_STOCK_PATH,
+  { "vector-toolpath", N_("Path Toolpath"), GIMP_ICON_PATH,
+    vector_toolpath_actions_setup,
+    vector_toolpath_actions_update },
+  { "vectors", N_("Paths"), GIMP_ICON_PATH,
     vectors_actions_setup,
     vectors_actions_update },
-  { "view", N_("View"), GIMP_STOCK_VISIBLE,
+  { "view", N_("View"), GIMP_ICON_VISIBLE,
     view_actions_setup,
     view_actions_update },
   { "windows", N_("Windows"), NULL,
@@ -220,22 +255,18 @@ static GimpActionFactoryEntry action_groups[] =
 void
 actions_init (Gimp *gimp)
 {
-  GimpGuiConfig *gui_config;
-  gint           i;
+  gint i;
 
   g_return_if_fail (GIMP_IS_GIMP (gimp));
   g_return_if_fail (global_action_factory == NULL);
 
-  gui_config = GIMP_GUI_CONFIG (gimp->config);
-
-  global_action_factory = gimp_action_factory_new (gimp,
-                                                   gui_config->menu_mnemonics);
+  global_action_factory = gimp_action_factory_new (gimp);
 
   for (i = 0; i < G_N_ELEMENTS (action_groups); i++)
     gimp_action_factory_group_register (global_action_factory,
                                         action_groups[i].identifier,
                                         gettext (action_groups[i].label),
-                                        action_groups[i].stock_id,
+                                        action_groups[i].icon_name,
                                         action_groups[i].setup_func,
                                         action_groups[i].update_func);
 }
@@ -247,134 +278,240 @@ actions_exit (Gimp *gimp)
   g_return_if_fail (global_action_factory != NULL);
   g_return_if_fail (global_action_factory->gimp == gimp);
 
-  g_object_unref (global_action_factory);
-  global_action_factory = NULL;
+  g_clear_object (&global_action_factory);
 }
 
 Gimp *
 action_data_get_gimp (gpointer data)
 {
-  GimpContext *context = NULL;
+  Gimp            *result    = NULL;
+  static gboolean  recursion = FALSE;
 
-  if (! data)
+  if (! data || recursion)
     return NULL;
 
-  if (GIMP_IS_DISPLAY (data))
-    return ((GimpDisplay *) data)->gimp;
-  else if (GIMP_IS_GIMP (data))
-    return data;
-  else if (GIMP_IS_DOCK (data))
-    context = ((GimpDock *) data)->context;
-  else if (GIMP_IS_CONTAINER_VIEW (data))
-    context = gimp_container_view_get_context ((GimpContainerView *) data);
-  else if (GIMP_IS_CONTAINER_EDITOR (data))
-    context = gimp_container_view_get_context (((GimpContainerEditor *) data)->view);
-  else if (GIMP_IS_IMAGE_EDITOR (data))
-    context = ((GimpImageEditor *) data)->context;
-  else if (GIMP_IS_NAVIGATION_EDITOR (data))
-    context = ((GimpNavigationEditor *) data)->context;
+  recursion = TRUE;
 
-  if (context)
-    return context->gimp;
+  if (GIMP_IS_GIMP (data))
+    result = data;
 
-  return NULL;
+  if (! result)
+    {
+      GimpDisplay *display = action_data_get_display (data);
+
+      if (display)
+        result = display->gimp;
+    }
+
+  if (! result)
+    {
+      GimpContext *context = action_data_get_context (data);
+
+      if (context)
+        result = context->gimp;
+    }
+
+  recursion = FALSE;
+
+  return result;
 }
 
 GimpContext *
 action_data_get_context (gpointer data)
 {
-  if (! data)
+  GimpContext     *result    = NULL;
+  static gboolean  recursion = FALSE;
+
+  if (! data || recursion)
     return NULL;
 
-  if (GIMP_IS_DISPLAY (data))
-    return gimp_get_user_context (((GimpDisplay *) data)->gimp);
-  else if (GIMP_IS_GIMP (data))
-    return gimp_get_user_context (data);
-  else if (GIMP_IS_DOCK (data))
-    return ((GimpDock *) data)->context;
-  else if (GIMP_IS_CONTAINER_VIEW (data))
-    return gimp_container_view_get_context ((GimpContainerView *) data);
-  else if (GIMP_IS_CONTAINER_EDITOR (data))
-    return gimp_container_view_get_context (((GimpContainerEditor *) data)->view);
-  else if (GIMP_IS_IMAGE_EDITOR (data))
-    return ((GimpImageEditor *) data)->context;
-  else if (GIMP_IS_NAVIGATION_EDITOR (data))
-    return ((GimpNavigationEditor *) data)->context;
+  recursion = TRUE;
 
-  return NULL;
+  if (GIMP_IS_DOCK (data))
+    result = gimp_dock_get_context ((GimpDock *) data);
+  else if (GIMP_IS_DOCK_WINDOW (data))
+    result = gimp_dock_window_get_context (((GimpDockWindow *) data));
+  else if (GIMP_IS_CONTAINER_VIEW (data))
+    result = gimp_container_view_get_context ((GimpContainerView *) data);
+  else if (GIMP_IS_CONTAINER_EDITOR (data))
+    result = gimp_container_view_get_context (((GimpContainerEditor *) data)->view);
+  else if (GIMP_IS_IMAGE_EDITOR (data))
+    result = ((GimpImageEditor *) data)->context;
+  else if (GIMP_IS_NAVIGATION_EDITOR (data))
+    result = ((GimpNavigationEditor *) data)->context;
+
+  if (! result)
+    {
+      Gimp *gimp = action_data_get_gimp (data);
+
+      if (gimp)
+        result = gimp_get_user_context (gimp);
+    }
+
+  recursion = FALSE;
+
+  return result;
 }
 
 GimpImage *
 action_data_get_image (gpointer data)
 {
-  GimpContext *context = NULL;
+  GimpImage       *result    = NULL;
+  static gboolean  recursion = FALSE;
 
-  if (! data)
+  if (! data || recursion)
     return NULL;
 
-  if (GIMP_IS_DISPLAY (data))
-    return ((GimpDisplay *) data)->image;
-  else if (GIMP_IS_GIMP (data))
-    context = gimp_get_user_context (data);
-  else if (GIMP_IS_DOCK (data))
-    context = ((GimpDock *) data)->context;
-  else if (GIMP_IS_ITEM_TREE_VIEW (data))
-    return ((GimpItemTreeView *) data)->image;
+  recursion = TRUE;
+
+  if (GIMP_IS_ITEM_TREE_VIEW (data))
+    result = gimp_item_tree_view_get_image ((GimpItemTreeView *) data);
   else if (GIMP_IS_IMAGE_EDITOR (data))
-    return ((GimpImageEditor *) data)->image;
-  else if (GIMP_IS_NAVIGATION_EDITOR (data))
-    context = ((GimpNavigationEditor *) data)->context;
+    result = ((GimpImageEditor *) data)->image;
 
-  if (context)
-    return gimp_context_get_image (context);
+  if (! result)
+    {
+      GimpDisplay *display = action_data_get_display (data);
 
-  return NULL;
+      if (display)
+        result = gimp_display_get_image (display);
+    }
+
+  if (! result)
+    {
+      GimpContext *context = action_data_get_context (data);
+
+      if (context)
+        result = gimp_context_get_image (context);
+    }
+
+  recursion = FALSE;
+
+  return result;
 }
 
 GimpDisplay *
 action_data_get_display (gpointer data)
 {
-  GimpContext *context = NULL;
+  GimpDisplay     *result    = NULL;
+  static gboolean  recursion = FALSE;
 
-  if (! data)
+  if (! data || recursion)
     return NULL;
 
+  recursion = TRUE;
+
   if (GIMP_IS_DISPLAY (data))
-    return data;
-  else if (GIMP_IS_GIMP (data))
-    context = gimp_get_user_context (data);
-  else if (GIMP_IS_DOCK (data))
-    context = ((GimpDock *) data)->context;
-  else if (GIMP_IS_NAVIGATION_EDITOR (data))
-    context = ((GimpNavigationEditor *) data)->context;
+    result = data;
+  else if (GIMP_IS_IMAGE_WINDOW (data))
+    {
+      GimpDisplayShell *shell = gimp_image_window_get_active_shell (data);
+      result = shell ? shell->display : NULL;
+    }
 
-  if (context)
-    return gimp_context_get_display (context);
+  if (! result)
+    {
+      GimpContext *context = action_data_get_context (data);
 
-  return NULL;
+      if (context)
+        result = gimp_context_get_display (context);
+    }
+
+  recursion = FALSE;
+
+  return result;
+}
+
+GimpDisplayShell *
+action_data_get_shell (gpointer data)
+{
+  GimpDisplayShell *result    = NULL;
+  static gboolean   recursion = FALSE;
+
+  if (! data || recursion)
+    return NULL;
+
+  recursion = TRUE;
+
+  if (! result)
+    {
+      GimpDisplay *display = action_data_get_display (data);
+
+      if (display)
+        result = gimp_display_get_shell (display);
+    }
+
+  recursion = FALSE;
+
+  return result;
 }
 
 GtkWidget *
 action_data_get_widget (gpointer data)
 {
-  GimpDisplay *display = NULL;
+  GtkWidget       *result    = NULL;
+  static gboolean  recursion = FALSE;
 
-  if (! data)
+  if (! data || recursion)
     return NULL;
 
-  if (GIMP_IS_DISPLAY (data))
-    display = data;
-  else if (GIMP_IS_GIMP (data))
-    display = gimp_context_get_display (gimp_get_user_context (data));
-  else if (GTK_IS_WIDGET (data))
-    return data;
+  recursion = TRUE;
 
-  if (display)
-    return display->shell;
+  if (GTK_IS_WIDGET (data))
+    result = data;
 
-  return dialogs_get_toolbox ();
+  if (! result)
+    {
+      GimpDisplay *display = action_data_get_display (data);
+
+      if (display)
+        result = GTK_WIDGET (gimp_display_get_shell (display));
+    }
+
+  if (! result)
+    result = dialogs_get_toolbox ();
+
+  recursion = FALSE;
+
+  return result;
 }
 
+gint
+action_data_sel_count (gpointer data)
+{
+  if (GIMP_IS_CONTAINER_EDITOR (data))
+    {
+      GimpContainerEditor  *editor;
+
+      editor = GIMP_CONTAINER_EDITOR (data);
+      return gimp_container_view_get_selected (editor->view, NULL, NULL);
+    }
+  else
+    {
+      return 0;
+    }
+}
+
+/* action_select_value:
+ * @select_type:
+ * @value:
+ * @min:
+ * max:
+ * def:
+ * small_inc:
+ * inc:
+ * skip_inc:
+ * delta_factor:
+ * wrap:
+ *
+ * For any valid enum @value (which are all negative), the corresponding
+ * action computes the semantic value (default, first, next, etc.).
+ * For a positive @value, it is considered as a per-thousand value
+ * between the @min and @max (possibly wrapped if @wrap is set, clamped
+ * otherwise), allowing to compute the returned value.
+ *
+ * Returns: the computed value to use for the action.
+ */
 gdouble
 action_select_value (GimpActionSelectType  select_type,
                      gdouble               value,
@@ -449,7 +586,7 @@ action_select_value (GimpActionSelectType  select_type,
         value = max - (min - value);
 
       while (value > max)
-        value = min + (max - value);
+        value = min + (value - max);
     }
   else
     {
@@ -461,15 +598,18 @@ action_select_value (GimpActionSelectType  select_type,
 
 void
 action_select_property (GimpActionSelectType  select_type,
+                        GimpDisplay          *display,
                         GObject              *object,
                         const gchar          *property_name,
                         gdouble               small_inc,
                         gdouble               inc,
                         gdouble               skip_inc,
+                        gdouble               delta_factor,
                         gboolean              wrap)
 {
   GParamSpec *pspec;
 
+  g_return_if_fail (display == NULL || GIMP_IS_DISPLAY (display));
   g_return_if_fail (G_IS_OBJECT (object));
   g_return_if_fail (property_name != NULL);
 
@@ -487,9 +627,20 @@ action_select_property (GimpActionSelectType  select_type,
                                    G_PARAM_SPEC_DOUBLE (pspec)->minimum,
                                    G_PARAM_SPEC_DOUBLE (pspec)->maximum,
                                    G_PARAM_SPEC_DOUBLE (pspec)->default_value,
-                                   small_inc, inc, skip_inc, 0, wrap);
+                                   small_inc, inc, skip_inc, delta_factor, wrap);
 
       g_object_set (object, property_name, value, NULL);
+
+      if (display)
+        {
+          const gchar *blurb = g_param_spec_get_blurb (pspec);
+
+          if (blurb)
+            {
+              /*  value description and new value shown in the status bar  */
+              action_message (display, object, _("%s: %.2f"), blurb, value);
+            }
+        }
     }
   else if (G_IS_PARAM_SPEC_INT (pspec))
     {
@@ -502,9 +653,20 @@ action_select_property (GimpActionSelectType  select_type,
                                    G_PARAM_SPEC_INT (pspec)->minimum,
                                    G_PARAM_SPEC_INT (pspec)->maximum,
                                    G_PARAM_SPEC_INT (pspec)->default_value,
-                                   small_inc, inc, skip_inc, 0, wrap);
+                                   small_inc, inc, skip_inc, delta_factor, wrap);
 
       g_object_set (object, property_name, value, NULL);
+
+      if (display)
+        {
+          const gchar *blurb = g_param_spec_get_blurb (pspec);
+
+          if (blurb)
+            {
+              /*  value description and new value shown in the status bar  */
+              action_message (display, object, _("%s: %d"), blurb, value);
+            }
+        }
     }
   else
     {
@@ -523,10 +685,12 @@ action_select_object (GimpActionSelectType  select_type,
   g_return_val_if_fail (GIMP_IS_CONTAINER (container), NULL);
   g_return_val_if_fail (current == NULL || GIMP_IS_OBJECT (current), NULL);
 
-  if (! current)
+  if (! current                               &&
+      select_type != GIMP_ACTION_SELECT_FIRST &&
+      select_type != GIMP_ACTION_SELECT_LAST)
     return NULL;
 
-  n_children = gimp_container_num_children (container);
+  n_children = gimp_container_get_n_children (container);
 
   if (n_children == 0)
     return NULL;
@@ -569,3 +733,31 @@ action_select_object (GimpActionSelectType  select_type,
 
   return gimp_container_get_child_by_index (container, select_index);
 }
+
+void
+action_message (GimpDisplay *display,
+                GObject     *object,
+                const gchar *format,
+                ...)
+{
+  GimpDisplayShell *shell     = gimp_display_get_shell (display);
+  GimpStatusbar    *statusbar = gimp_display_shell_get_statusbar (shell);
+  const gchar      *icon_name = NULL;
+  va_list           args;
+
+  if (GIMP_IS_TOOL_OPTIONS (object))
+    {
+      GimpToolInfo *tool_info = GIMP_TOOL_OPTIONS (object)->tool_info;
+
+      icon_name = gimp_viewable_get_icon_name (GIMP_VIEWABLE (tool_info));
+    }
+  else if (GIMP_IS_VIEWABLE (object))
+    {
+      icon_name = gimp_viewable_get_icon_name (GIMP_VIEWABLE (object));
+    }
+
+  va_start (args, format);
+  gimp_statusbar_push_temp_valist (statusbar, GIMP_MESSAGE_INFO,
+                                   icon_name, format, args);
+  va_end (args);
+}
