@@ -91,13 +91,6 @@ void               action_message          (GimpDisplay          *display,
   if (! widget) \
     return
 
-
-#define return_if_no_drawable(image,drawable,data) \
-  return_if_no_image (image,data); \
-  drawable = gimp_image_get_active_drawable (image); \
-  if (! drawable) \
-    return
-
 #define return_if_no_drawables(image,drawables,data) \
   return_if_no_image (image,data); \
   drawables = gimp_image_get_selected_drawables (image); \
@@ -134,5 +127,11 @@ void               action_message          (GimpDisplay          *display,
   if (! vectors) \
     return
 
+#define return_if_no_vectors_list(image,list,data) \
+  return_if_no_image (image,data); \
+  list = gimp_image_get_selected_vectors (image); \
+  if (! list) \
+    return
+
 
 #endif /* __ACTIONS_H__ */
