@@ -36,6 +36,7 @@
 #include "core/gimpfilloptions.h"
 #include "core/gimplayer.h"
 #include "core/gimplayer-new.h"
+#include "core/gimplayermask.h"
 #include "core/gimpimage.h"
 #include "core/gimpimage-undo.h"
 
@@ -68,6 +69,7 @@ static gboolean   check_drawable_alpha               (GimpDrawable  *drawable,
                                                       gpointer       data);
 static void       edit_paste                         (GimpDisplay   *display,
                                                       GimpPasteType  paste_type,
+                                                      gboolean       merged,
                                                       gboolean       try_svg);
 static void       cut_named_buffer_callback          (GtkWidget     *widget,
                                                       const gchar   *name,
@@ -340,8 +342,14 @@ edit_paste_cmd_callback (GimpAction *action,
                          GVariant   *value,
                          gpointer    data)
 {
+  GimpImage     *image;
   GimpDisplay   *display    = action_data_get_display (data);
   GimpPasteType  paste_type = (GimpPasteType) g_variant_get_int32 (value);
+  GimpPasteType  converted_type;
+  GList         *drawables;
+  gboolean       merged     = FALSE;
+
+  return_if_no_image (image, data);
 
   if (paste_type == GIMP_PASTE_TYPE_FLOATING)
     {
@@ -361,12 +369,43 @@ edit_paste_cmd_callback (GimpAction *action,
     case GIMP_PASTE_TYPE_FLOATING_IN_PLACE:
     case GIMP_PASTE_TYPE_FLOATING_INTO:
     case GIMP_PASTE_TYPE_FLOATING_INTO_IN_PLACE:
-      edit_paste (display, paste_type, TRUE);
+      edit_paste (display, paste_type, merged, TRUE);
       break;
 
     case GIMP_PASTE_TYPE_NEW_LAYER:
     case GIMP_PASTE_TYPE_NEW_LAYER_IN_PLACE:
-      edit_paste (display, paste_type, FALSE);
+      edit_paste (display, paste_type, merged, FALSE);
+      break;
+
+    case GIMP_PASTE_TYPE_NEW_MERGED_LAYER_OR_FLOATING:
+    case GIMP_PASTE_TYPE_NEW_MERGED_LAYER_OR_FLOATING_IN_PLACE:
+      merged = TRUE;
+    case GIMP_PASTE_TYPE_NEW_LAYER_OR_FLOATING:
+    case GIMP_PASTE_TYPE_NEW_LAYER_OR_FLOATING_IN_PLACE:
+      drawables = gimp_image_get_selected_drawables (image);
+
+      if (drawables &&
+         (g_list_length (drawables) == 1) &&
+          GIMP_IS_LAYER_MASK (drawables->data))
+        {
+          converted_type = (paste_type == GIMP_PASTE_TYPE_NEW_LAYER_OR_FLOATING ||
+                            paste_type == GIMP_PASTE_TYPE_NEW_MERGED_LAYER_OR_FLOATING) ?
+                            GIMP_PASTE_TYPE_FLOATING :
+                            GIMP_PASTE_TYPE_FLOATING_IN_PLACE;
+
+          edit_paste (display, converted_type, merged, TRUE);
+        }
+      else
+        {
+          converted_type = (paste_type == GIMP_PASTE_TYPE_NEW_LAYER_OR_FLOATING ||
+                            paste_type == GIMP_PASTE_TYPE_NEW_MERGED_LAYER_OR_FLOATING) ?
+                            GIMP_PASTE_TYPE_NEW_LAYER :
+                            GIMP_PASTE_TYPE_NEW_LAYER_IN_PLACE;
+
+          edit_paste (display, converted_type, merged, FALSE);
+        }
+      g_list_free (drawables);
+
       break;
     }
 }
@@ -595,11 +634,17 @@ check_drawable_alpha (GimpDrawable *drawable,
 static void
 edit_paste (GimpDisplay   *display,
             GimpPasteType  paste_type,
+            gboolean       merged,
             gboolean       try_svg)
 {
   GimpImage  *image = gimp_display_get_image (display);
   GimpObject *paste;
 
+  g_return_if_fail (paste_type != GIMP_PASTE_TYPE_NEW_LAYER_OR_FLOATING          &&
+                    paste_type != GIMP_PASTE_TYPE_NEW_LAYER_OR_FLOATING_IN_PLACE &&
+                    paste_type != GIMP_PASTE_TYPE_NEW_MERGED_LAYER_OR_FLOATING   &&
+                    paste_type != GIMP_PASTE_TYPE_NEW_MERGED_LAYER_OR_FLOATING_IN_PLACE);
+
   if (try_svg)
     {
       gchar *svg;
@@ -629,33 +674,29 @@ edit_paste (GimpDisplay   *display,
     {
       GimpDisplayShell *shell     = gimp_display_get_shell (display);
       GList            *drawables = gimp_image_get_selected_drawables (image);
-      GimpDrawable     *drawable  = NULL;
       GList            *pasted_layers;
       gint              x, y;
       gint              width, height;
 
-      /* Paste on multiple selection is probably wrong right now (though
-       * I guess it can be argued). For now we just default to the same
-       * as pasting with no selected drawable, which will just create a
-       * new drawable.
-       */
-      if (g_list_length (drawables) == 1)
-        drawable = drawables->data;
-
-      g_list_free (drawables);
-
-      if (drawable                                &&
-          paste_type != GIMP_PASTE_TYPE_NEW_LAYER &&
-          paste_type != GIMP_PASTE_TYPE_NEW_LAYER_IN_PLACE)
+      if (g_list_length (drawables) != 1 ||
+          (paste_type != GIMP_PASTE_TYPE_NEW_LAYER &&
+           paste_type != GIMP_PASTE_TYPE_NEW_LAYER_IN_PLACE))
         {
-          if (gimp_viewable_get_children (GIMP_VIEWABLE (drawable)))
+          if (g_list_length (drawables) != 1)
+            {
+              gimp_message_literal (display->gimp, G_OBJECT (display),
+                                    GIMP_MESSAGE_INFO,
+                                    _("Pasted as new layer because the "
+                                      "target is not a single layer or layer mask."));
+            }
+          else if (gimp_viewable_get_children (GIMP_VIEWABLE (drawables->data)))
             {
               gimp_message_literal (display->gimp, G_OBJECT (display),
                                     GIMP_MESSAGE_INFO,
                                     _("Pasted as new layer because the "
                                       "target is a layer group."));
             }
-          else if (gimp_item_is_content_locked (GIMP_ITEM (drawable), NULL))
+          else if (gimp_item_is_content_locked (GIMP_ITEM (drawables->data), NULL))
             {
               gimp_message_literal (display->gimp, G_OBJECT (display),
                                     GIMP_MESSAGE_INFO,
@@ -671,14 +712,16 @@ edit_paste (GimpDisplay   *display,
         ! gimp_display_shell_get_infinite_canvas (shell),
         &x, &y, &width, &height);
 
-      if ((pasted_layers = gimp_edit_paste (image, drawable, paste,
-                                            paste_type, x, y, width, height)))
+      if ((pasted_layers = gimp_edit_paste (image, drawables, paste, paste_type,
+                                            gimp_get_user_context (display->gimp),
+                                            merged, x, y, width, height)))
         {
           gimp_image_set_selected_layers (image, pasted_layers);
           g_list_free (pasted_layers);
           gimp_image_flush (image);
         }
 
+      g_list_free (drawables);
       g_object_unref (paste);
     }
   else
