@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -47,6 +47,9 @@ enum
   PROP_GIMP,
 
   PROP_COLOR_PROFILE_POLICY,
+  PROP_METADATA_ROTATION_POLICY,
+
+  PROP_COLOR_PROFILE_PATH,
 
   PROP_IMAGE_CONVERT_PROFILE_INTENT,
   PROP_IMAGE_CONVERT_PROFILE_BPC,
@@ -96,6 +99,7 @@ enum
   PROP_VECTORS_IMPORT_SCALE,
 
   PROP_SELECTION_FEATHER_RADIUS,
+  PROP_SELECTION_FEATHER_EDGE_LOCK,
 
   PROP_SELECTION_GROW_RADIUS,
 
@@ -119,9 +123,7 @@ struct _GimpDialogConfigPrivate
 };
 
 #define GET_PRIVATE(config) \
-        G_TYPE_INSTANCE_GET_PRIVATE (config, \
-                                     GIMP_TYPE_DIALOG_CONFIG, \
-                                     GimpDialogConfigPrivate)
+        ((GimpDialogConfigPrivate *) gimp_dialog_config_get_instance_private ((GimpDialogConfig *) (config)))
 
 
 static void  gimp_dialog_config_constructed           (GObject      *object);
@@ -143,7 +145,8 @@ static void  gimp_dialog_config_stroke_options_notify (GObject      *object,
                                                        gpointer      data);
 
 
-G_DEFINE_TYPE (GimpDialogConfig, gimp_dialog_config, GIMP_TYPE_GUI_CONFIG)
+G_DEFINE_TYPE_WITH_PRIVATE (GimpDialogConfig, gimp_dialog_config,
+                            GIMP_TYPE_GUI_CONFIG)
 
 #define parent_class gimp_dialog_config_parent_class
 
@@ -174,9 +177,25 @@ gimp_dialog_config_class_init (GimpDialogConfigClass *klass)
                          GIMP_COLOR_PROFILE_POLICY_ASK,
                          GIMP_PARAM_STATIC_STRINGS);
 
+  GIMP_CONFIG_PROP_ENUM (object_class, PROP_METADATA_ROTATION_POLICY,
+                         "metadata-rotation-policy",
+                         "Metadata rotation policy",
+                         METADATA_ROTATION_POLICY_BLURB,
+                         GIMP_TYPE_METADATA_ROTATION_POLICY,
+                         GIMP_METADATA_ROTATION_POLICY_ASK,
+                         GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_PATH (object_class, PROP_COLOR_PROFILE_PATH,
+                         "color-profile-path",
+                         "Default color profile folder path",
+                         COLOR_PROFILE_PATH_BLURB,
+                         GIMP_CONFIG_PATH_FILE,
+                         NULL,
+                         GIMP_PARAM_STATIC_STRINGS);
+
   GIMP_CONFIG_PROP_ENUM (object_class, PROP_IMAGE_CONVERT_PROFILE_INTENT,
                          "image-convert-profile-intent",
-                         "Default rendering intent fo color profile conversion",
+                         "Default rendering intent for color profile conversion",
                          IMAGE_CONVERT_PROFILE_INTENT_BLURB,
                          GIMP_TYPE_COLOR_RENDERING_INTENT,
                          GIMP_COLOR_RENDERING_INTENT_RELATIVE_COLORIMETRIC,
@@ -302,7 +321,7 @@ gimp_dialog_config_class_init (GimpDialogConfigClass *klass)
                          "Default new layer mode",
                          LAYER_NEW_MODE_BLURB,
                          GIMP_TYPE_LAYER_MODE,
-                         GIMP_LAYER_MODE_NORMAL_LEGACY,
+                         GIMP_LAYER_MODE_NORMAL,
                          GIMP_PARAM_STATIC_STRINGS);
 
   GIMP_CONFIG_PROP_ENUM (object_class, PROP_LAYER_NEW_BLEND_SPACE,
@@ -378,7 +397,7 @@ gimp_dialog_config_class_init (GimpDialogConfigClass *klass)
 
   GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_LAYER_MERGE_ACTIVE_GROUP_ONLY,
                             "layer-merge-active-group-only",
-                            "Default layer merge active group only",
+                            "Default layer merge active groups only",
                             LAYER_MERGE_ACTIVE_GROUP_ONLY_BLURB,
                             TRUE,
                             GIMP_PARAM_STATIC_STRINGS);
@@ -422,7 +441,7 @@ gimp_dialog_config_class_init (GimpDialogConfigClass *klass)
 
   GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_VECTORS_EXPORT_ACTIVE_ONLY,
                             "path-export-active-only",
-                            "Default export only the active path",
+                            "Default export only the selected paths",
                             VECTORS_EXPORT_ACTIVE_ONLY_BLURB,
                             TRUE,
                             GIMP_PARAM_STATIC_STRINGS);
@@ -456,6 +475,13 @@ gimp_dialog_config_class_init (GimpDialogConfigClass *klass)
                            0.0, 32767.0, 5.0,
                            GIMP_PARAM_STATIC_STRINGS);
 
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_SELECTION_FEATHER_EDGE_LOCK,
+                            "selection-feather-edge-lock",
+                            "Selection feather edge lock",
+                            SELECTION_FEATHER_EDGE_LOCK_BLURB,
+                            TRUE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
   GIMP_CONFIG_PROP_DOUBLE (object_class, PROP_SELECTION_GROW_RADIUS,
                            "selection-grow-radius",
                            "Selection grow radius",
@@ -514,8 +540,6 @@ gimp_dialog_config_class_init (GimpDialogConfigClass *klass)
                            GIMP_TYPE_STROKE_OPTIONS,
                            GIMP_PARAM_STATIC_STRINGS |
                            GIMP_CONFIG_PARAM_AGGREGATE);
-
-  g_type_class_add_private (klass, sizeof (GimpDialogConfigPrivate));
 }
 
 static void
@@ -532,7 +556,7 @@ gimp_dialog_config_constructed (GObject *object)
 
   G_OBJECT_CLASS (parent_class)->constructed (object);
 
-  g_assert (GIMP_IS_GIMP (priv->gimp));
+  gimp_assert (GIMP_IS_GIMP (priv->gimp));
 
   context = gimp_get_user_context (priv->gimp);
 
@@ -558,35 +582,15 @@ gimp_dialog_config_finalize (GObject *object)
 {
   GimpDialogConfig *config = GIMP_DIALOG_CONFIG (object);
 
-  if (config->layer_new_name)
-    {
-      g_free (config->layer_new_name);
-      config->layer_new_name = NULL;
-    }
+  g_clear_pointer (&config->color_profile_path,  g_free);
+  g_clear_pointer (&config->layer_new_name,      g_free);
+  g_clear_pointer (&config->channel_new_name,    g_free);
+  g_clear_pointer (&config->vectors_new_name,    g_free);
+  g_clear_pointer (&config->vectors_export_path, g_free);
+  g_clear_pointer (&config->vectors_import_path, g_free);
 
-  if (config->channel_new_name)
-    {
-      g_free (config->channel_new_name);
-      config->channel_new_name = NULL;
-    }
-
-  if (config->vectors_new_name)
-    {
-      g_free (config->vectors_new_name);
-      config->vectors_new_name = NULL;
-    }
-
-  if (config->fill_options)
-    {
-      g_object_unref (config->fill_options);
-      config->fill_options = NULL;
-    }
-
-  if (config->stroke_options)
-    {
-      g_object_unref (config->stroke_options);
-      config->stroke_options = NULL;
-    }
+  g_clear_object (&config->fill_options);
+  g_clear_object (&config->stroke_options);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -609,6 +613,15 @@ gimp_dialog_config_set_property (GObject      *object,
     case PROP_COLOR_PROFILE_POLICY:
       config->color_profile_policy = g_value_get_enum (value);
       break;
+    case PROP_METADATA_ROTATION_POLICY:
+      config->metadata_rotation_policy = g_value_get_enum (value);
+      break;
+
+    case PROP_COLOR_PROFILE_PATH:
+      if (config->color_profile_path)
+        g_free (config->color_profile_path);
+      config->color_profile_path = g_value_dup_string (value);
+      break;
 
     case PROP_IMAGE_CONVERT_PROFILE_INTENT:
       config->image_convert_profile_intent = g_value_get_enum (value);
@@ -743,6 +756,9 @@ gimp_dialog_config_set_property (GObject      *object,
     case PROP_SELECTION_FEATHER_RADIUS:
       config->selection_feather_radius = g_value_get_double (value);
       break;
+    case PROP_SELECTION_FEATHER_EDGE_LOCK:
+      config->selection_feather_edge_lock = g_value_get_boolean (value);
+      break;
 
     case PROP_SELECTION_GROW_RADIUS:
       config->selection_grow_radius = g_value_get_double (value);
@@ -800,6 +816,13 @@ gimp_dialog_config_get_property (GObject    *object,
     case PROP_COLOR_PROFILE_POLICY:
       g_value_set_enum (value, config->color_profile_policy);
       break;
+    case PROP_METADATA_ROTATION_POLICY:
+      g_value_set_enum (value, config->metadata_rotation_policy);
+      break;
+
+    case PROP_COLOR_PROFILE_PATH:
+      g_value_set_string (value, config->color_profile_path);
+      break;
 
     case PROP_IMAGE_CONVERT_PROFILE_INTENT:
       g_value_set_enum (value, config->image_convert_profile_intent);
@@ -924,6 +947,9 @@ gimp_dialog_config_get_property (GObject    *object,
     case PROP_SELECTION_FEATHER_RADIUS:
       g_value_set_double (value, config->selection_feather_radius);
       break;
+    case PROP_SELECTION_FEATHER_EDGE_LOCK:
+      g_value_set_boolean (value, config->selection_feather_edge_lock);
+      break;
 
     case PROP_SELECTION_GROW_RADIUS:
       g_value_set_double (value, config->selection_grow_radius);
