@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -23,26 +23,25 @@
 #include <gtk/gtk.h>
 
 #include "libgimpbase/gimpbase.h"
+#include "libgimpwidgets/gimpwidgets.h"
 
 #include "display-types.h"
 
 #include "core/gimp.h"
 #include "core/gimp-edit.h"
 #include "core/gimpbuffer.h"
-#include "core/gimpcontainer.h"
-#include "core/gimpcontext.h"
-#include "core/gimpdrawable-bucket-fill.h"
+#include "core/gimpdrawable-edit.h"
+#include "core/gimpfilloptions.h"
 #include "core/gimpimage.h"
-#include "core/gimpimage-merge.h"
 #include "core/gimpimage-new.h"
 #include "core/gimpimage-undo.h"
 #include "core/gimplayer.h"
+#include "core/gimplayer-new.h"
 #include "core/gimplayermask.h"
 #include "core/gimppattern.h"
 #include "core/gimpprogress.h"
 
 #include "file/file-open.h"
-#include "file/file-utils.h"
 
 #include "text/gimptext.h"
 #include "text/gimptextlayer.h"
@@ -180,7 +179,10 @@ gimp_display_shell_dnd_position_item (GimpDisplayShell *shell,
       gint x, y;
       gint width, height;
 
-      gimp_display_shell_untransform_viewport (shell, &x, &y, &width, &height);
+      gimp_display_shell_untransform_viewport (
+        shell,
+        ! gimp_display_shell_get_infinite_canvas (shell),
+        &x, &y, &width, &height);
 
       off_x = x + (width  - item_width)  / 2;
       off_y = y + (height - item_height) / 2;
@@ -225,7 +227,8 @@ gimp_display_shell_drop_drawable (GtkWidget    *widget,
     {
       image = gimp_image_new_from_drawable (shell->display->gimp,
                                             GIMP_DRAWABLE (viewable));
-      gimp_create_display (shell->display->gimp, image, GIMP_UNIT_PIXEL, 1.0);
+      gimp_create_display (shell->display->gimp, image, GIMP_UNIT_PIXEL, 1.0,
+                           G_OBJECT (gimp_widget_get_monitor (widget)));
       g_object_unref (image);
 
       return;
@@ -248,7 +251,6 @@ gimp_display_shell_drop_drawable (GtkWidget    *widget,
       gimp_display_shell_dnd_position_item (shell, image, new_item);
 
       gimp_item_set_visible (new_item, TRUE, FALSE);
-      gimp_item_set_linked (new_item, FALSE, FALSE);
 
       gimp_image_add_layer (image, new_layer,
                             GIMP_IMAGE_ACTIVE_PARENT, -1, TRUE);
@@ -324,8 +326,8 @@ gimp_display_shell_drop_svg (GtkWidget     *widget,
                                     NULL, &error))
     {
       gimp_message_literal (shell->display->gimp, G_OBJECT (shell->display),
-			    GIMP_MESSAGE_ERROR,
-			    error->message);
+                            GIMP_MESSAGE_ERROR,
+                            error->message);
       g_clear_error (&error);
     }
   else
@@ -335,13 +337,13 @@ gimp_display_shell_drop_svg (GtkWidget     *widget,
 }
 
 static void
-gimp_display_shell_dnd_bucket_fill (GimpDisplayShell   *shell,
-                                    GimpBucketFillMode  fill_mode,
-                                    const GimpRGB      *color,
-                                    GimpPattern        *pattern)
+gimp_display_shell_dnd_fill (GimpDisplayShell *shell,
+                             GimpFillOptions  *options,
+                             const gchar      *undo_desc)
 {
   GimpImage    *image = gimp_display_get_image (shell->display);
-  GimpDrawable *drawable;
+  GList        *drawables;
+  GList        *iter;
 
   if (shell->display->gimp->busy)
     return;
@@ -349,49 +351,58 @@ gimp_display_shell_dnd_bucket_fill (GimpDisplayShell   *shell,
   if (! image)
     return;
 
-  drawable = gimp_image_get_active_drawable (image);
+  drawables = gimp_image_get_selected_drawables (image);
 
-  if (! drawable)
+  if (! drawables)
     return;
 
-  if (gimp_viewable_get_children (GIMP_VIEWABLE (drawable)))
+  for (iter = drawables; iter; iter = iter->next)
     {
-      gimp_message_literal (shell->display->gimp, G_OBJECT (shell->display),
-                            GIMP_MESSAGE_ERROR,
-                            _("Cannot modify the pixels of layer groups."));
-      return;
+      if (gimp_viewable_get_children (iter->data))
+        {
+          gimp_message_literal (shell->display->gimp, G_OBJECT (shell->display),
+                                GIMP_MESSAGE_ERROR,
+                                _("Cannot modify the pixels of layer groups."));
+          g_list_free (drawables);
+          return;
+        }
+
+      if (gimp_item_is_content_locked (iter->data, NULL))
+        {
+          gimp_message_literal (shell->display->gimp, G_OBJECT (shell->display),
+                                GIMP_MESSAGE_ERROR,
+                                _("A selected layer's pixels are locked."));
+          g_list_free (drawables);
+          return;
+        }
     }
 
-  if (gimp_item_is_content_locked (GIMP_ITEM (drawable)))
+  gimp_image_undo_group_start (image, GIMP_UNDO_GROUP_PAINT, undo_desc);
+
+  for (iter = drawables; iter; iter = iter->next)
     {
-      gimp_message_literal (shell->display->gimp, G_OBJECT (shell->display),
-                            GIMP_MESSAGE_ERROR,
-                            _("The active layer's pixels are locked."));
-      return;
-    }
+      /* FIXME: there should be a virtual method for this that the
+       *        GimpTextLayer can override.
+       */
+      if (gimp_fill_options_get_style (options) == GIMP_FILL_STYLE_SOLID &&
+          gimp_item_is_text_layer (iter->data))
+        {
+          GimpRGB color;
 
-  /* FIXME: there should be a virtual method for this that the
-   *        GimpTextLayer can override.
-   */
-  if (color && gimp_item_is_text_layer (GIMP_ITEM (drawable)))
-    {
-      gimp_text_layer_set (GIMP_TEXT_LAYER (drawable), NULL,
-                           "color", color,
-                           NULL);
-    }
-  else
-    {
-      gimp_drawable_bucket_fill_full (drawable,
-                                      fill_mode,
-                                      GIMP_NORMAL_MODE, GIMP_OPACITY_OPAQUE,
-                                      FALSE,             /* no seed fill */
-                                      FALSE,             /* don't fill transp */
-                                      GIMP_SELECT_CRITERION_COMPOSITE,
-                                      0.0, FALSE,        /* fill params  */
-                                      0.0, 0.0,          /* ignored      */
-                                      color, pattern);
+          gimp_context_get_foreground (GIMP_CONTEXT (options), &color);
+
+          gimp_text_layer_set (iter->data, NULL,
+                               "color", &color,
+                               NULL);
+        }
+      else
+        {
+          gimp_drawable_edit_fill (iter->data, options, undo_desc);
+        }
     }
 
+  g_list_free (drawables);
+  gimp_image_undo_group_end (image);
   gimp_display_shell_dnd_flush (shell, image);
 }
 
@@ -402,12 +413,19 @@ gimp_display_shell_drop_pattern (GtkWidget    *widget,
                                  GimpViewable *viewable,
                                  gpointer      data)
 {
+  GimpDisplayShell *shell   = GIMP_DISPLAY_SHELL (data);
+  GimpFillOptions  *options = gimp_fill_options_new (shell->display->gimp,
+                                                     NULL, FALSE);
+
   GIMP_LOG (DND, NULL);
 
-  if (GIMP_IS_PATTERN (viewable))
-    gimp_display_shell_dnd_bucket_fill (GIMP_DISPLAY_SHELL (data),
-                                        GIMP_PATTERN_BUCKET_FILL,
-                                        NULL, GIMP_PATTERN (viewable));
+  gimp_fill_options_set_style (options, GIMP_FILL_STYLE_PATTERN);
+  gimp_context_set_pattern (GIMP_CONTEXT (options), GIMP_PATTERN (viewable));
+
+  gimp_display_shell_dnd_fill (shell, options,
+                               C_("undo-type", "Drop pattern to layer"));
+
+  g_object_unref (options);
 }
 
 static void
@@ -417,11 +435,19 @@ gimp_display_shell_drop_color (GtkWidget     *widget,
                                const GimpRGB *color,
                                gpointer       data)
 {
+  GimpDisplayShell *shell   = GIMP_DISPLAY_SHELL (data);
+  GimpFillOptions  *options = gimp_fill_options_new (shell->display->gimp,
+                                                     NULL, FALSE);
+
   GIMP_LOG (DND, NULL);
 
-  gimp_display_shell_dnd_bucket_fill (GIMP_DISPLAY_SHELL (data),
-                                      GIMP_FG_BUCKET_FILL,
-                                      color, NULL);
+  gimp_fill_options_set_style (options, GIMP_FILL_STYLE_SOLID);
+  gimp_context_set_foreground (GIMP_CONTEXT (options), color);
+
+  gimp_display_shell_dnd_fill (shell, options,
+                               C_("undo-type", "Drop color to layer"));
+
+  g_object_unref (options);
 }
 
 static void
@@ -431,10 +457,12 @@ gimp_display_shell_drop_buffer (GtkWidget    *widget,
                                 GimpViewable *viewable,
                                 gpointer      data)
 {
-  GimpDisplayShell *shell = GIMP_DISPLAY_SHELL (data);
-  GimpImage        *image = gimp_display_get_image (shell->display);
-  GimpDrawable     *drawable;
+  GimpDisplayShell *shell    = GIMP_DISPLAY_SHELL (data);
+  GimpImage        *image    = gimp_display_get_image (shell->display);
+  GimpContext      *context;
+  GList            *drawables;
   GimpBuffer       *buffer;
+  GimpPasteType     paste_type;
   gint              x, y, width, height;
 
   GIMP_LOG (DND, NULL);
@@ -444,44 +472,32 @@ gimp_display_shell_drop_buffer (GtkWidget    *widget,
 
   if (! image)
     {
-      image = gimp_image_new_from_buffer (shell->display->gimp, NULL,
+      image = gimp_image_new_from_buffer (shell->display->gimp,
                                           GIMP_BUFFER (viewable));
-      gimp_create_display (image->gimp, image, GIMP_UNIT_PIXEL, 1.0);
+      gimp_create_display (image->gimp, image, GIMP_UNIT_PIXEL, 1.0,
+                           G_OBJECT (gimp_widget_get_monitor (widget)));
       g_object_unref (image);
 
       return;
     }
 
-  drawable = gimp_image_get_active_drawable (image);
+  paste_type = GIMP_PASTE_TYPE_NEW_LAYER_OR_FLOATING;
+  drawables  = gimp_image_get_selected_drawables (image);
+  context    = gimp_get_user_context (shell->display->gimp);
+  buffer     = GIMP_BUFFER (viewable);
 
-  if (drawable)
-    {
-      if (gimp_viewable_get_children (GIMP_VIEWABLE (drawable)))
-        {
-          gimp_message_literal (shell->display->gimp, G_OBJECT (shell->display),
-                                GIMP_MESSAGE_ERROR,
-                                _("Cannot modify the pixels of layer groups."));
-          return;
-        }
-
-      if (gimp_item_is_content_locked (GIMP_ITEM (drawable)))
-        {
-          gimp_message_literal (shell->display->gimp, G_OBJECT (shell->display),
-                                GIMP_MESSAGE_ERROR,
-                                _("The active layer's pixels are locked."));
-          return;
-        }
-    }
-
-  buffer = GIMP_BUFFER (viewable);
-
-  gimp_display_shell_untransform_viewport (shell, &x, &y, &width, &height);
+  gimp_display_shell_untransform_viewport (
+    shell,
+    ! gimp_display_shell_get_infinite_canvas (shell),
+    &x, &y, &width, &height);
 
   /* FIXME: popup a menu for selecting "Paste Into" */
 
-  gimp_edit_paste (image, drawable, buffer, FALSE,
-                   x, y, width, height);
+  g_list_free (gimp_edit_paste (image, drawables, GIMP_OBJECT (buffer),
+                                paste_type, context, FALSE,
+                                x, y, width, height));
 
+  g_list_free (drawables);
   gimp_display_shell_dnd_flush (shell, image);
 }
 
@@ -498,12 +514,11 @@ gimp_display_shell_drop_uri_list (GtkWidget *widget,
   GList            *list;
   gboolean          open_as_layers;
 
-  /* If the app is already being torn down, shell->display might be NULL here.
-   * Play it safe. */
+  /* If the app is already being torn down, shell->display might be
+   * NULL here.  Play it safe.
+   */
   if (! shell->display)
-    {
-      return;
-    }
+    return;
 
   image = gimp_display_get_image (shell->display);
   context = gimp_get_user_context (shell->display->gimp);
@@ -512,9 +527,12 @@ gimp_display_shell_drop_uri_list (GtkWidget *widget,
 
   open_as_layers = (image != NULL);
 
+  if (image)
+    g_object_ref (image);
+
   for (list = uri_list; list; list = g_list_next (list))
     {
-      const gchar       *uri   = list->data;
+      GFile             *file  = g_file_new_for_uri (list->data);
       GimpPDBStatusType  status;
       GError            *error = NULL;
       gboolean           warn  = FALSE;
@@ -522,6 +540,8 @@ gimp_display_shell_drop_uri_list (GtkWidget *widget,
       if (! shell->display)
         {
           /* It seems as if GIMP is being torn down for quitting. Bail out. */
+          g_object_unref (file);
+          g_clear_object (&image);
           return;
         }
 
@@ -532,16 +552,23 @@ gimp_display_shell_drop_uri_list (GtkWidget *widget,
           new_layers = file_open_layers (shell->display->gimp, context,
                                          GIMP_PROGRESS (shell->display),
                                          image, FALSE,
-                                         uri, GIMP_RUN_INTERACTIVE, NULL,
+                                         file, GIMP_RUN_INTERACTIVE, NULL,
                                          &status, &error);
 
           if (new_layers)
             {
-              gint x, y;
-              gint width, height;
+              gint x      = 0;
+              gint y      = 0;
+              gint width  = gimp_image_get_width  (image);
+              gint height = gimp_image_get_height (image);
 
-              gimp_display_shell_untransform_viewport (shell, &x, &y,
-                                                       &width, &height);
+              if (gimp_display_get_image (shell->display))
+                {
+                  gimp_display_shell_untransform_viewport (
+                    shell,
+                    ! gimp_display_shell_get_infinite_canvas (shell),
+                    &x, &y, &width, &height);
+                }
 
               gimp_image_add_layers (image, new_layers,
                                      GIMP_IMAGE_ACTIVE_PARENT, -1,
@@ -550,7 +577,7 @@ gimp_display_shell_drop_uri_list (GtkWidget *widget,
 
               g_list_free (new_layers);
             }
-          else if (status != GIMP_PDB_CANCEL)
+          else if (status != GIMP_PDB_CANCEL && status != GIMP_PDB_SUCCESS)
             {
               warn = TRUE;
             }
@@ -562,10 +589,11 @@ gimp_display_shell_drop_uri_list (GtkWidget *widget,
 
           new_image = file_open_with_display (shell->display->gimp, context,
                                               NULL,
-                                              uri, FALSE,
+                                              file, FALSE,
+                                              G_OBJECT (gimp_widget_get_monitor (widget)),
                                               &status, &error);
 
-          if (! new_image && status != GIMP_PDB_CANCEL)
+          if (! new_image && status != GIMP_PDB_CANCEL && status != GIMP_PDB_SUCCESS)
             warn = TRUE;
         }
       else
@@ -573,32 +601,40 @@ gimp_display_shell_drop_uri_list (GtkWidget *widget,
           /*  open the first image in the empty display  */
           image = file_open_with_display (shell->display->gimp, context,
                                           GIMP_PROGRESS (shell->display),
-                                          uri, FALSE,
+                                          file, FALSE,
+                                          G_OBJECT (gimp_widget_get_monitor (widget)),
                                           &status, &error);
 
-          if (! image && status != GIMP_PDB_CANCEL)
-            warn = TRUE;
+          if (image)
+            {
+              g_object_ref (image);
+            }
+          else if (status != GIMP_PDB_CANCEL && status != GIMP_PDB_SUCCESS)
+            {
+              warn = TRUE;
+            }
         }
 
       /* Something above might have run a few rounds of the main loop. Check
        * that shell->display is still there, otherwise ignore this as the app
-       * is being torn down for quitting. */
+       * is being torn down for quitting.
+       */
       if (warn && shell->display)
         {
-          gchar *filename = file_utils_uri_display_name (uri);
-
           gimp_message (shell->display->gimp, G_OBJECT (shell->display),
                         GIMP_MESSAGE_ERROR,
                         _("Opening '%s' failed:\n\n%s"),
-                        filename, error->message);
-
+                        gimp_file_get_utf8_name (file), error->message);
           g_clear_error (&error);
-          g_free (filename);
         }
+
+      g_object_unref (file);
     }
 
   if (image)
     gimp_display_shell_dnd_flush (shell, image);
+
+  g_clear_object (&image);
 }
 
 static void
@@ -624,7 +660,8 @@ gimp_display_shell_drop_component (GtkWidget       *widget,
     {
       dest_image = gimp_image_new_from_component (image->gimp,
                                                   image, component);
-      gimp_create_display (dest_image->gimp, dest_image, GIMP_UNIT_PIXEL, 1.0);
+      gimp_create_display (dest_image->gimp, dest_image, GIMP_UNIT_PIXEL, 1.0,
+                           G_OBJECT (gimp_widget_get_monitor (widget)));
       g_object_unref (dest_image);
 
       return;
@@ -669,7 +706,7 @@ gimp_display_shell_drop_pixbuf (GtkWidget *widget,
   GimpDisplayShell *shell = GIMP_DISPLAY_SHELL (data);
   GimpImage        *image = gimp_display_get_image (shell->display);
   GimpLayer        *new_layer;
-  GimpImageType     image_type;
+  gboolean          has_alpha = FALSE;
 
   GIMP_LOG (DND, NULL);
 
@@ -680,24 +717,25 @@ gimp_display_shell_drop_pixbuf (GtkWidget *widget,
     {
       image = gimp_image_new_from_pixbuf (shell->display->gimp, pixbuf,
                                           _("Dropped Buffer"));
-      gimp_create_display (image->gimp, image, GIMP_UNIT_PIXEL, 1.0);
+      gimp_create_display (image->gimp, image, GIMP_UNIT_PIXEL, 1.0,
+                           G_OBJECT (gimp_widget_get_monitor (widget)));
       g_object_unref (image);
 
       return;
     }
 
-  image_type = GIMP_IMAGE_TYPE_FROM_BASE_TYPE (gimp_image_base_type (image));
-
   if (gdk_pixbuf_get_n_channels (pixbuf) == 2 ||
       gdk_pixbuf_get_n_channels (pixbuf) == 4)
     {
-      image_type = GIMP_IMAGE_TYPE_WITH_ALPHA (image_type);
+      has_alpha = TRUE;
     }
 
   new_layer =
-    gimp_layer_new_from_pixbuf (pixbuf, image, image_type,
+    gimp_layer_new_from_pixbuf (pixbuf, image,
+                                gimp_image_get_layer_format (image, has_alpha),
                                 _("Dropped Buffer"),
-                                GIMP_OPACITY_OPAQUE, GIMP_NORMAL_MODE);
+                                GIMP_OPACITY_OPAQUE,
+                                gimp_image_get_default_new_layer_mode (image));
 
   if (new_layer)
     {
