@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -28,11 +28,7 @@
 #include <windows.h>
 #endif
 
-#ifdef GDK_WINDOWING_QUARTZ
-#import <AppKit/AppKit.h>
-#include <gdk/gdkquartz.h>
-#endif /* !GDK_WINDOWING_QUARTZ */
-
+#include "libgimpbase/gimpbase.h"
 #include "libgimpmath/gimpmath.h"
 #include "libgimpcolor/gimpcolor.h"
 #include "libgimpwidgets/gimpwidgets.h"
@@ -70,6 +66,8 @@
 #include "gimpdisplayshell.h"
 #include "gimpdisplayshell-appearance.h"
 #include "gimpdisplayshell-close.h"
+#include "gimpdisplayshell-expose.h"
+#include "gimpdisplayshell-render.h"
 #include "gimpdisplayshell-scale.h"
 #include "gimpdisplayshell-scroll.h"
 #include "gimpdisplayshell-tool-events.h"
@@ -78,6 +76,8 @@
 #include "gimpstatusbar.h"
 
 #include "gimp-log.h"
+#include "gimp-priorities.h"
+
 #include "gimp-intl.h"
 
 
@@ -100,7 +100,6 @@ enum
   PROP_0,
   PROP_GIMP,
   PROP_DIALOG_FACTORY,
-  PROP_INITIAL_SCREEN,
   PROP_INITIAL_MONITOR
 };
 
@@ -129,8 +128,9 @@ struct _GimpImageWindowPrivate
 
   const gchar       *entry_id;
 
-  GdkScreen         *initial_screen;
-  gint               initial_monitor;
+  GdkMonitor        *initial_monitor;
+
+  gint               scale_factor;
 
   gint               suspend_keep_pos;
 
@@ -147,9 +147,7 @@ typedef struct
 
 
 #define GIMP_IMAGE_WINDOW_GET_PRIVATE(window) \
-        G_TYPE_INSTANCE_GET_PRIVATE (window, \
-                                     GIMP_TYPE_IMAGE_WINDOW, \
-                                     GimpImageWindowPrivate)
+        ((GimpImageWindowPrivate *) gimp_image_window_get_instance_private ((GimpImageWindow *) (window)))
 
 
 /*  local function prototypes  */
@@ -172,19 +170,16 @@ static void      gimp_image_window_get_property        (GObject             *obj
                                                         GValue              *value,
                                                         GParamSpec          *pspec);
 
-static void      gimp_image_window_map                 (GtkWidget           *widget);
 static gboolean  gimp_image_window_delete_event        (GtkWidget           *widget,
                                                         GdkEventAny         *event);
 static gboolean  gimp_image_window_configure_event     (GtkWidget           *widget,
                                                         GdkEventConfigure   *event);
 static gboolean  gimp_image_window_window_state_event  (GtkWidget           *widget,
                                                         GdkEventWindowState *event);
-static void      gimp_image_window_style_set           (GtkWidget           *widget,
-                                                        GtkStyle            *prev_style);
+static void      gimp_image_window_style_updated       (GtkWidget           *widget);
 
 static void      gimp_image_window_monitor_changed     (GimpWindow          *window,
-                                                        GdkScreen           *screen,
-                                                        gint                 monitor);
+                                                        GdkMonitor          *monitor);
 
 static GList *   gimp_image_window_get_docks           (GimpDockContainer   *dock_container);
 static GimpDialogFactory *
@@ -209,13 +204,11 @@ static void      gimp_image_window_config_notify       (GimpImageWindow     *win
 static void      gimp_image_window_session_clear       (GimpImageWindow     *window);
 static void      gimp_image_window_session_apply       (GimpImageWindow     *window,
                                                         const gchar         *entry_id,
-                                                        GdkScreen           *screen,
-                                                        gint                 monitor);
+                                                        GdkMonitor          *monitor);
 static void      gimp_image_window_session_update      (GimpImageWindow     *window,
                                                         GimpDisplay         *new_display,
                                                         const gchar         *new_entry_id,
-                                                        GdkScreen           *screen,
-                                                        gint                 monitor);
+                                                        GdkMonitor          *monitor);
 static const gchar *
                  gimp_image_window_config_to_entry_id  (GimpGuiConfig       *config);
 static void      gimp_image_window_show_tooltip        (GimpUIManager       *manager,
@@ -259,9 +252,6 @@ static void      gimp_image_window_shell_rotated       (GimpDisplayShell    *she
 static void      gimp_image_window_shell_title_notify  (GimpDisplayShell    *shell,
                                                         const GParamSpec    *pspec,
                                                         GimpImageWindow     *window);
-static void      gimp_image_window_shell_icon_notify   (GimpDisplayShell    *shell,
-                                                        const GParamSpec    *pspec,
-                                                        GimpImageWindow     *window);
 static GtkWidget *
                  gimp_image_window_create_tab_label    (GimpImageWindow     *window,
                                                         GimpDisplayShell    *shell);
@@ -269,6 +259,7 @@ static void      gimp_image_window_update_tab_labels   (GimpImageWindow     *win
 
 
 G_DEFINE_TYPE_WITH_CODE (GimpImageWindow, gimp_image_window, GIMP_TYPE_WINDOW,
+                         G_ADD_PRIVATE (GimpImageWindow)
                          G_IMPLEMENT_INTERFACE (GIMP_TYPE_DOCK_CONTAINER,
                                                 gimp_image_window_dock_container_iface_init)
                          G_IMPLEMENT_INTERFACE (GIMP_TYPE_SESSION_MANAGED,
@@ -277,14 +268,6 @@ G_DEFINE_TYPE_WITH_CODE (GimpImageWindow, gimp_image_window, GIMP_TYPE_WINDOW,
 #define parent_class gimp_image_window_parent_class
 
 
-static const gchar image_window_rc_style[] =
-  "style \"fullscreen-menubar-style\"\n"
-  "{\n"
-  "  GtkMenuBar::shadow-type      = none\n"
-  "  GtkMenuBar::internal-padding = 0\n"
-  "}\n"
-  "widget \"*.gimp-menubar-fullscreen\" style \"fullscreen-menubar-style\"\n";
-
 static void
 gimp_image_window_class_init (GimpImageWindowClass *klass)
 {
@@ -298,11 +281,10 @@ gimp_image_window_class_init (GimpImageWindowClass *klass)
   object_class->set_property       = gimp_image_window_set_property;
   object_class->get_property       = gimp_image_window_get_property;
 
-  widget_class->map                = gimp_image_window_map;
   widget_class->delete_event       = gimp_image_window_delete_event;
   widget_class->configure_event    = gimp_image_window_configure_event;
   widget_class->window_state_event = gimp_image_window_window_state_event;
-  widget_class->style_set          = gimp_image_window_style_set;
+  widget_class->style_updated      = gimp_image_window_style_updated;
 
   window_class->monitor_changed    = gimp_image_window_monitor_changed;
 
@@ -320,22 +302,12 @@ gimp_image_window_class_init (GimpImageWindowClass *klass)
                                                         GIMP_PARAM_READWRITE |
                                                         G_PARAM_CONSTRUCT_ONLY));
 
-  g_object_class_install_property (object_class, PROP_INITIAL_SCREEN,
-                                   g_param_spec_object ("initial-screen",
+  g_object_class_install_property (object_class, PROP_INITIAL_MONITOR,
+                                   g_param_spec_object ("initial-monitor",
                                                         NULL, NULL,
-                                                        GDK_TYPE_SCREEN,
+                                                        GDK_TYPE_MONITOR,
                                                         GIMP_PARAM_READWRITE |
                                                         G_PARAM_CONSTRUCT_ONLY));
-  g_object_class_install_property (object_class, PROP_INITIAL_MONITOR,
-                                   g_param_spec_int ("initial-monitor",
-                                                     NULL, NULL,
-                                                     0, 16, 0,
-                                                     GIMP_PARAM_READWRITE |
-                                                     G_PARAM_CONSTRUCT_ONLY));
-
-  g_type_class_add_private (klass, sizeof (GimpImageWindowPrivate));
-
-  gtk_rc_parse_string (image_window_rc_style);
 }
 
 static void
@@ -378,15 +350,14 @@ gimp_image_window_constructed (GObject *object)
 
   G_OBJECT_CLASS (parent_class)->constructed (object);
 
-  g_assert (GIMP_IS_GIMP (private->gimp));
-  g_assert (GIMP_IS_DIALOG_FACTORY (private->dialog_factory));
+  gimp_assert (GIMP_IS_GIMP (private->gimp));
+  gimp_assert (GIMP_IS_DIALOG_FACTORY (private->dialog_factory));
 
   menu_factory = gimp_dialog_factory_get_menu_factory (private->dialog_factory);
 
   private->menubar_manager = gimp_menu_factory_manager_new (menu_factory,
                                                             "<Image>",
-                                                            window,
-                                                            FALSE);
+                                                            window);
 
   g_signal_connect_object (private->dialog_factory, "dock-window-added",
                            G_CALLBACK (gimp_image_window_update_ui_manager),
@@ -396,7 +367,7 @@ gimp_image_window_constructed (GObject *object)
                            window, G_CONNECT_SWAPPED);
 
   gtk_window_add_accel_group (GTK_WINDOW (window),
-                              gtk_ui_manager_get_accel_group (GTK_UI_MANAGER (private->menubar_manager)));
+                              gimp_ui_manager_get_accel_group (private->menubar_manager));
 
   g_signal_connect (private->menubar_manager, "show-tooltip",
                     G_CALLBACK (gimp_image_window_show_tooltip),
@@ -414,9 +385,8 @@ gimp_image_window_constructed (GObject *object)
 
   /* Create the menubar */
 #ifndef GDK_WINDOWING_QUARTZ
-  private->menubar =
-    gtk_ui_manager_get_widget (GTK_UI_MANAGER (private->menubar_manager),
-                               "/image-menubar");
+  private->menubar = gimp_ui_manager_get_widget (private->menubar_manager,
+                                                 "/image-menubar");
 #endif /* !GDK_WINDOWING_QUARTZ */
   if (private->menubar)
     {
@@ -424,7 +394,7 @@ gimp_image_window_constructed (GObject *object)
                           private->menubar, FALSE, FALSE, 0);
 
       /*  make sure we can activate accels even if the menubar is invisible
-       *  (see http://bugzilla.gnome.org/show_bug.cgi?id=137151)
+       *  (see https://bugzilla.gnome.org/show_bug.cgi?id=137151)
        */
       g_signal_connect (private->menubar, "can-activate-accel",
                         G_CALLBACK (gtk_true),
@@ -450,6 +420,7 @@ gimp_image_window_constructed (GObject *object)
 
   /* Create the left pane */
   private->left_hpane = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
+  gtk_paned_set_wide_handle (GTK_PANED (private->left_hpane), TRUE);
   gtk_box_pack_start (GTK_BOX (private->hbox), private->left_hpane,
                       TRUE, TRUE, 0);
   gtk_widget_show (private->left_hpane);
@@ -465,6 +436,7 @@ gimp_image_window_constructed (GObject *object)
 
   /* Create the right pane */
   private->right_hpane = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
+  gtk_paned_set_wide_handle (GTK_PANED (private->right_hpane), TRUE);
   gtk_paned_pack2 (GTK_PANED (private->left_hpane), private->right_hpane,
                    TRUE, FALSE);
   gtk_widget_show (private->right_hpane);
@@ -501,6 +473,9 @@ gimp_image_window_constructed (GObject *object)
   g_signal_connect_object (config, "notify::single-window-mode",
                            G_CALLBACK (gimp_image_window_config_notify),
                            window, G_CONNECT_SWAPPED);
+  g_signal_connect_object (config, "notify::show-tabs",
+                           G_CALLBACK (gimp_image_window_config_notify),
+                           window, G_CONNECT_SWAPPED);
   g_signal_connect_object (config, "notify::hide-docks",
                            G_CALLBACK (gimp_image_window_config_notify),
                            window, G_CONNECT_SWAPPED);
@@ -511,7 +486,6 @@ gimp_image_window_constructed (GObject *object)
   gimp_image_window_session_update (window,
                                     NULL /*new_display*/,
                                     gimp_image_window_config_to_entry_id (config),
-                                    private->initial_screen,
                                     private->initial_monitor);
 }
 
@@ -528,11 +502,7 @@ gimp_image_window_dispose (GObject *object)
       private->dialog_factory = NULL;
     }
 
-  if (private->menubar_manager)
-    {
-      g_object_unref (private->menubar_manager);
-      private->menubar_manager = NULL;
-    }
+  g_clear_object (&private->menubar_manager);
 
   if (private->update_ui_manager_idle_id)
     {
@@ -574,11 +544,8 @@ gimp_image_window_set_property (GObject      *object,
     case PROP_DIALOG_FACTORY:
       private->dialog_factory = g_value_get_object (value);
       break;
-    case PROP_INITIAL_SCREEN:
-      private->initial_screen = g_value_get_object (value);
-      break;
     case PROP_INITIAL_MONITOR:
-      private->initial_monitor = g_value_get_int (value);
+      private->initial_monitor = g_value_get_object (value);
       break;
 
     default:
@@ -604,11 +571,8 @@ gimp_image_window_get_property (GObject    *object,
     case PROP_DIALOG_FACTORY:
       g_value_set_object (value, private->dialog_factory);
       break;
-    case PROP_INITIAL_SCREEN:
-      g_value_set_object (value, private->initial_screen);
-      break;
     case PROP_INITIAL_MONITOR:
-      g_value_set_int (value, private->initial_monitor);
+      g_value_set_object (value, private->initial_monitor);
       break;
 
     default:
@@ -617,34 +581,6 @@ gimp_image_window_get_property (GObject    *object,
     }
 }
 
-static void
-gimp_image_window_map (GtkWidget *widget)
-{
-#ifdef GDK_WINDOWING_QUARTZ
-  GdkWindow *gdk_window;
-  NSWindow  *ns_window;
-#endif /* !GDK_WINDOWING_QUARTZ */
-
-  GTK_WIDGET_CLASS (parent_class)->map (widget);
-
-#ifdef GDK_WINDOWING_QUARTZ
-  gdk_window = gtk_widget_get_window (GTK_WIDGET (widget));
-  ns_window = gdk_quartz_window_get_nswindow (gdk_window);
-
-  /* Disable the new-style full screen mode. For now only the "old-style"
-   * full screen mode, via the "View" menu, is supported. In the future, and
-   * as soon as GTK+ has proper support for this, we will migrate to the
-   * new-style full screen mode.
-   */
-#if MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_7
-  ns_window.collectionBehavior |= NSWindowCollectionBehaviorFullScreenAuxiliary;
-#else
-  /* Hard code the define ... */
-  ns_window.collectionBehavior |= 1 << 8;
-#endif
-#endif /* !GDK_WINDOWING_QUARTZ */
-}
-
 static gboolean
 gimp_image_window_delete_event (GtkWidget   *widget,
                                 GdkEventAny *event)
@@ -667,10 +603,12 @@ static gboolean
 gimp_image_window_configure_event (GtkWidget         *widget,
                                    GdkEventConfigure *event)
 {
-  GimpImageWindow *window = GIMP_IMAGE_WINDOW (widget);
-  GtkAllocation    allocation;
-  gint             current_width;
-  gint             current_height;
+  GimpImageWindow        *window  = GIMP_IMAGE_WINDOW (widget);
+  GimpImageWindowPrivate *private = GIMP_IMAGE_WINDOW_GET_PRIVATE (window);
+  GtkAllocation           allocation;
+  gint                    current_width;
+  gint                    current_height;
+  gint                    scale_factor;
 
   gtk_widget_get_allocation (widget, &allocation);
 
@@ -695,9 +633,36 @@ gimp_image_window_configure_event (GtkWidget         *widget,
         shell->size_allocate_from_configure_event = TRUE;
     }
 
+  scale_factor = gdk_window_get_scale_factor (gtk_widget_get_window (widget));
+
+  if (scale_factor != private->scale_factor)
+    {
+      GList *list;
+
+      private->scale_factor = scale_factor;
+
+      for (list = private->shells; list; list = g_list_next (list))
+        {
+          GimpDisplayShell *shell = list->data;
+
+          gimp_display_shell_render_set_scale (shell, scale_factor);
+          gimp_display_shell_expose_full (shell);
+        }
+    }
+
   return TRUE;
 }
 
+static void
+gimp_image_window_update_csd_on_fullscreen (GtkWidget *child,
+                                            gpointer   user_data)
+{
+  gboolean fullscreen = GPOINTER_TO_INT (user_data);
+
+  if (GTK_IS_HEADER_BAR (child))
+    gtk_widget_set_visible (child, !fullscreen);
+}
+
 static gboolean
 gimp_image_window_window_state_event (GtkWidget           *widget,
                                       GdkEventWindowState *event)
@@ -706,6 +671,10 @@ gimp_image_window_window_state_event (GtkWidget           *widget,
   GimpImageWindowPrivate *private = GIMP_IMAGE_WINDOW_GET_PRIVATE (window);
   GimpDisplayShell       *shell   = gimp_image_window_get_active_shell (window);
 
+  /* Run the parent implementation */
+  if (GTK_WIDGET_CLASS (parent_class)->window_state_event)
+    GTK_WIDGET_CLASS (parent_class)->window_state_event (widget, event);
+
   if (! shell)
     return FALSE;
 
@@ -720,13 +689,16 @@ gimp_image_window_window_state_event (GtkWidget           *widget,
                 widget,
                 fullscreen ? "TRUE" : "FALSE");
 
-      if (private->menubar)
-        gtk_widget_set_name (private->menubar,
-                             fullscreen ? "gimp-menubar-fullscreen" : NULL);
-
       gimp_image_window_suspend_keep_pos (window);
       gimp_display_shell_appearance_update (shell);
       gimp_image_window_resume_keep_pos (window);
+
+      /* When using CSD (for example in Wayland), our title bar stays visible
+       * when going fullscreen by default. There is no getter for it and it's
+       * an internal child, so we use this workaround instead */
+      gtk_container_forall (GTK_CONTAINER (window),
+                            gimp_image_window_update_csd_on_fullscreen,
+                            GINT_TO_POINTER (fullscreen));
     }
 
   if (event->changed_mask & GDK_WINDOW_STATE_ICONIFIED)
@@ -766,8 +738,7 @@ gimp_image_window_window_state_event (GtkWidget           *widget,
 }
 
 static void
-gimp_image_window_style_set (GtkWidget *widget,
-                             GtkStyle  *prev_style)
+gimp_image_window_style_updated (GtkWidget *widget)
 {
   GimpImageWindow        *window        = GIMP_IMAGE_WINDOW (widget);
   GimpImageWindowPrivate *private       = GIMP_IMAGE_WINDOW_GET_PRIVATE (window);
@@ -777,14 +748,14 @@ gimp_image_window_style_set (GtkWidget *widget,
   GdkGeometry             geometry      = { 0, };
   GdkWindowHints          geometry_mask = 0;
 
-  GTK_WIDGET_CLASS (parent_class)->style_set (widget, prev_style);
+  GTK_WIDGET_CLASS (parent_class)->style_updated (widget);
 
   if (! shell)
     return;
 
   statusbar = gimp_display_shell_get_statusbar (shell);
 
-  gtk_widget_size_request (GTK_WIDGET (statusbar), &requisition);
+  gtk_widget_get_preferred_size (GTK_WIDGET (statusbar), &requisition, NULL);
 
   geometry.min_height = 23;
 
@@ -793,7 +764,7 @@ gimp_image_window_style_set (GtkWidget *widget,
 
   if (private->menubar)
     {
-      gtk_widget_size_request (private->menubar, &requisition);
+      gtk_widget_get_preferred_size (private->menubar, &requisition, NULL);
 
       geometry.min_height += requisition.height;
     }
@@ -802,7 +773,7 @@ gimp_image_window_style_set (GtkWidget *widget,
 
   /*  Only set user pos on the empty display because it gets a pos
    *  set by gimp. All other displays should be placed by the window
-   *  manager. See http://bugzilla.gnome.org/show_bug.cgi?id=559580
+   *  manager. See https://bugzilla.gnome.org/show_bug.cgi?id=559580
    */
   if (! gimp_display_get_image (shell->display))
     geometry_mask |= GDK_HINT_USER_POS;
@@ -815,8 +786,7 @@ gimp_image_window_style_set (GtkWidget *widget,
 
 static void
 gimp_image_window_monitor_changed (GimpWindow *window,
-                                   GdkScreen  *screen,
-                                   gint        monitor)
+                                   GdkMonitor *monitor)
 {
   GimpImageWindowPrivate *private = GIMP_IMAGE_WINDOW_GET_PRIVATE (window);
   GList                  *list;
@@ -1130,8 +1100,7 @@ GimpImageWindow *
 gimp_image_window_new (Gimp              *gimp,
                        GimpImage         *image,
                        GimpDialogFactory *dialog_factory,
-                       GdkScreen         *screen,
-                       gint               monitor)
+                       GdkMonitor        *monitor)
 {
   GimpImageWindow        *window;
   GimpImageWindowPrivate *private;
@@ -1139,13 +1108,13 @@ gimp_image_window_new (Gimp              *gimp,
   g_return_val_if_fail (GIMP_IS_GIMP (gimp), NULL);
   g_return_val_if_fail (image == NULL || GIMP_IS_IMAGE (image), NULL);
   g_return_val_if_fail (GIMP_IS_DIALOG_FACTORY (dialog_factory), NULL);
-  g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
+  g_return_val_if_fail (GDK_IS_MONITOR (monitor), NULL);
 
   window = g_object_new (GIMP_TYPE_IMAGE_WINDOW,
                          "gimp",            gimp,
                          "dialog-factory",  dialog_factory,
-                         "initial-screen",  screen,
                          "initial-monitor", monitor,
+                         "application",     g_application_get_default (),
                          /* The window position will be overridden by the
                           * dialog factory, it is only really used on first
                           * startup.
@@ -1160,32 +1129,23 @@ gimp_image_window_new (Gimp              *gimp,
 
   if (! GIMP_GUI_CONFIG (private->gimp->config)->single_window_mode)
     {
-      GdkScreen *pointer_screen;
-      gint       pointer_monitor;
-
-      pointer_monitor = gimp_get_monitor_at_pointer (&pointer_screen);
+      GdkMonitor *pointer_monitor = gimp_get_monitor_at_pointer ();
 
       /*  If we are supposed to go to a monitor other than where the
        *  pointer is, place the window on that monitor manually,
        *  otherwise simply let the window manager place the window on
        *  the poiner's monitor.
        */
-      if (pointer_screen  != screen ||
-          pointer_monitor != monitor)
+      if (pointer_monitor != monitor)
         {
           GdkRectangle rect;
-          gchar        geom[32];
 
-          gdk_screen_get_monitor_workarea (screen, monitor, &rect);
+          gdk_monitor_get_workarea (monitor, &rect);
 
-          /*  FIXME: image window placement
-           *
-           *  This is ugly beyond description but better than showing
-           *  the window on the wrong monitor
-           */
-          g_snprintf (geom, sizeof (geom), "%+d%+d",
-                      rect.x + 300, rect.y + 30);
-          gtk_window_parse_geometry (GTK_WINDOW (window), geom);
+          gtk_window_move (GTK_WINDOW (window),
+                           rect.x + 300, rect.y + 30);
+          gtk_window_set_geometry_hints (GTK_WINDOW (window),
+                                         NULL, NULL, GDK_HINT_USER_POS);
         }
     }
 
@@ -1468,15 +1428,12 @@ gimp_image_window_shrink_wrap (GimpImageWindow *window,
                                gboolean         grow_only)
 {
   GimpDisplayShell *active_shell;
-  GimpImage        *image;
   GtkWidget        *widget;
   GtkAllocation     allocation;
-  GdkScreen        *screen;
+  GdkMonitor       *monitor;
   GdkRectangle      rect;
-  gint              monitor;
   gint              disp_width, disp_height;
-  gdouble           x, y;
-  gdouble           width, height;
+  gint              width, height;
   gint              max_auto_width, max_auto_height;
   gint              border_width, border_height;
   gboolean          resize = FALSE;
@@ -1493,26 +1450,24 @@ gimp_image_window_shrink_wrap (GimpImageWindow *window,
   if (!active_shell)
     return;
 
-  image = gimp_display_get_image (active_shell->display);
-
-  widget = GTK_WIDGET (window);
-  screen = gtk_widget_get_screen (widget);
+  widget  = GTK_WIDGET (window);
+  monitor = gimp_widget_get_monitor (widget);
 
   gtk_widget_get_allocation (widget, &allocation);
 
-  monitor = gdk_screen_get_monitor_at_window (screen,
-                                              gtk_widget_get_window (widget));
-  gdk_screen_get_monitor_workarea (screen, monitor, &rect);
+  gdk_monitor_get_workarea (monitor, &rect);
 
-  gimp_display_shell_transform_bounds (active_shell,
-                                       0, 0,
-                                       gimp_image_get_width (image),
-                                       gimp_image_get_height (image),
-                                       &x, &y,
-                                       &width, &height);
-
-  width  = ceil (width  - x);
-  height = ceil (height - y);
+  if (! gimp_display_shell_get_infinite_canvas (active_shell))
+    {
+      gimp_display_shell_scale_get_image_size (active_shell,
+                                               &width, &height);
+    }
+  else
+    {
+      gimp_display_shell_scale_get_image_bounding_box (active_shell,
+                                                       NULL,   NULL,
+                                                       &width, &height);
+    }
 
   disp_width  = active_shell->disp_width;
   disp_height = active_shell->disp_height;
@@ -1615,7 +1570,7 @@ gimp_image_window_shrink_wrap (GimpImageWindow *window,
    * GimpDisplayShell::configure_event().
    */
   /* FIXME multiple shells */
-  gimp_display_shell_scroll_center_image (active_shell, TRUE, TRUE);
+  gimp_display_shell_scroll_center_content (active_shell, TRUE, TRUE);
 }
 
 static GtkWidget *
@@ -1644,8 +1599,8 @@ gimp_image_window_get_first_dockbook (GimpDockColumns *columns)
  * Gets the default dockbook, which is the dockbook in which new
  * dockables should be put in single-window mode.
  *
- * Returns: The default dockbook for new dockables, or NULL if no
- *          dockbook were avaialble.
+ * Returns: (nullable): The default dockbook for new dockables, or %NULL if no
+ *          dockbook were available.
  **/
 GtkWidget *
 gimp_image_window_get_default_dockbook (GimpImageWindow  *window)
@@ -1721,6 +1676,18 @@ gimp_image_window_keep_canvas_pos (GimpImageWindow *window)
                              G_CALLBACK (gimp_image_window_shell_size_allocate),
                              data, (GClosureNotify) g_free,
                              G_CONNECT_AFTER);
+      /*
+       * XXX Asking a resize should not be necessary and should happen
+       * automatically with the translate coordinates. And it does, but
+       * sometimes only after some further event (like clicking on shell
+       * widgets). Until such event happens, the shell look broken, for
+       * instance with rulers and canvas overlapping and misaligned
+       * ruler values.
+       * So I queue a resize explicitly to force the correct shell
+       * visualization immediately.
+       * See also #5813.
+       */
+      gtk_widget_queue_resize (GTK_WIDGET (shell));
     }
 }
 
@@ -1777,6 +1744,7 @@ gimp_image_window_update_tabs (GimpImageWindow *window)
   /* Tab visibility. */
   gtk_notebook_set_show_tabs (GTK_NOTEBOOK (private->notebook),
                               config->single_window_mode &&
+                              config->show_tabs          &&
                               ! config->hide_docks       &&
                               ((private->active_shell          &&
                                 private->active_shell->display &&
@@ -1837,6 +1805,7 @@ gimp_image_window_config_notify (GimpImageWindow *window,
   /* Dock column visibility */
   if (strcmp (pspec->name, "single-window-mode") == 0 ||
       strcmp (pspec->name, "hide-docks")         == 0 ||
+      strcmp (pspec->name, "show-tabs")          == 0 ||
       strcmp (pspec->name, "tabs-position")      == 0)
     {
       if (strcmp (pspec->name, "single-window-mode") == 0 ||
@@ -1883,7 +1852,6 @@ gimp_image_window_config_notify (GimpImageWindow *window,
       gimp_image_window_session_update (window,
                                         NULL /*new_display*/,
                                         gimp_image_window_config_to_entry_id (config),
-                                        gtk_widget_get_screen (GTK_WIDGET (window)),
                                         gimp_widget_get_monitor (GTK_WIDGET (window)));
     }
 }
@@ -1908,7 +1876,7 @@ gimp_image_window_update_ui_manager_idle (GimpImageWindow *window)
 {
   GimpImageWindowPrivate *private = GIMP_IMAGE_WINDOW_GET_PRIVATE (window);
 
-  g_assert (private->active_shell != NULL);
+  gimp_assert (private->active_shell != NULL);
 
   gimp_ui_manager_update (private->menubar_manager,
                           private->active_shell->display);
@@ -1926,8 +1894,10 @@ gimp_image_window_update_ui_manager (GimpImageWindow *window)
   if (! private->update_ui_manager_idle_id)
     {
       private->update_ui_manager_idle_id =
-        g_idle_add ((GSourceFunc) gimp_image_window_update_ui_manager_idle,
-                    window);
+        g_idle_add_full (GIMP_PRIORITY_IMAGE_WINDOW_UPDATE_UI_MANAGER_IDLE,
+                         (GSourceFunc) gimp_image_window_update_ui_manager_idle,
+                         window,
+                         NULL);
     }
 }
 
@@ -2009,12 +1979,8 @@ gimp_image_window_switch_page (GtkNotebook     *notebook,
   g_signal_connect (private->active_shell, "notify::title",
                     G_CALLBACK (gimp_image_window_shell_title_notify),
                     window);
-  g_signal_connect (private->active_shell, "notify::icon",
-                    G_CALLBACK (gimp_image_window_shell_icon_notify),
-                    window);
 
   gtk_window_set_title (GTK_WINDOW (window), shell->title);
-  gtk_window_set_icon (GTK_WINDOW (window), shell->icon);
 
   gimp_display_shell_appearance_update (private->active_shell);
 
@@ -2025,7 +1991,6 @@ gimp_image_window_switch_page (GtkNotebook     *notebook,
       gimp_image_window_session_update (window,
                                         active_display,
                                         NULL /*new_entry_id*/,
-                                        gtk_widget_get_screen (GTK_WIDGET (window)),
                                         gimp_widget_get_monitor (GTK_WIDGET (window)));
     }
   else
@@ -2037,7 +2002,6 @@ gimp_image_window_switch_page (GtkNotebook     *notebook,
       gimp_image_window_session_update (window,
                                         active_display,
                                         NULL /*new_entry_id*/,
-                                        private->initial_screen,
                                         private->initial_monitor);
     }
 
@@ -2117,9 +2081,6 @@ gimp_image_window_disconnect_from_active_shell (GimpImageWindow *window)
   g_signal_handlers_disconnect_by_func (private->active_shell,
                                         gimp_image_window_shell_title_notify,
                                         window);
-  g_signal_handlers_disconnect_by_func (private->active_shell,
-                                        gimp_image_window_shell_icon_notify,
-                                        window);
 
   if (private->menubar_manager)
     gimp_image_window_hide_tooltip (private->menubar_manager, window);
@@ -2144,7 +2105,6 @@ gimp_image_window_image_notify (GimpDisplay      *display,
   gimp_image_window_session_update (window,
                                     display,
                                     NULL /*new_entry_id*/,
-                                    gtk_widget_get_screen (GTK_WIDGET (window)),
                                     gimp_widget_get_monitor (GTK_WIDGET (window)));
 
   tab_label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (private->notebook),
@@ -2173,8 +2133,7 @@ gimp_image_window_session_clear (GimpImageWindow *window)
 static void
 gimp_image_window_session_apply (GimpImageWindow *window,
                                  const gchar     *entry_id,
-                                 GdkScreen       *screen,
-                                 gint             monitor)
+                                 GdkMonitor      *monitor)
 {
   GimpImageWindowPrivate *private      = GIMP_IMAGE_WINDOW_GET_PRIVATE (window);
   GimpSessionInfo        *session_info = NULL;
@@ -2208,7 +2167,6 @@ gimp_image_window_session_apply (GimpImageWindow *window,
   gimp_dialog_factory_add_foreign (private->dialog_factory,
                                    entry_id,
                                    GTK_WIDGET (window),
-                                   screen,
                                    monitor);
 
   gtk_window_unmaximize (GTK_WINDOW (window));
@@ -2219,8 +2177,7 @@ static void
 gimp_image_window_session_update (GimpImageWindow *window,
                                   GimpDisplay     *new_display,
                                   const gchar     *new_entry_id,
-                                  GdkScreen       *screen,
-                                  gint             monitor)
+                                  GdkMonitor      *monitor)
 {
   GimpImageWindowPrivate *private = GIMP_IMAGE_WINDOW_GET_PRIVATE (window);
 
@@ -2236,8 +2193,7 @@ gimp_image_window_session_update (GimpImageWindow *window,
            */
           if (strcmp (new_entry_id, GIMP_SINGLE_IMAGE_WINDOW_ENTRY_ID) == 0)
             {
-              gimp_image_window_session_apply (window, new_entry_id,
-                                               screen, monitor);
+              gimp_image_window_session_apply (window, new_entry_id, monitor);
             }
         }
       else if (strcmp (private->entry_id, new_entry_id) != 0)
@@ -2258,7 +2214,7 @@ gimp_image_window_session_update (GimpImageWindow *window,
                   g_list_length (private->shells) <= 1)
                 {
                   gimp_image_window_session_apply (window, new_entry_id,
-                                                   screen, monitor);
+                                                   monitor);
                 }
             }
           else if (strcmp (new_entry_id, GIMP_SINGLE_IMAGE_WINDOW_ENTRY_ID) == 0)
@@ -2268,7 +2224,7 @@ gimp_image_window_session_update (GimpImageWindow *window,
                * is exited
                */
               gimp_image_window_session_apply (window, new_entry_id,
-                                               screen, monitor);
+                                               monitor);
             }
         }
 
@@ -2296,7 +2252,7 @@ gimp_image_window_session_update (GimpImageWindow *window,
            * contain images) we should become the empty image window
            */
           gimp_image_window_session_apply (window, private->entry_id,
-                                           screen, monitor);
+                                           monitor);
         }
     }
 }
@@ -2333,14 +2289,6 @@ gimp_image_window_shell_title_notify (GimpDisplayShell *shell,
   gtk_window_set_title (GTK_WINDOW (window), shell->title);
 }
 
-static void
-gimp_image_window_shell_icon_notify (GimpDisplayShell *shell,
-                                     const GParamSpec *pspec,
-                                     GimpImageWindow  *window)
-{
-  gtk_window_set_icon (GTK_WINDOW (window), shell->icon);
-}
-
 static void
 gimp_image_window_shell_close_button_callback (GimpDisplayShell *shell)
 {
