@@ -12,26 +12,16 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
-#include <errno.h>
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#include <glib/gstdio.h>
+#include <gegl.h>
 #include <gtk/gtk.h>
 
 #include "libgimpbase/gimpbase.h"
 
-#ifdef G_OS_WIN32
-#include "libgimpbase/gimpwin32-io.h"
-#endif
-
 #include "menus-types.h"
 
 #include "config/gimpconfig-file.h"
@@ -40,6 +30,7 @@
 #include "core/gimp.h"
 
 #include "widgets/gimpactionfactory.h"
+#include "widgets/gimpdashboard.h"
 #include "widgets/gimpmenufactory.h"
 
 #include "dockable-menu.h"
@@ -93,7 +84,6 @@ menus_init (Gimp              *gimp,
 
   gimp_menu_factory_manager_register (global_menu_factory, "<Image>",
                                       "file",
-                                      "config",
                                       "context",
                                       "debug",
                                       "help",
@@ -109,6 +99,7 @@ menus_init (Gimp              *gimp,
                                       "dialogs",
                                       "windows",
                                       "plug-in",
+                                      "filters",
                                       "quick-mask",
                                       NULL,
                                       "/image-menubar",
@@ -121,11 +112,11 @@ menus_init (Gimp              *gimp,
 
   gimp_menu_factory_manager_register (global_menu_factory, "<Toolbox>",
                                       "file",
-                                      "config",
                                       "context",
                                       "help",
                                       "edit",
                                       "select",
+                                      "view",
                                       "image",
                                       "drawable",
                                       "layers",
@@ -135,16 +126,17 @@ menus_init (Gimp              *gimp,
                                       "windows",
                                       "dialogs",
                                       "plug-in",
+                                      "filters",
                                       "quick-mask",
                                       NULL,
                                       NULL);
 
   gimp_menu_factory_manager_register (global_menu_factory, "<Dock>",
                                       "file",
-                                      "config",
                                       "context",
                                       "edit",
                                       "select",
+                                      "view",
                                       "image",
                                       "drawable",
                                       "layers",
@@ -162,6 +154,7 @@ menus_init (Gimp              *gimp,
   gimp_menu_factory_manager_register (global_menu_factory, "<Layers>",
                                       "layers",
                                       "plug-in",
+                                      "filters",
                                       NULL,
                                       "/layers-popup",
                                       "layers-menu.xml", plug_in_menus_setup,
@@ -170,6 +163,7 @@ menus_init (Gimp              *gimp,
   gimp_menu_factory_manager_register (global_menu_factory, "<Channels>",
                                       "channels",
                                       "plug-in",
+                                      "filters",
                                       NULL,
                                       "/channels-popup",
                                       "channels-menu.xml", plug_in_menus_setup,
@@ -183,6 +177,14 @@ menus_init (Gimp              *gimp,
                                       "vectors-menu.xml", plug_in_menus_setup,
                                       NULL);
 
+  gimp_menu_factory_manager_register (global_menu_factory, "<VectorToolPath>",
+                                      "vector-toolpath",
+                                      NULL,
+                                      "/vector-toolpath-popup",
+                                      "vector-toolpath-menu.xml",
+                                      NULL,
+                                      NULL);
+
   gimp_menu_factory_manager_register (global_menu_factory, "<Colormap>",
                                       "colormap",
                                       "plug-in",
@@ -215,6 +217,14 @@ menus_init (Gimp              *gimp,
                                       "dynamics-menu.xml", plug_in_menus_setup,
                                       NULL);
 
+  gimp_menu_factory_manager_register (global_menu_factory, "<MyPaintBrushes>",
+                                      "mypaint-brushes",
+                                      "plug-in",
+                                      NULL,
+                                      "/mypaint-brushes-popup",
+                                      "mypaint-brushes-menu.xml", plug_in_menus_setup,
+                                      NULL);
+
   gimp_menu_factory_manager_register (global_menu_factory, "<Patterns>",
                                       "patterns",
                                       "plug-in",
@@ -386,6 +396,13 @@ menus_init (Gimp              *gimp,
                                       "sample-points-menu.xml",
                                       NULL,
                                       NULL);
+
+  gimp_menu_factory_manager_register (global_menu_factory, "<Dashboard>",
+                                      "dashboard",
+                                      NULL,
+                                      "/dashboard-popup",
+                                      "dashboard-menu.xml", gimp_dashboard_menu_setup,
+                                       NULL);
 }
 
 void
@@ -405,23 +422,28 @@ menus_exit (Gimp *gimp)
 void
 menus_restore (Gimp *gimp)
 {
+  GFile *file;
   gchar *filename;
 
   g_return_if_fail (GIMP_IS_GIMP (gimp));
 
-  filename = gimp_personal_rc_file ("menurc");
+  file = gimp_directory_file ("menurc", NULL);
 
   if (gimp->be_verbose)
-    g_print ("Parsing '%s'\n", gimp_filename_to_utf8 (filename));
+    g_print ("Parsing '%s'\n", gimp_file_get_utf8_name (file));
 
+  filename = g_file_get_path (file);
   gtk_accel_map_load (filename);
   g_free (filename);
+
+  g_object_unref (file);
 }
 
 void
 menus_save (Gimp     *gimp,
             gboolean  always_save)
 {
+  GFile *file;
   gchar *filename;
 
   g_return_if_fail (GIMP_IS_GIMP (gimp));
@@ -429,14 +451,17 @@ menus_save (Gimp     *gimp,
   if (menurc_deleted && ! always_save)
     return;
 
-  filename = gimp_personal_rc_file ("menurc");
+  file = gimp_directory_file ("menurc", NULL);
 
   if (gimp->be_verbose)
-    g_print ("Writing '%s'\n", gimp_filename_to_utf8 (filename));
+    g_print ("Writing '%s'\n", gimp_file_get_utf8_name (file));
 
+  filename = g_file_get_path (file);
   gtk_accel_map_save (filename);
   g_free (filename);
 
+  g_object_unref (file);
+
   menurc_deleted = FALSE;
 }
 
@@ -444,25 +469,28 @@ gboolean
 menus_clear (Gimp    *gimp,
              GError **error)
 {
-  gchar    *filename;
-  gchar    *source;
-  gboolean  success = TRUE;
+  GFile    *file;
+  GFile    *source;
+  gboolean  success  = TRUE;
+  GError   *my_error = NULL;
 
   g_return_val_if_fail (GIMP_IS_GIMP (gimp), FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  filename = gimp_personal_rc_file ("menurc");
-  source = g_build_filename (gimp_sysconf_directory (), "menurc", NULL);
+  file   = gimp_directory_file ("menurc", NULL);
+  source = gimp_sysconf_directory_file ("menurc", NULL);
 
-  if (gimp_config_file_copy (source, filename, NULL))
+  if (g_file_copy (source, file, G_FILE_COPY_OVERWRITE,
+                   NULL, NULL, NULL, NULL))
     {
       menurc_deleted = TRUE;
     }
-  else if (g_unlink (filename) != 0 && errno != ENOENT)
+  else if (! g_file_delete (file, NULL, &my_error) &&
+           my_error->code != G_IO_ERROR_NOT_FOUND)
     {
-      g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno),
-		   _("Deleting \"%s\" failed: %s"),
-                   gimp_filename_to_utf8 (filename), g_strerror (errno));
+      g_set_error (error, my_error->domain, my_error->code,
+                   _("Deleting \"%s\" failed: %s"),
+                   gimp_file_get_utf8_name (file), my_error->message);
       success = FALSE;
     }
   else
@@ -470,8 +498,9 @@ menus_clear (Gimp    *gimp,
       menurc_deleted = TRUE;
     }
 
-  g_free (source);
-  g_free (filename);
+  g_clear_error (&my_error);
+  g_object_unref (source);
+  g_object_unref (file);
 
   return success;
 }
