@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -20,6 +20,7 @@
 #include <stdarg.h>
 #include <sys/types.h>
 
+#include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gegl.h>
 
 #include "libgimpbase/gimpbase.h"
@@ -27,8 +28,9 @@
 #include "pdb-types.h"
 
 #include "core/gimp.h"
-#include "core/gimp-utils.h"
+#include "core/gimp-memsize.h"
 #include "core/gimpchannel.h"
+#include "core/gimpdisplay.h"
 #include "core/gimplayer.h"
 #include "core/gimpparamspecs.h"
 #include "core/gimpprogress.h"
@@ -42,34 +44,43 @@
 #include "gimp-intl.h"
 
 
-static void          gimp_procedure_finalize      (GObject        *object);
+static void          gimp_procedure_finalize            (GObject         *object);
 
-static gint64        gimp_procedure_get_memsize   (GimpObject     *object,
-                                                   gint64         *gui_size);
+static gint64        gimp_procedure_get_memsize         (GimpObject      *object,
+                                                         gint64          *gui_size);
 
-static GValueArray * gimp_procedure_real_execute  (GimpProcedure  *procedure,
-                                                   Gimp           *gimp,
-                                                   GimpContext    *context,
-                                                   GimpProgress   *progress,
-                                                   GValueArray    *args,
-                                                   GError        **error);
-static void    gimp_procedure_real_execute_async  (GimpProcedure  *procedure,
-                                                   Gimp           *gimp,
-                                                   GimpContext    *context,
-                                                   GimpProgress   *progress,
-                                                   GValueArray    *args,
-                                                   GimpObject     *display);
+static const gchar * gimp_procedure_real_get_label      (GimpProcedure   *procedure);
+static const gchar * gimp_procedure_real_get_menu_label (GimpProcedure   *procedure);
+static const gchar * gimp_procedure_real_get_blurb      (GimpProcedure   *procedure);
+static const gchar * gimp_procedure_real_get_help_id    (GimpProcedure   *procedure);
+static gboolean      gimp_procedure_real_get_sensitive  (GimpProcedure   *procedure,
+                                                         GimpObject      *object,
+                                                         const gchar    **tooltip);
+static GimpValueArray * gimp_procedure_real_execute     (GimpProcedure   *procedure,
+                                                         Gimp            *gimp,
+                                                         GimpContext     *context,
+                                                         GimpProgress    *progress,
+                                                         GimpValueArray  *args,
+                                                         GError         **error);
+static void        gimp_procedure_real_execute_async    (GimpProcedure   *procedure,
+                                                         Gimp            *gimp,
+                                                         GimpContext     *context,
+                                                         GimpProgress    *progress,
+                                                         GimpValueArray  *args,
+                                                         GimpDisplay     *display);
 
-static void          gimp_procedure_free_strings  (GimpProcedure  *procedure);
-static gboolean      gimp_procedure_validate_args (GimpProcedure  *procedure,
-                                                   GParamSpec    **param_specs,
-                                                   gint            n_param_specs,
-                                                   GValueArray    *args,
-                                                   gboolean        return_vals,
-                                                   GError        **error);
+static void          gimp_procedure_free_help           (GimpProcedure   *procedure);
+static void          gimp_procedure_free_attribution    (GimpProcedure   *procedure);
+
+static gboolean      gimp_procedure_validate_args       (GimpProcedure   *procedure,
+                                                         GParamSpec     **param_specs,
+                                                         gint             n_param_specs,
+                                                         GimpValueArray  *args,
+                                                         gboolean         return_vals,
+                                                         GError         **error);
 
 
-G_DEFINE_TYPE (GimpProcedure, gimp_procedure, GIMP_TYPE_OBJECT)
+G_DEFINE_TYPE (GimpProcedure, gimp_procedure, GIMP_TYPE_VIEWABLE)
 
 #define parent_class gimp_procedure_parent_class
 
@@ -84,6 +95,11 @@ gimp_procedure_class_init (GimpProcedureClass *klass)
 
   gimp_object_class->get_memsize = gimp_procedure_get_memsize;
 
+  klass->get_label               = gimp_procedure_real_get_label;
+  klass->get_menu_label          = gimp_procedure_real_get_menu_label;
+  klass->get_blurb               = gimp_procedure_real_get_blurb;
+  klass->get_help_id             = gimp_procedure_real_get_help_id;
+  klass->get_sensitive           = gimp_procedure_real_get_sensitive;
   klass->execute                 = gimp_procedure_real_execute;
   klass->execute_async           = gimp_procedure_real_execute_async;
 }
@@ -91,7 +107,7 @@ gimp_procedure_class_init (GimpProcedureClass *klass)
 static void
 gimp_procedure_init (GimpProcedure *procedure)
 {
-  procedure->proc_type = GIMP_INTERNAL;
+  procedure->proc_type = GIMP_PDB_PROC_TYPE_INTERNAL;
 }
 
 static void
@@ -100,15 +116,18 @@ gimp_procedure_finalize (GObject *object)
   GimpProcedure *procedure = GIMP_PROCEDURE (object);
   gint           i;
 
-  gimp_procedure_free_strings (procedure);
+  gimp_procedure_free_help        (procedure);
+  gimp_procedure_free_attribution (procedure);
+
+  g_clear_pointer (&procedure->deprecated, g_free);
+  g_clear_pointer (&procedure->label,      g_free);
 
   if (procedure->args)
     {
       for (i = 0; i < procedure->num_args; i++)
         g_param_spec_unref (procedure->args[i]);
 
-      g_free (procedure->args);
-      procedure->args = NULL;
+      g_clear_pointer (&procedure->args, g_free);
     }
 
   if (procedure->values)
@@ -116,8 +135,7 @@ gimp_procedure_finalize (GObject *object)
       for (i = 0; i < procedure->num_values; i++)
         g_param_spec_unref (procedure->values[i]);
 
-      g_free (procedure->values);
-      procedure->values = NULL;
+      g_clear_pointer (&procedure->values, g_free);
     }
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -131,17 +149,22 @@ gimp_procedure_get_memsize (GimpObject *object,
   gint64         memsize   = 0;
   gint           i;
 
-  if (! procedure->static_strings)
+  if (! procedure->static_help)
     {
-      memsize += gimp_string_get_memsize (procedure->original_name);
       memsize += gimp_string_get_memsize (procedure->blurb);
       memsize += gimp_string_get_memsize (procedure->help);
-      memsize += gimp_string_get_memsize (procedure->author);
+      memsize += gimp_string_get_memsize (procedure->help_id);
+    }
+
+  if (! procedure->static_attribution)
+    {
+      memsize += gimp_string_get_memsize (procedure->authors);
       memsize += gimp_string_get_memsize (procedure->copyright);
       memsize += gimp_string_get_memsize (procedure->date);
-      memsize += gimp_string_get_memsize (procedure->deprecated);
     }
 
+  memsize += gimp_string_get_memsize (procedure->deprecated);
+
   memsize += procedure->num_args * sizeof (GParamSpec *);
 
   for (i = 0; i < procedure->num_args; i++)
@@ -156,15 +179,69 @@ gimp_procedure_get_memsize (GimpObject *object,
                                                                   gui_size);
 }
 
-static GValueArray *
-gimp_procedure_real_execute (GimpProcedure  *procedure,
-                             Gimp           *gimp,
-                             GimpContext    *context,
-                             GimpProgress   *progress,
-                             GValueArray    *args,
-                             GError        **error)
+static const gchar *
+gimp_procedure_real_get_label (GimpProcedure *procedure)
 {
-  g_return_val_if_fail (args->n_values >= procedure->num_args, NULL);
+  gchar *ellipsis;
+  gchar *label;
+
+  if (procedure->label)
+    return procedure->label;
+
+  label = gimp_strip_uline (gimp_procedure_get_menu_label (procedure));
+
+  ellipsis = strstr (label, "...");
+
+  if (! ellipsis)
+    ellipsis = strstr (label, "\342\200\246" /* U+2026 HORIZONTAL ELLIPSIS */);
+
+  if (ellipsis && ellipsis == (label + strlen (label) - 3))
+    *ellipsis = '\0';
+
+  procedure->label = label;
+
+  return procedure->label;
+}
+
+static const gchar *
+gimp_procedure_real_get_menu_label (GimpProcedure *procedure)
+{
+  return gimp_object_get_name (procedure); /* lame fallback */
+}
+
+static const gchar *
+gimp_procedure_real_get_blurb (GimpProcedure *procedure)
+{
+  return procedure->blurb;
+}
+
+static const gchar *
+gimp_procedure_real_get_help_id (GimpProcedure *procedure)
+{
+  if (procedure->help_id)
+    return procedure->help_id;
+
+  return gimp_object_get_name (procedure);
+}
+
+static gboolean
+gimp_procedure_real_get_sensitive (GimpProcedure  *procedure,
+                                   GimpObject     *object,
+                                   const gchar   **tooltip)
+{
+  return TRUE /* random fallback */;
+}
+
+static GimpValueArray *
+gimp_procedure_real_execute (GimpProcedure   *procedure,
+                             Gimp            *gimp,
+                             GimpContext     *context,
+                             GimpProgress    *progress,
+                             GimpValueArray  *args,
+                             GError         **error)
+{
+  g_return_val_if_fail (gimp_value_array_length (args) >=
+                        procedure->num_args, NULL);
 
   return procedure->marshal_func (procedure, gimp,
                                   context, progress,
@@ -176,13 +253,13 @@ gimp_procedure_real_execute_async (GimpProcedure  *procedure,
                                    Gimp           *gimp,
                                    GimpContext    *context,
                                    GimpProgress   *progress,
-                                   GValueArray    *args,
-                                   GimpObject     *display)
+                                   GimpValueArray *args,
+                                   GimpDisplay    *display)
 {
-  GValueArray *return_vals;
-  GError      *error = NULL;
+  GimpValueArray *return_vals;
+  GError         *error = NULL;
 
-  g_return_if_fail (args->n_values >= procedure->num_args);
+  g_return_if_fail (gimp_value_array_length (args) >= procedure->num_args);
 
   return_vals = GIMP_PROCEDURE_GET_CLASS (procedure)->execute (procedure,
                                                                gimp,
@@ -191,12 +268,12 @@ gimp_procedure_real_execute_async (GimpProcedure  *procedure,
                                                                args,
                                                                &error);
 
-  g_value_array_free (return_vals);
+  gimp_value_array_unref (return_vals);
 
   if (error)
     {
       gimp_message_literal (gimp, G_OBJECT (progress), GIMP_MESSAGE_ERROR,
-			    error->message);
+                            error->message);
       g_error_free (error);
     }
 }
@@ -219,90 +296,188 @@ gimp_procedure_new (GimpMarshalFunc marshal_func)
 }
 
 void
-gimp_procedure_set_strings (GimpProcedure *procedure,
-                            const gchar   *original_name,
-                            const gchar   *blurb,
-                            const gchar   *help,
-                            const gchar   *author,
-                            const gchar   *copyright,
-                            const gchar   *date,
-                            const gchar   *deprecated)
+gimp_procedure_set_help (GimpProcedure *procedure,
+                         const gchar   *blurb,
+                         const gchar   *help,
+                         const gchar   *help_id)
 {
   g_return_if_fail (GIMP_IS_PROCEDURE (procedure));
 
-  gimp_procedure_free_strings (procedure);
+  gimp_procedure_free_help (procedure);
 
-  procedure->original_name = g_strdup (original_name);
-  procedure->blurb         = g_strdup (blurb);
-  procedure->help          = g_strdup (help);
-  procedure->author        = g_strdup (author);
-  procedure->copyright     = g_strdup (copyright);
-  procedure->date          = g_strdup (date);
-  procedure->deprecated    = g_strdup (deprecated);
+  procedure->blurb   = g_strdup (blurb);
+  procedure->help    = g_strdup (help);
+  procedure->help_id = g_strdup (help_id);
 
-  procedure->static_strings = FALSE;
+  procedure->static_help = FALSE;
 }
 
 void
-gimp_procedure_set_static_strings (GimpProcedure *procedure,
-                                   const gchar   *original_name,
-                                   const gchar   *blurb,
-                                   const gchar   *help,
-                                   const gchar   *author,
-                                   const gchar   *copyright,
-                                   const gchar   *date,
-                                   const gchar   *deprecated)
+gimp_procedure_set_static_help (GimpProcedure *procedure,
+                                const gchar   *blurb,
+                                const gchar   *help,
+                                const gchar   *help_id)
 {
   g_return_if_fail (GIMP_IS_PROCEDURE (procedure));
 
-  gimp_procedure_free_strings (procedure);
+  gimp_procedure_free_help (procedure);
 
-  procedure->original_name = (gchar *) original_name;
-  procedure->blurb         = (gchar *) blurb;
-  procedure->help          = (gchar *) help;
-  procedure->author        = (gchar *) author;
-  procedure->copyright     = (gchar *) copyright;
-  procedure->date          = (gchar *) date;
-  procedure->deprecated    = (gchar *) deprecated;
+  procedure->blurb   = (gchar *) blurb;
+  procedure->help    = (gchar *) help;
+  procedure->help_id = (gchar *) help_id;
 
-  procedure->static_strings = TRUE;
+  procedure->static_help = TRUE;
 }
 
 void
-gimp_procedure_take_strings (GimpProcedure *procedure,
-                             gchar         *original_name,
-                             gchar         *blurb,
-                             gchar         *help,
-                             gchar         *author,
-                             gchar         *copyright,
-                             gchar         *date,
-                             gchar         *deprecated)
+gimp_procedure_take_help (GimpProcedure *procedure,
+                          gchar         *blurb,
+                          gchar         *help,
+                          gchar         *help_id)
 {
   g_return_if_fail (GIMP_IS_PROCEDURE (procedure));
 
-  gimp_procedure_free_strings (procedure);
+  gimp_procedure_free_help (procedure);
 
-  procedure->original_name = original_name;
-  procedure->blurb         = blurb;
-  procedure->help          = help;
-  procedure->author        = author;
-  procedure->copyright     = copyright;
-  procedure->date          = date;
-  procedure->deprecated    = deprecated;
+  procedure->blurb   = blurb;
+  procedure->help    = help;
+  procedure->help_id = help_id;
 
-  procedure->static_strings = FALSE;
+  procedure->static_help = FALSE;
 }
 
-GValueArray *
-gimp_procedure_execute (GimpProcedure  *procedure,
-                        Gimp           *gimp,
-                        GimpContext    *context,
-                        GimpProgress   *progress,
-                        GValueArray    *args,
-                        GError        **error)
+void
+gimp_procedure_set_attribution (GimpProcedure *procedure,
+                                const gchar   *authors,
+                                const gchar   *copyright,
+                                const gchar   *date)
 {
-  GValueArray *return_vals;
-  GError      *pdb_error = NULL;
+  g_return_if_fail (GIMP_IS_PROCEDURE (procedure));
+
+  gimp_procedure_free_attribution (procedure);
+
+  procedure->authors   = g_strdup (authors);
+  procedure->copyright = g_strdup (copyright);
+  procedure->date      = g_strdup (date);
+
+  procedure->static_attribution = FALSE;
+}
+
+void
+gimp_procedure_set_static_attribution (GimpProcedure *procedure,
+                                       const gchar   *authors,
+                                       const gchar   *copyright,
+                                       const gchar   *date)
+{
+  g_return_if_fail (GIMP_IS_PROCEDURE (procedure));
+
+  gimp_procedure_free_attribution (procedure);
+
+  procedure->authors   = (gchar *) authors;
+  procedure->copyright = (gchar *) copyright;
+  procedure->date      = (gchar *) date;
+
+  procedure->static_attribution = TRUE;
+}
+
+void
+gimp_procedure_take_attribution (GimpProcedure *procedure,
+                                 gchar         *authors,
+                                 gchar         *copyright,
+                                 gchar         *date)
+{
+  g_return_if_fail (GIMP_IS_PROCEDURE (procedure));
+
+  gimp_procedure_free_attribution (procedure);
+
+  procedure->authors   = authors;
+  procedure->copyright = copyright;
+  procedure->date      = date;
+
+  procedure->static_attribution = FALSE;
+}
+
+void
+gimp_procedure_set_deprecated (GimpProcedure *procedure,
+                               const gchar   *deprecated)
+{
+  g_return_if_fail (GIMP_IS_PROCEDURE (procedure));
+
+  g_free (procedure->deprecated);
+  procedure->deprecated = g_strdup (deprecated);
+}
+
+const gchar *
+gimp_procedure_get_label (GimpProcedure *procedure)
+{
+  g_return_val_if_fail (GIMP_IS_PROCEDURE (procedure), NULL);
+
+  return GIMP_PROCEDURE_GET_CLASS (procedure)->get_label (procedure);
+}
+
+const gchar *
+gimp_procedure_get_menu_label (GimpProcedure *procedure)
+{
+  g_return_val_if_fail (GIMP_IS_PROCEDURE (procedure), NULL);
+
+  return GIMP_PROCEDURE_GET_CLASS (procedure)->get_menu_label (procedure);
+}
+
+const gchar *
+gimp_procedure_get_blurb (GimpProcedure *procedure)
+{
+  g_return_val_if_fail (GIMP_IS_PROCEDURE (procedure), NULL);
+
+  return GIMP_PROCEDURE_GET_CLASS (procedure)->get_blurb (procedure);
+}
+
+const gchar *
+gimp_procedure_get_help (GimpProcedure *procedure)
+{
+  g_return_val_if_fail (GIMP_IS_PROCEDURE (procedure), NULL);
+
+  return procedure->help;
+}
+
+const gchar *
+gimp_procedure_get_help_id (GimpProcedure *procedure)
+{
+  g_return_val_if_fail (GIMP_IS_PROCEDURE (procedure), NULL);
+
+  return GIMP_PROCEDURE_GET_CLASS (procedure)->get_help_id (procedure);
+}
+
+gboolean
+gimp_procedure_get_sensitive (GimpProcedure  *procedure,
+                              GimpObject     *object,
+                              const gchar   **reason)
+{
+  const gchar *my_reason  = NULL;
+  gboolean     sensitive;
+
+  g_return_val_if_fail (GIMP_IS_PROCEDURE (procedure), FALSE);
+  g_return_val_if_fail (object == NULL || GIMP_IS_OBJECT (object), FALSE);
+
+  sensitive = GIMP_PROCEDURE_GET_CLASS (procedure)->get_sensitive (procedure,
+                                                                   object,
+                                                                   &my_reason);
+
+  if (reason)
+    *reason = my_reason;
+
+  return sensitive;
+}
+
+GimpValueArray *
+gimp_procedure_execute (GimpProcedure   *procedure,
+                        Gimp            *gimp,
+                        GimpContext     *context,
+                        GimpProgress    *progress,
+                        GimpValueArray  *args,
+                        GError         **error)
+{
+  GimpValueArray *return_vals;
+  GError         *pdb_error = NULL;
 
   g_return_val_if_fail (GIMP_IS_PROCEDURE (procedure), NULL);
   g_return_val_if_fail (GIMP_IS_GIMP (gimp), NULL);
@@ -317,6 +492,12 @@ gimp_procedure_execute (GimpProcedure  *procedure,
     {
       return_vals = gimp_procedure_get_return_values (procedure, FALSE,
                                                       pdb_error);
+      if (! error)
+        /* If we can't properly propagate the error, at least print it
+         * to standard error stream. This makes debugging easier.
+         */
+        g_printerr ("%s failed to validate arguments: %s\n", G_STRFUNC, pdb_error->message);
+
       g_propagate_error (error, pdb_error);
 
       return return_vals;
@@ -327,6 +508,9 @@ gimp_procedure_execute (GimpProcedure  *procedure,
   else
     context = gimp_pdb_context_new (gimp, context, TRUE);
 
+  if (progress)
+    g_object_ref (progress);
+
   /*  call the procedure  */
   return_vals = GIMP_PROCEDURE_GET_CLASS (procedure)->execute (procedure,
                                                                gimp,
@@ -335,11 +519,14 @@ gimp_procedure_execute (GimpProcedure  *procedure,
                                                                args,
                                                                error);
 
+  if (progress)
+    g_object_unref (progress);
+
   g_object_unref (context);
 
   if (return_vals)
     {
-      switch (g_value_get_enum (&return_vals->values[0]))
+      switch (g_value_get_enum (gimp_value_array_index (return_vals, 0)))
         {
         case GIMP_PDB_CALLING_ERROR:
         case GIMP_PDB_EXECUTION_ERROR:
@@ -347,15 +534,17 @@ gimp_procedure_execute (GimpProcedure  *procedure,
            *  from the error message that is optionally passed with
            *  the return values.
            */
-          if (error && *error == NULL)
+          if (error && *error == NULL &&
+              gimp_value_array_length (return_vals) > 1 &&
+              G_VALUE_HOLDS_STRING (gimp_value_array_index (return_vals, 1)))
             {
-              if (return_vals->n_values > 1 &&
-                  G_VALUE_HOLDS_STRING (&return_vals->values[1]))
-                {
-                  g_set_error_literal (error, GIMP_PDB_ERROR,
-                                       GIMP_PDB_ERROR_FAILED,
-				       g_value_get_string (&return_vals->values[1]));
-                }
+              GValue      *value   = gimp_value_array_index (return_vals, 1);
+              const gchar *message = g_value_get_string (value);
+
+              if (message)
+                g_set_error_literal (error, GIMP_PDB_ERROR,
+                                     GIMP_PDB_ERROR_FAILED,
+                                     message);
             }
           break;
 
@@ -389,8 +578,8 @@ gimp_procedure_execute_async (GimpProcedure  *procedure,
                               Gimp           *gimp,
                               GimpContext    *context,
                               GimpProgress   *progress,
-                              GValueArray    *args,
-                              GimpObject     *display,
+                              GimpValueArray *args,
+                              GimpDisplay    *display,
                               GError        **error)
 {
   g_return_if_fail (GIMP_IS_PROCEDURE (procedure));
@@ -398,7 +587,7 @@ gimp_procedure_execute_async (GimpProcedure  *procedure,
   g_return_if_fail (GIMP_IS_CONTEXT (context));
   g_return_if_fail (progress == NULL || GIMP_IS_PROGRESS (progress));
   g_return_if_fail (args != NULL);
-  g_return_if_fail (display == NULL || GIMP_IS_OBJECT (display));
+  g_return_if_fail (display == NULL || GIMP_IS_DISPLAY (display));
   g_return_if_fail (error == NULL || *error == NULL);
 
   if (gimp_procedure_validate_args (procedure,
@@ -410,66 +599,74 @@ gimp_procedure_execute_async (GimpProcedure  *procedure,
       else
         context = gimp_pdb_context_new (gimp, context, TRUE);
 
+      if (progress)
+        g_object_ref (progress);
+
       GIMP_PROCEDURE_GET_CLASS (procedure)->execute_async (procedure, gimp,
                                                            context, progress,
                                                            args, display);
 
+      if (progress)
+        g_object_unref (progress);
+
       g_object_unref (context);
     }
 }
 
-GValueArray *
+GimpValueArray *
 gimp_procedure_get_arguments (GimpProcedure *procedure)
 {
-  GValueArray *args;
-  GValue       value = { 0, };
-  gint         i;
+  GimpValueArray *args;
+  GValue          value = G_VALUE_INIT;
+  gint            i;
 
   g_return_val_if_fail (GIMP_IS_PROCEDURE (procedure), NULL);
 
-  args = g_value_array_new (procedure->num_args);
+  args = gimp_value_array_new (procedure->num_args);
 
   for (i = 0; i < procedure->num_args; i++)
     {
       g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (procedure->args[i]));
-      g_value_array_append (args, &value);
+      g_param_value_set_default (procedure->args[i], &value);
+      gimp_value_array_append (args, &value);
       g_value_unset (&value);
     }
 
   return args;
 }
 
-GValueArray *
+GimpValueArray *
 gimp_procedure_get_return_values (GimpProcedure *procedure,
                                   gboolean       success,
                                   const GError  *error)
 {
-  GValueArray *args;
-  GValue       value = { 0, };
-  gint         i;
+  GimpValueArray *args;
+  GValue          value = G_VALUE_INIT;
+  gint            i;
 
   g_return_val_if_fail (success == FALSE || GIMP_IS_PROCEDURE (procedure),
                         NULL);
 
   if (success)
     {
-      args = g_value_array_new (procedure->num_values + 1);
+      args = gimp_value_array_new (procedure->num_values + 1);
 
       g_value_init (&value, GIMP_TYPE_PDB_STATUS_TYPE);
       g_value_set_enum (&value, GIMP_PDB_SUCCESS);
-      g_value_array_append (args, &value);
+      gimp_value_array_append (args, &value);
       g_value_unset (&value);
 
       for (i = 0; i < procedure->num_values; i++)
         {
           g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (procedure->values[i]));
-          g_value_array_append (args, &value);
+          g_param_value_set_default (procedure->values[i], &value);
+          gimp_value_array_append (args, &value);
           g_value_unset (&value);
         }
     }
   else
     {
-      args = g_value_array_new ((error && error->message) ? 2 : 1);
+      args = gimp_value_array_new ((error && error->message) ? 2 : 1);
 
       g_value_init (&value, GIMP_TYPE_PDB_STATUS_TYPE);
 
@@ -491,7 +688,7 @@ gimp_procedure_get_return_values (GimpProcedure *procedure,
               break;
 
             default:
-              g_assert_not_reached ();
+              gimp_assert_not_reached ();
             }
         }
       else
@@ -499,14 +696,14 @@ gimp_procedure_get_return_values (GimpProcedure *procedure,
           g_value_set_enum (&value, GIMP_PDB_EXECUTION_ERROR);
         }
 
-      g_value_array_append (args, &value);
+      gimp_value_array_append (args, &value);
       g_value_unset (&value);
 
       if (error && error->message)
         {
           g_value_init (&value, G_TYPE_STRING);
           g_value_set_string (&value, error->message);
-          g_value_array_append (args, &value);
+          gimp_value_array_append (args, &value);
           g_value_unset (&value);
         }
     }
@@ -585,55 +782,64 @@ gimp_procedure_name_compare (GimpProcedure *proc1,
                              GimpProcedure *proc2)
 {
   /* Assume there always is a name, don't bother with NULL checks */
-  return strcmp (proc1->original_name,
-                 proc2->original_name);
+  return strcmp (gimp_object_get_name (proc1),
+                 gimp_object_get_name (proc2));
 }
 
 /*  private functions  */
 
 static void
-gimp_procedure_free_strings (GimpProcedure *procedure)
+gimp_procedure_free_help (GimpProcedure *procedure)
 {
-  if (! procedure->static_strings)
+  if (! procedure->static_help)
     {
-      g_free (procedure->original_name);
       g_free (procedure->blurb);
       g_free (procedure->help);
-      g_free (procedure->author);
-      g_free (procedure->copyright);
-      g_free (procedure->date);
-      g_free (procedure->deprecated);
+      g_free (procedure->help_id);
     }
 
-  procedure->original_name = NULL;
-  procedure->blurb         = NULL;
-  procedure->help          = NULL;
-  procedure->author        = NULL;
-  procedure->copyright     = NULL;
-  procedure->date          = NULL;
-  procedure->deprecated    = NULL;
+  procedure->blurb   = NULL;
+  procedure->help    = NULL;
+  procedure->help_id = NULL;
 
-  procedure->static_strings = FALSE;
+  procedure->static_help = FALSE;
+}
+
+static void
+gimp_procedure_free_attribution (GimpProcedure *procedure)
+{
+  if (! procedure->static_attribution)
+    {
+      g_free (procedure->authors);
+      g_free (procedure->copyright);
+      g_free (procedure->date);
+    }
+
+  procedure->authors   = NULL;
+  procedure->copyright = NULL;
+  procedure->date      = NULL;
+
+  procedure->static_attribution = FALSE;
 }
 
 static gboolean
 gimp_procedure_validate_args (GimpProcedure  *procedure,
                               GParamSpec    **param_specs,
                               gint            n_param_specs,
-                              GValueArray    *args,
+                              GimpValueArray *args,
                               gboolean        return_vals,
                               GError        **error)
 {
   gint i;
 
-  for (i = 0; i < MIN (args->n_values, n_param_specs); i++)
+  for (i = 0; i < MIN (gimp_value_array_length (args), n_param_specs); i++)
     {
-      GValue     *arg       = &args->values[i];
+      GValue     *arg       = gimp_value_array_index (args, i);
       GParamSpec *pspec     = param_specs[i];
       GType       arg_type  = G_VALUE_TYPE (arg);
       GType       spec_type = G_PARAM_SPEC_VALUE_TYPE (pspec);
 
-      if (arg_type != spec_type)
+      if (! g_type_is_a (arg_type, spec_type))
         {
           if (return_vals)
             {
@@ -664,7 +870,7 @@ gimp_procedure_validate_args (GimpProcedure  *procedure,
         }
       else if (! (pspec->flags & GIMP_PARAM_NO_VALIDATE))
         {
-          GValue string_value = { 0, };
+          GValue string_value = G_VALUE_INIT;
 
           g_value_init (&string_value, G_TYPE_STRING);
 
@@ -676,8 +882,8 @@ gimp_procedure_validate_args (GimpProcedure  *procedure,
 
           if (g_param_value_validate (pspec, arg))
             {
-              if (GIMP_IS_PARAM_SPEC_DRAWABLE_ID (pspec) &&
-                  g_value_get_int (arg) == -1)
+              if (GIMP_IS_PARAM_SPEC_DRAWABLE (pspec) &&
+                  g_value_get_object (arg) == NULL)
                 {
                   if (return_vals)
                     {
@@ -706,8 +912,8 @@ gimp_procedure_validate_args (GimpProcedure  *procedure,
                                    g_param_spec_get_name (pspec));
                     }
                 }
-              else if (GIMP_IS_PARAM_SPEC_IMAGE_ID (pspec) &&
-                       g_value_get_int (arg) == -1)
+              else if (GIMP_IS_PARAM_SPEC_IMAGE (pspec) &&
+                       g_value_get_object (arg) == NULL)
                 {
                   if (return_vals)
                     {
@@ -778,6 +984,66 @@ gimp_procedure_validate_args (GimpProcedure  *procedure,
               return FALSE;
             }
 
+          /*  UTT-8 validate all strings  */
+          if (G_PARAM_SPEC_TYPE (pspec) == G_TYPE_PARAM_STRING ||
+              (G_PARAM_SPEC_TYPE (pspec) == G_TYPE_PARAM_BOXED &&
+               G_PARAM_SPEC_VALUE_TYPE (pspec) == G_TYPE_STRV))
+            {
+              gboolean valid = TRUE;
+
+              if (G_PARAM_SPEC_TYPE (pspec) == G_TYPE_PARAM_STRING)
+                {
+                  const gchar *string = g_value_get_string (arg);
+
+                  if (string)
+                    valid = g_utf8_validate (string, -1, NULL);
+                }
+              else
+                {
+                  const gchar **array = g_value_get_boxed (arg);
+
+                  if (array)
+                    {
+                      gint i;
+
+                      for (i = 0; array[i]; i++)
+                        {
+                          valid = g_utf8_validate (array[i], -1, NULL);
+                          if (! valid)
+                            break;
+                        }
+                    }
+                }
+
+              if (! valid)
+                {
+                  if (return_vals)
+                    {
+                      g_set_error (error,
+                                   GIMP_PDB_ERROR,
+                                   GIMP_PDB_ERROR_INVALID_RETURN_VALUE,
+                                   _("Procedure '%s' returned an "
+                                     "invalid UTF-8 string for argument '%s'."),
+                                   gimp_object_get_name (procedure),
+                                   g_param_spec_get_name (pspec));
+                    }
+                  else
+                    {
+                      g_set_error (error,
+                                   GIMP_PDB_ERROR,
+                                   GIMP_PDB_ERROR_INVALID_ARGUMENT,
+                                   _("Procedure '%s' has been called with an "
+                                     "invalid UTF-8 string for argument '%s'."),
+                                   gimp_object_get_name (procedure),
+                                   g_param_spec_get_name (pspec));
+                    }
+
+                  g_value_unset (&string_value);
+
+                  return FALSE;
+                }
+            }
+
           g_value_unset (&string_value);
         }
     }
