@@ -17,7 +17,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_TEXT_TOOL_H__
@@ -62,7 +62,10 @@ struct _GimpTextTool
   gboolean        text_box_fixed;
 
   GimpTextLayout *layout;
-  gboolean        drawing_blocked;
+  gint            drawing_blocked;
+
+  GimpToolWidget *widget;
+  GimpToolWidget *grab_widget;
 
   /* text editor state: */
 
@@ -77,11 +80,11 @@ struct _GimpTextTool
   GtkIMContext   *im_context;
   gboolean        needs_im_reset;
 
-  GtkWidget      *preedit_overlay;
-  GtkWidget      *preedit_label;
-
+  gboolean        preedit_active;
   gchar          *preedit_string;
   gint            preedit_cursor;
+  GtkTextMark    *preedit_start;
+  GtkTextMark    *preedit_end;
 
   gboolean        overwrite_mode;
   gint            x_pos;
@@ -103,22 +106,28 @@ void       gimp_text_tool_register               (GimpToolRegisterCallback  call
 
 GType      gimp_text_tool_get_type               (void) G_GNUC_CONST;
 
-void       gimp_text_tool_set_layer              (GimpTextTool *text_tool,
-                                                  GimpLayer    *layer);
+gboolean   gimp_text_tool_set_layer              (GimpTextTool  *text_tool,
+                                                  GimpLayer     *layer);
 
-gboolean   gimp_text_tool_get_has_text_selection (GimpTextTool *text_tool);
+gboolean   gimp_text_tool_get_has_text_selection (GimpTextTool  *text_tool);
 
-void       gimp_text_tool_delete_selection       (GimpTextTool *text_tool);
-void       gimp_text_tool_cut_clipboard          (GimpTextTool *text_tool);
-void       gimp_text_tool_copy_clipboard         (GimpTextTool *text_tool);
-void       gimp_text_tool_paste_clipboard        (GimpTextTool *text_tool);
+void       gimp_text_tool_delete_selection       (GimpTextTool  *text_tool);
+void       gimp_text_tool_cut_clipboard          (GimpTextTool  *text_tool);
+void       gimp_text_tool_copy_clipboard         (GimpTextTool  *text_tool);
+void       gimp_text_tool_paste_clipboard        (GimpTextTool  *text_tool);
 
-void       gimp_text_tool_create_vectors         (GimpTextTool *text_tool);
-void       gimp_text_tool_create_vectors_warped  (GimpTextTool *text_tool);
+void       gimp_text_tool_create_vectors         (GimpTextTool  *text_tool);
+gboolean   gimp_text_tool_create_vectors_warped  (GimpTextTool  *text_tool,
+                                                  GError       **error);
+
+GimpTextDirection
+           gimp_text_tool_get_direction          (GimpTextTool  *text_tool);
 
 /*  only for the text editor  */
-void       gimp_text_tool_clear_layout           (GimpTextTool *text_tool);
-gboolean   gimp_text_tool_ensure_layout          (GimpTextTool *text_tool);
+void       gimp_text_tool_clear_layout           (GimpTextTool  *text_tool);
+gboolean   gimp_text_tool_ensure_layout          (GimpTextTool  *text_tool);
+void       gimp_text_tool_apply                  (GimpTextTool  *text_tool,
+                                                  gboolean       push_undo);
 
 
 #endif /* __GIMP_TEXT_TOOL_H__ */
