@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_ACTION_GROUP_H__
@@ -38,7 +38,7 @@ struct _GimpActionGroup
 
   Gimp                      *gimp;
   gchar                     *label;
-  gchar                     *stock_id;
+  gchar                     *icon_name;
 
   gpointer                   user_data;
 
@@ -53,30 +53,35 @@ struct _GimpActionGroupClass
 
   /* signals */
   void (* action_added) (GimpActionGroup *group,
-                         GtkAction       *action);
+                         GimpAction      *action);
 };
 
+
+typedef void (* GimpActionCallback) (GimpAction *action,
+                                     GVariant   *value,
+                                     gpointer    data);
+
 struct _GimpActionEntry
 {
-  const gchar *name;
-  const gchar *stock_id;
-  const gchar *label;
-  const gchar *accelerator;
-  const gchar *tooltip;
-  GCallback    callback;
+  const gchar        *name;
+  const gchar        *icon_name;
+  const gchar        *label;
+  const gchar        *accelerator;
+  const gchar        *tooltip;
+  GimpActionCallback  callback;
 
-  const gchar *help_id;
+  const gchar        *help_id;
 };
 
 struct _GimpToggleActionEntry
 {
-  const gchar *name;
-  const gchar *stock_id;
-  const gchar *label;
-  const gchar *accelerator;
-  const gchar *tooltip;
-  GCallback    callback;
-  gboolean     is_active;
+  const gchar        *name;
+  const gchar        *icon_name;
+  const gchar        *label;
+  const gchar        *accelerator;
+  const gchar        *tooltip;
+  GimpActionCallback  callback;
+  gboolean            is_active;
 
   const gchar *help_id;
 };
@@ -84,7 +89,7 @@ struct _GimpToggleActionEntry
 struct _GimpRadioActionEntry
 {
   const gchar *name;
-  const gchar *stock_id;
+  const gchar *icon_name;
   const gchar *label;
   const gchar *accelerator;
   const gchar *tooltip;
@@ -96,7 +101,7 @@ struct _GimpRadioActionEntry
 struct _GimpEnumActionEntry
 {
   const gchar *name;
-  const gchar *stock_id;
+  const gchar *icon_name;
   const gchar *label;
   const gchar *accelerator;
   const gchar *tooltip;
@@ -109,7 +114,7 @@ struct _GimpEnumActionEntry
 struct _GimpStringActionEntry
 {
   const gchar *name;
-  const gchar *stock_id;
+  const gchar *icon_name;
   const gchar *label;
   const gchar *accelerator;
   const gchar *tooltip;
@@ -118,16 +123,28 @@ struct _GimpStringActionEntry
   const gchar *help_id;
 };
 
-struct _GimpPlugInActionEntry
+struct _GimpDoubleActionEntry
 {
-  const gchar         *name;
-  const gchar         *stock_id;
-  const gchar         *label;
-  const gchar         *accelerator;
-  const gchar         *tooltip;
-  GimpPlugInProcedure *procedure;
+  const gchar   *name;
+  const gchar   *icon_name;
+  const gchar   *label;
+  const gchar   *accelerator;
+  const gchar   *tooltip;
+  const gdouble  value;
 
-  const gchar         *help_id;
+  const gchar   *help_id;
+};
+
+struct _GimpProcedureActionEntry
+{
+  const gchar   *name;
+  const gchar   *icon_name;
+  const gchar   *label;
+  const gchar   *accelerator;
+  const gchar   *tooltip;
+  GimpProcedure *procedure;
+
+  const gchar   *help_id;
 };
 
 
@@ -136,44 +153,66 @@ GType            gimp_action_group_get_type   (void) G_GNUC_CONST;
 GimpActionGroup *gimp_action_group_new        (Gimp                  *gimp,
                                                const gchar           *name,
                                                const gchar           *label,
-                                               const gchar           *stock_id,
+                                               const gchar           *icon_name,
                                                gpointer               user_data,
                                                GimpActionGroupUpdateFunc update_func);
 
 GList *gimp_action_groups_from_name           (const gchar           *name);
 
+const gchar * gimp_action_group_get_name       (GimpActionGroup       *group);
+
+void  gimp_action_group_add_action            (GimpActionGroup *action_group,
+                                               GimpAction      *action);
+void  gimp_action_group_add_action_with_accel (GimpActionGroup *action_group,
+                                               GimpAction      *action,
+                                               const gchar     *accelerator);
+void  gimp_action_group_remove_action         (GimpActionGroup *action_group,
+                                               GimpAction      *action);
+
+GimpAction  * gimp_action_group_get_action     (GimpActionGroup       *group,
+                                                const gchar           *action_name);
+GList       * gimp_action_group_list_actions   (GimpActionGroup       *group);
+
 void   gimp_action_group_update               (GimpActionGroup       *group,
                                                gpointer               update_data);
 
 void   gimp_action_group_add_actions          (GimpActionGroup             *group,
-					       const gchar                 *msg_context,
+                                               const gchar                 *msg_context,
                                                const GimpActionEntry       *entries,
                                                guint                        n_entries);
 void   gimp_action_group_add_toggle_actions   (GimpActionGroup             *group,
-					       const gchar                 *msg_context,
+                                               const gchar                 *msg_context,
                                                const GimpToggleActionEntry *entries,
                                                guint                        n_entries);
 GSList *gimp_action_group_add_radio_actions   (GimpActionGroup             *group,
-					       const gchar                 *msg_context,
+                                               const gchar                 *msg_context,
                                                const GimpRadioActionEntry  *entries,
                                                guint                        n_entries,
                                                GSList                      *radio_group,
                                                gint                         value,
-                                               GCallback                    callback);
+                                               GimpActionCallback           callback);
 void   gimp_action_group_add_enum_actions     (GimpActionGroup             *group,
-					       const gchar                 *msg_context,
+                                               const gchar                 *msg_context,
                                                const GimpEnumActionEntry   *entries,
                                                guint                        n_entries,
-                                               GCallback                    callback);
+                                               GimpActionCallback           callback);
 void   gimp_action_group_add_string_actions   (GimpActionGroup             *group,
-					       const gchar                 *msg_context,
+                                               const gchar                 *msg_context,
                                                const GimpStringActionEntry *entries,
                                                guint                        n_entries,
-                                               GCallback                    callback);
-void   gimp_action_group_add_plug_in_actions  (GimpActionGroup             *group,
-                                               const GimpPlugInActionEntry *entries,
+                                               GimpActionCallback           callback);
+void   gimp_action_group_add_double_actions   (GimpActionGroup             *group,
+                                               const gchar                 *msg_context,
+                                               const GimpDoubleActionEntry *entries,
                                                guint                        n_entries,
-                                               GCallback                    callback);
+                                               GimpActionCallback           callback);
+void   gimp_action_group_add_procedure_actions(GimpActionGroup             *group,
+                                               const GimpProcedureActionEntry *entries,
+                                               guint                        n_entries,
+                                               GimpActionCallback           callback);
+
+void   gimp_action_group_remove_action_and_accel (GimpActionGroup          *group,
+                                                  GimpAction               *action);
 
 void          gimp_action_group_activate_action       (GimpActionGroup *group,
                                                        const gchar     *action_name);
@@ -182,18 +221,25 @@ void          gimp_action_group_set_action_visible    (GimpActionGroup *group,
                                                        gboolean         visible);
 void          gimp_action_group_set_action_sensitive  (GimpActionGroup *group,
                                                        const gchar     *action_name,
-                                                       gboolean         sensitive);
+                                                       gboolean         sensitive,
+                                                       const gchar     *reason);
 void          gimp_action_group_set_action_active     (GimpActionGroup *group,
                                                        const gchar     *action_name,
                                                        gboolean         active);
 void          gimp_action_group_set_action_label      (GimpActionGroup *group,
                                                        const gchar     *action_name,
                                                        const gchar     *label);
+void          gimp_action_group_set_action_pixbuf     (GimpActionGroup *group,
+                                                       const gchar     *action_name,
+                                                       GdkPixbuf       *pixbuf);
 void          gimp_action_group_set_action_tooltip    (GimpActionGroup *group,
                                                        const gchar     *action_name,
                                                        const gchar     *tooltip);
 const gchar * gimp_action_group_get_action_tooltip    (GimpActionGroup *group,
                                                        const gchar     *action_name);
+void          gimp_action_group_set_action_context    (GimpActionGroup *group,
+                                                       const gchar     *action_name,
+                                                       GimpContext     *context);
 void          gimp_action_group_set_action_color      (GimpActionGroup *group,
                                                        const gchar     *action_name,
                                                        const GimpRGB   *color,
