@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_HELP_IDS_H__
@@ -37,13 +37,12 @@
 #define GIMP_HELP_FILE_EXPORT                     "gimp-file-export"
 #define GIMP_HELP_FILE_EXPORT_AS                  "gimp-file-export-as"
 #define GIMP_HELP_FILE_OVERWRITE                  "gimp-file-overwrite"
-#define GIMP_HELP_FILE_SAVE_AS_TEMPLATE           "gimp-file-save-as-template"
 #define GIMP_HELP_FILE_REVERT                     "gimp-file-revert"
 #define GIMP_HELP_FILE_CLOSE                      "gimp-file-close"
 #define GIMP_HELP_FILE_CLOSE_ALL                  "gimp-file-close-all"
 #define GIMP_HELP_FILE_COPY_LOCATION              "gimp-file-copy-location"
 #define GIMP_HELP_FILE_SHOW_IN_FILE_MANAGER       "gimp-file-show-in-file-manager"
-#define GIMP_HELP_FILE_CREATE_TEMPLATE            "gimp-file-save-as-template" /* Update string along with gimp-help-2 */
+#define GIMP_HELP_FILE_CREATE_TEMPLATE            "gimp-file-create-template"
 #define GIMP_HELP_FILE_QUIT                       "gimp-file-quit"
 
 #define GIMP_HELP_EDIT_UNDO                       "gimp-edit-undo"
@@ -51,13 +50,15 @@
 #define GIMP_HELP_EDIT_STRONG_UNDO                "gimp-edit-strong-undo"
 #define GIMP_HELP_EDIT_STRONG_REDO                "gimp-edit-strong-redo"
 #define GIMP_HELP_EDIT_UNDO_CLEAR                 "gimp-edit-undo-clear"
-#define GIMP_HELP_EDIT_FADE                       "gimp-edit-fade"
 #define GIMP_HELP_EDIT_CUT                        "gimp-edit-cut"
 #define GIMP_HELP_EDIT_COPY                       "gimp-edit-copy"
 #define GIMP_HELP_EDIT_COPY_VISIBLE               "gimp-edit-copy-visible"
 #define GIMP_HELP_EDIT_PASTE                      "gimp-edit-paste"
+#define GIMP_HELP_EDIT_PASTE_IN_PLACE             "gimp-edit-paste-in-place"
 #define GIMP_HELP_EDIT_PASTE_INTO                 "gimp-edit-paste-into"
+#define GIMP_HELP_EDIT_PASTE_INTO_IN_PLACE        "gimp-edit-paste-into-in-place"
 #define GIMP_HELP_EDIT_PASTE_AS_NEW_LAYER         "gimp-edit-paste-as-new-layer"
+#define GIMP_HELP_EDIT_PASTE_AS_NEW_LAYER_IN_PLACE "gimp-edit-paste-as-new-layer-in-place"
 #define GIMP_HELP_EDIT_PASTE_AS_NEW_IMAGE         "gimp-edit-paste-as-new-image"
 #define GIMP_HELP_EDIT_CLEAR                      "gimp-edit-clear"
 #define GIMP_HELP_EDIT_FILL_FG                    "gimp-edit-fill-fg"
@@ -81,7 +82,10 @@
 #define GIMP_HELP_SELECTION_TO_PATH               "gimp-selection-to-path"
 
 #define GIMP_HELP_VIEW_NEW                        "gimp-view-new"
+#define GIMP_HELP_VIEW_SHOW_ALL                   "gimp-view-show-all"
 #define GIMP_HELP_VIEW_DOT_FOR_DOT                "gimp-view-dot-for-dot"
+#define GIMP_HELP_VIEW_RESET                      "gimp-view-reset"
+#define GIMP_HELP_VIEW_SCROLL_CENTER              "gimp-view-scroll-center"
 #define GIMP_HELP_VIEW_ZOOM_REVERT                "gimp-view-zoom-revert"
 #define GIMP_HELP_VIEW_ZOOM_OUT                   "gimp-view-zoom-out"
 #define GIMP_HELP_VIEW_ZOOM_IN                    "gimp-view-zoom-in"
@@ -91,6 +95,7 @@
 #define GIMP_HELP_VIEW_ZOOM_SELECTION             "gimp-view-zoom-selection"
 #define GIMP_HELP_VIEW_ZOOM_OTHER                 "gimp-view-zoom-other"
 #define GIMP_HELP_VIEW_FLIP                       "gimp-view-flip"
+#define GIMP_HELP_VIEW_FLIP_RESET                 "gimp-view-flip-reset"
 #define GIMP_HELP_VIEW_ROTATE_RESET               "gimp-view-rotate-reset"
 #define GIMP_HELP_VIEW_ROTATE_15                  "gimp-view-rotate-15"
 #define GIMP_HELP_VIEW_ROTATE_90                  "gimp-view-rotate-90"
@@ -101,6 +106,7 @@
 #define GIMP_HELP_VIEW_COLOR_MANAGEMENT           "gimp-view-color-management"
 #define GIMP_HELP_VIEW_SHOW_SELECTION             "gimp-view-show-selection"
 #define GIMP_HELP_VIEW_SHOW_LAYER_BOUNDARY        "gimp-view-show-layer-boundary"
+#define GIMP_HELP_VIEW_SHOW_CANVAS_BOUNDARY       "gimp-view-show-canvas-boundary"
 #define GIMP_HELP_VIEW_SHOW_GUIDES                "gimp-view-show-guides"
 #define GIMP_HELP_VIEW_SHOW_GRID                  "gimp-view-show-grid"
 #define GIMP_HELP_VIEW_SHOW_SAMPLE_POINTS         "gimp-view-show-sample-points"
@@ -153,10 +159,10 @@
 #define GIMP_HELP_IMAGE_CROP                      "gimp-image-crop"
 #define GIMP_HELP_IMAGE_MERGE_LAYERS              "gimp-image-merge-layers"
 #define GIMP_HELP_IMAGE_FLATTEN                   "gimp-image-flatten"
-#define GIMP_HELP_IMAGE_COLOR_MANAGEMENT_ENABLED  "gimp-image-color-management-enabled"
 #define GIMP_HELP_IMAGE_COLOR_PROFILE_ASSIGN      "gimp-image-color-profile-assign"
 #define GIMP_HELP_IMAGE_COLOR_PROFILE_CONVERT     "gimp-image-color-profile-convert"
 #define GIMP_HELP_IMAGE_COLOR_PROFILE_DISCARD     "gimp-image-color-profile-discard"
+#define GIMP_HELP_IMAGE_COLOR_PROFILE_USE_SRGB    "gimp-image-color-profile-use-srgb"
 #define GIMP_HELP_IMAGE_COLOR_PROFILE_SAVE        "gimp-image-color-profile-save"
 #define GIMP_HELP_IMAGE_COLOR_PROFILE_IMPORT      "gimp-image-color-profile-import"
 #define GIMP_HELP_IMAGE_GRID                      "gimp-image-grid"
@@ -194,6 +200,7 @@
 #define GIMP_HELP_LAYER_LOCK_ALPHA                "gimp-layer-lock-alpha"
 #define GIMP_HELP_LAYER_LOCK_PIXELS               "gimp-layer-lock-pixels"
 #define GIMP_HELP_LAYER_LOCK_POSITION             "gimp-layer-lock-position"
+#define GIMP_HELP_LAYER_LOCK_VISIBILITY           "gimp-layer-lock-visibility"
 #define GIMP_HELP_LAYER_MASK_ADD                  "gimp-layer-mask-add"
 #define GIMP_HELP_LAYER_MASK_APPLY                "gimp-layer-mask-apply"
 #define GIMP_HELP_LAYER_MASK_DELETE               "gimp-layer-mask-delete"
@@ -215,7 +222,6 @@
 #define GIMP_HELP_LAYER_ROTATE_90                 "gimp-layer-rotate-90"
 #define GIMP_HELP_LAYER_ROTATE_180                "gimp-layer-rotate-180"
 #define GIMP_HELP_LAYER_ROTATE_270                "gimp-layer-rotate-270"
-#define GIMP_HELP_LAYER_OFFSET                    "gimp-layer-offset"
 #define GIMP_HELP_LAYER_RESIZE                    "gimp-layer-resize"
 #define GIMP_HELP_LAYER_RESIZE_TO_IMAGE           "gimp-layer-resize-to-image"
 #define GIMP_HELP_LAYER_SCALE                     "gimp-layer-scale"
@@ -230,11 +236,16 @@
 #define GIMP_HELP_CHANNEL_LOWER_TO_BOTTOM         "gimp-channel-lower-to-bottom"
 #define GIMP_HELP_CHANNEL_DUPLICATE               "gimp-channel-duplicate"
 #define GIMP_HELP_CHANNEL_DELETE                  "gimp-channel-delete"
+#define GIMP_HELP_CHANNEL_PREVIOUS                "gimp-channel-previous"
+#define GIMP_HELP_CHANNEL_NEXT                    "gimp-channel-next"
+#define GIMP_HELP_CHANNEL_TOP                     "gimp-channel-top"
+#define GIMP_HELP_CHANNEL_BOTTOM                  "gimp-channel-bottom"
 #define GIMP_HELP_CHANNEL_VISIBLE                 "gimp-channel-visible"
 #define GIMP_HELP_CHANNEL_LINKED                  "gimp-channel-linked"
 #define GIMP_HELP_CHANNEL_COLOR_TAG               "gimp-channel-color-tag"
 #define GIMP_HELP_CHANNEL_LOCK_PIXELS             "gimp-channel-lock-pixels"
 #define GIMP_HELP_CHANNEL_LOCK_POSITION           "gimp-channel-lock-position"
+#define GIMP_HELP_CHANNEL_LOCK_VISIBILITY         "gimp-channel-lock-visibility"
 #define GIMP_HELP_CHANNEL_SELECTION_REPLACE       "gimp-channel-selection-replace"
 #define GIMP_HELP_CHANNEL_SELECTION_ADD           "gimp-channel-selection-add"
 #define GIMP_HELP_CHANNEL_SELECTION_SUBTRACT      "gimp-channel-selection-subtract"
@@ -248,6 +259,10 @@
 
 #define GIMP_HELP_PATH_DIALOG                     "gimp-path-dialog"
 #define GIMP_HELP_PATH_NEW                        "gimp-path-new"
+#define GIMP_HELP_PATH_PREVIOUS                   "gimp-path-previous"
+#define GIMP_HELP_PATH_NEXT                       "gimp-path-next"
+#define GIMP_HELP_PATH_TOP                        "gimp-path-top"
+#define GIMP_HELP_PATH_BOTTOM                     "gimp-path-bottom"
 #define GIMP_HELP_PATH_RAISE                      "gimp-path-raise"
 #define GIMP_HELP_PATH_RAISE_TO_TOP               "gimp-path-raise-to-top"
 #define GIMP_HELP_PATH_LOWER                      "gimp-path-lower"
@@ -272,11 +287,8 @@
 #define GIMP_HELP_PATH_EXPORT                     "gimp-path-export"
 #define GIMP_HELP_PATH_EDIT                       "gimp-path-edit"
 
-#define GIMP_HELP_TOOLS_DIALOG                    "gimp-tools-dialog"
-
 #define GIMP_HELP_TOOL_AIRBRUSH                   "gimp-tool-airbrush"
 #define GIMP_HELP_TOOL_ALIGN                      "gimp-tool-align"
-#define GIMP_HELP_TOOL_BLEND                      "gimp-tool-blend"
 #define GIMP_HELP_TOOL_BRIGHTNESS_CONTRAST        "gimp-tool-brightness-contrast"
 #define GIMP_HELP_TOOL_BUCKET_FILL                "gimp-tool-bucket-fill"
 #define GIMP_HELP_TOOL_BY_COLOR_SELECT            "gimp-tool-by-color-select"
@@ -288,7 +300,6 @@
 #define GIMP_HELP_TOOL_CONVOLVE                   "gimp-tool-convolve"
 #define GIMP_HELP_TOOL_CROP                       "gimp-tool-crop"
 #define GIMP_HELP_TOOL_CURVES                     "gimp-tool-curves"
-#define GIMP_HELP_TOOL_CURVES                     "gimp-tool-curves"
 #define GIMP_HELP_TOOL_DODGE_BURN                 "gimp-tool-dodge-burn"
 #define GIMP_HELP_TOOL_ELLIPSE_SELECT             "gimp-tool-ellipse-select"
 #define GIMP_HELP_TOOL_ERASER                     "gimp-tool-eraser"
@@ -297,6 +308,7 @@
 #define GIMP_HELP_TOOL_FOREGROUND_SELECT          "gimp-tool-foreground-select"
 #define GIMP_HELP_TOOL_FUZZY_SELECT               "gimp-tool-fuzzy-select"
 #define GIMP_HELP_TOOL_GEGL                       "gimp-tool-gegl"
+#define GIMP_HELP_TOOL_GRADIENT                   "gimp-tool-gradient"
 #define GIMP_HELP_TOOL_HANDLE_TRANSFORM           "gimp-tool-handle-transform"
 #define GIMP_HELP_TOOL_HEAL                       "gimp-tool-heal"
 #define GIMP_HELP_TOOL_HUE_SATURATION             "gimp-tool-hue-saturation"
@@ -307,6 +319,7 @@
 #define GIMP_HELP_TOOL_MOVE                       "gimp-tool-move"
 #define GIMP_HELP_TOOL_MYPAINT_BRUSH              "gimp-tool-mypaint-brush"
 #define GIMP_HELP_TOOL_N_POINT_DEFORMATION        "gimp-tool-n-point-deformation"
+#define GIMP_HELP_TOOL_OFFSET                     "gimp-tool-offset"
 #define GIMP_HELP_TOOL_PATH                       "gimp-tool-path"
 #define GIMP_HELP_TOOL_PAINTBRUSH                 "gimp-tool-paintbrush"
 #define GIMP_HELP_TOOL_PENCIL                     "gimp-tool-pencil"
@@ -320,6 +333,7 @@
 #define GIMP_HELP_TOOL_SMUDGE                     "gimp-tool-smudge"
 #define GIMP_HELP_TOOL_TEXT                       "gimp-tool-text"
 #define GIMP_HELP_TOOL_THRESHOLD                  "gimp-tool-threshold"
+#define GIMP_HELP_TOOL_TRANSFORM_3D               "gimp-tool-transform-3d"
 #define GIMP_HELP_TOOL_UNIFIED_TRANSFORM          "gimp-tool-unified-transform"
 #define GIMP_HELP_TOOL_VECTORS                    "gimp-tool-vectors"
 #define GIMP_HELP_TOOL_WARP                       "gimp-tool-warp"
@@ -333,6 +347,8 @@
 #define GIMP_HELP_FILTER_ANTIALIAS                "gimp-filter-antialias"
 #define GIMP_HELP_FILTER_APPLY_CANVAS             "gimp-filter-apply-canvas"
 #define GIMP_HELP_FILTER_APPLY_LENS               "gimp-filter-apply-lens"
+#define GIMP_HELP_FILTER_BAYER_MATRIX             "gimp-filter-bayer-matrix"
+#define GIMP_HELP_FILTER_BLOOM                    "gimp-filter-bloom"
 #define GIMP_HELP_FILTER_BUMP_MAP                 "gimp-filter-bump-map"
 #define GIMP_HELP_FILTER_C2G                      "gimp-filter-c2g"
 #define GIMP_HELP_FILTER_CARTOON                  "gimp-filter-cartoon"
@@ -364,11 +380,12 @@
 #define GIMP_HELP_FILTER_ERODE                    "gimp-filter-erode"
 #define GIMP_HELP_FILTER_EXPOSURE                 "gimp-filter-exposure"
 #define GIMP_HELP_FILTER_FATTAL_2002              "gimp-filter-fattal-2002"
+#define GIMP_HELP_FILTER_FOCUS_BLUR               "gimp-filter-focus-blur"
 #define GIMP_HELP_FILTER_FRACTAL_TRACE            "gimp-filter-fractal-trace"
 #define GIMP_HELP_FILTER_GAUSSIAN_BLUR            "gimp-filter-gaussian-blur"
 #define GIMP_HELP_FILTER_GAUSSIAN_BLUR_SELECTIVE  "gimp-filter-gaussian-blur-selective"
 #define GIMP_HELP_FILTER_GEGL_GRAPH               "gimp-filter-gegl"
-#define GIMP_HELP_FILTER_GAUSSIAN_GRID            "gimp-filter-grid"
+#define GIMP_HELP_FILTER_GRID                     "gimp-filter-grid"
 #define GIMP_HELP_FILTER_HIGH_PASS                "gimp-filter-high-pass"
 #define GIMP_HELP_FILTER_HUE_CHROMA               "gimp-filter-hue-chroma"
 #define GIMP_HELP_FILTER_ILLUSION                 "gimp-filter-illusion"
@@ -377,15 +394,22 @@
 #define GIMP_HELP_FILTER_INVERT_VALUE             "gimp-filter-invert-value"
 #define GIMP_HELP_FILTER_IMAGE_GRADIENT           "gimp-filter-image-gradient"
 #define GIMP_HELP_FILTER_KALEIDOSCOPE             "gimp-filter-kaleidoscope"
+#define GIMP_HELP_FILTER_LENS_BLUR                "gimp-filter-lens-blur"
 #define GIMP_HELP_FILTER_LENS_DISTORTION          "gimp-filter-lens-distortion"
 #define GIMP_HELP_FILTER_LENS_FLARE               "gimp-filter-lens-flare"
+#define GIMP_HELP_FILTER_LINEAR_SINUSOID          "gimp-filter-linear-sinusoid"
+#define GIMP_HELP_FILTER_LITTLE_PLANET            "gimp-filter-little-planet"
+#define GIMP_HELP_FILTER_LONG_SHADOW              "gimp-filter-long-shadow"
 #define GIMP_HELP_FILTER_MANTIUK_2006             "gimp-filter-manituk-2006"
 #define GIMP_HELP_FILTER_MAZE                     "gimp-filter-maze"
+#define GIMP_HELP_FILTER_MEAN_CURVATURE_BLUR      "gimp-filter-mean-curvature-blur"
+#define GIMP_HELP_FILTER_MEDIAN_BLUR              "gimp-filter-median-blur"
 #define GIMP_HELP_FILTER_MONO_MIXER               "gimp-filter-mono-mixer"
 #define GIMP_HELP_FILTER_MOSAIC                   "gimp-filter-mosaic"
 #define GIMP_HELP_FILTER_MOTION_BLUR_CIRCULAR     "gimp-filter-motion-blur-circular"
 #define GIMP_HELP_FILTER_MOTION_BLUR_LINEAR       "gimp-filter-motion-blur-linear"
 #define GIMP_HELP_FILTER_MOTION_BLUR_ZOOM         "gimp-filter-motion-blur-zoom"
+#define GIMP_HELP_FILTER_NEWSPRINT                "gimp-filter-newsprint"
 #define GIMP_HELP_FILTER_NOISE_CELL               "gimp-filter-noise-cell"
 #define GIMP_HELP_FILTER_NOISE_CIE_LCH            "gimp-filter-noise-cie-lch"
 #define GIMP_HELP_FILTER_NOISE_HSV                "gimp-filter-noise-hsv"
@@ -398,6 +422,7 @@
 #define GIMP_HELP_FILTER_NOISE_SLUR               "gimp-filter-noise-slur"
 #define GIMP_HELP_FILTER_NOISE_SOLID              "gimp-filter-noise-solid"
 #define GIMP_HELP_FILTER_NOISE_SPREAD             "gimp-filter-noise-spread"
+#define GIMP_HELP_FILTER_NORMAL_MAP               "gimp-filter-normal-map"
 #define GIMP_HELP_FILTER_OILIFY                   "gimp-filter-oilify"
 #define GIMP_HELP_FILTER_PANORAMA_PROJECTION      "gimp-filter-panorama-projection"
 #define GIMP_HELP_FILTER_PHOTOCOPY                "gimp-filter-photocopy"
@@ -405,6 +430,7 @@
 #define GIMP_HELP_FILTER_PLASMA                   "gimp-filter-plasma"
 #define GIMP_HELP_FILTER_POLAR_COORDINATES        "gimp-filter-polar-coordinates"
 #define GIMP_HELP_FILTER_POSTERIZE                "gimp-filter-posterize"
+#define GIMP_HELP_FILTER_RECURSIVE_TRANSFORM      "gimp-filter-recursive-transform"
 #define GIMP_HELP_FILTER_RED_EYE_REMOVAL          "gimp-filter-red-eye-removal"
 #define GIMP_HELP_FILTER_REINHARD_2005            "gimp-filter-reinhard-2005"
 #define GIMP_HELP_FILTER_RGB_CLIP                 "gimp-filter-rgb-clip"
@@ -412,11 +438,14 @@
 #define GIMP_HELP_FILTER_SATURATION               "gimp-filter-saturation"
 #define GIMP_HELP_FILTER_SEMI_FLATTEN             "gimp-filter-semi-flatten"
 #define GIMP_HELP_FILTER_SEPIA                    "gimp-filter-sepia"
+#define GIMP_HELP_FILTER_SHADOWS_HIGHLIGHTS       "gimp-filter-shadows-highlights"
 #define GIMP_HELP_FILTER_SHIFT                    "gimp-filter-shift"
 #define GIMP_HELP_FILTER_SINUS                    "gimp-filter-sinus"
 #define GIMP_HELP_FILTER_SLIC                     "gimp-filter-slic"
 #define GIMP_HELP_FILTER_SNN_MEAN                 "gimp-filter-snn-mean"
 #define GIMP_HELP_FILTER_SOFTGLOW                 "gimp-filter-softglow"
+#define GIMP_HELP_FILTER_SPHERIZE                 "gimp-filter-spherize"
+#define GIMP_HELP_FILTER_SPIRAL                   "gimp-filter-spiral"
 #define GIMP_HELP_FILTER_STRETCH_CONTRAST         "gimp-filter-stretch-contrast"
 #define GIMP_HELP_FILTER_STRETCH_CONTRAST_HSV     "gimp-filter-stretch-contrast-hsv"
 #define GIMP_HELP_FILTER_STRESS                   "gimp-filter-stress"
@@ -427,6 +456,7 @@
 #define GIMP_HELP_FILTER_TILE_SEAMLESS            "gimp-filter-tile-seamless"
 #define GIMP_HELP_FILTER_UNSHARP_MASK             "gimp-filter-unsharp-mask"
 #define GIMP_HELP_FILTER_VALUE_PROPAGATE          "gimp-filter-value-propagate"
+#define GIMP_HELP_FILTER_VARIABLE_BLUR            "gimp-filter-variable-blur"
 #define GIMP_HELP_FILTER_VIDEO_DEGRADATION        "gimp-filter-video-degradation"
 #define GIMP_HELP_FILTER_VIGNETTE                 "gimp-filter-vignette"
 #define GIMP_HELP_FILTER_WATERPIXELS              "gimp-filter-waterpixels"
@@ -547,8 +577,11 @@
 #define GIMP_HELP_BUFFER_CUT                      "gimp-buffer-cut"
 #define GIMP_HELP_BUFFER_COPY                     "gimp-buffer-copy"
 #define GIMP_HELP_BUFFER_PASTE                    "gimp-buffer-paste"
+#define GIMP_HELP_BUFFER_PASTE_IN_PLACE           "gimp-buffer-paste-in-place"
 #define GIMP_HELP_BUFFER_PASTE_INTO               "gimp-buffer-paste-into"
+#define GIMP_HELP_BUFFER_PASTE_INTO_IN_PLACE      "gimp-buffer-paste-into-in-place"
 #define GIMP_HELP_BUFFER_PASTE_AS_NEW_LAYER       "gimp-buffer-paste-as-new-layer"
+#define GIMP_HELP_BUFFER_PASTE_AS_NEW_LAYER_IN_PLACE "gimp-buffer-paste-as-new-layer-in-place"
 #define GIMP_HELP_BUFFER_PASTE_AS_NEW_IMAGE       "gimp-buffer-paste-as-new-image"
 #define GIMP_HELP_BUFFER_DELETE                   "gimp-buffer-delete"
 
@@ -586,31 +619,36 @@
 #define GIMP_HELP_TOOL_OPTIONS_RESET              "gimp-tool-options-reset"
 
 #define GIMP_HELP_ERRORS_DIALOG                   "gimp-errors-dialog"
+#define GIMP_HELP_ERRORS_HIGHLIGHT                "gimp-errors-highlight"
 #define GIMP_HELP_ERRORS_CLEAR                    "gimp-errors-clear"
 #define GIMP_HELP_ERRORS_SAVE                     "gimp-errors-save"
 #define GIMP_HELP_ERRORS_SELECT_ALL               "gimp-errors-select-all"
 
 #define GIMP_HELP_PREFS_DIALOG                    "gimp-prefs-dialog"
+#define GIMP_HELP_PREFS_SYSTEM_RESOURCES          "gimp-prefs-system-resources"
+#define GIMP_HELP_PREFS_DEBUGGING                 "gimp-prefs-debugging"
+#define GIMP_HELP_PREFS_COLOR_MANAGEMENT          "gimp-prefs-color-management"
+#define GIMP_HELP_PREFS_IMPORT_EXPORT             "gimp-prefs-import-export"
+#define GIMP_HELP_PREFS_PLAYGROUND                "gimp-prefs-playground"
+#define GIMP_HELP_PREFS_TOOL_OPTIONS              "gimp-prefs-tool-options"
 #define GIMP_HELP_PREFS_NEW_IMAGE                 "gimp-prefs-new-image"
 #define GIMP_HELP_PREFS_DEFAULT_GRID              "gimp-prefs-default-grid"
 #define GIMP_HELP_PREFS_INTERFACE                 "gimp-prefs-interface"
-#define GIMP_HELP_PREFS_HELP                      "gimp-prefs-help"
 #define GIMP_HELP_PREFS_THEME                     "gimp-prefs-theme"
 #define GIMP_HELP_PREFS_ICON_THEME                "gimp-prefs-icon-theme"
-#define GIMP_HELP_PREFS_TOOL_OPTIONS              "gimp-prefs-tool-options"
 #define GIMP_HELP_PREFS_TOOLBOX                   "gimp-prefs-toolbox"
 #define GIMP_HELP_PREFS_DIALOG_DEFAULTS           "gimp-prefs-dialog-defaults"
-#define GIMP_HELP_PREFS_INPUT_DEVICES             "gimp-prefs-input-devices"
-#define GIMP_HELP_PREFS_INPUT_CONTROLLERS         "gimp-prefs-input-controllers"
+#define GIMP_HELP_PREFS_HELP                      "gimp-prefs-help"
+#define GIMP_HELP_PREFS_DISPLAY                   "gimp-prefs-display"
+#define GIMP_HELP_PREFS_WINDOW_MANAGEMENT         "gimp-prefs-window-management"
+#define GIMP_HELP_PREFS_CANVAS_INTERACTION        "gimp-prefs-canvas-interaction"
+#define GIMP_HELP_PREFS_CANVAS_MODIFIERS          "gimp-prefs-canvas-modifiers"
 #define GIMP_HELP_PREFS_IMAGE_WINDOW              "gimp-prefs-image-window"
 #define GIMP_HELP_PREFS_IMAGE_WINDOW_APPEARANCE   "gimp-prefs-image-window-appearance"
 #define GIMP_HELP_PREFS_IMAGE_WINDOW_TITLE        "gimp-prefs-image-window-title"
-#define GIMP_HELP_PREFS_DISPLAY                   "gimp-prefs-display"
-#define GIMP_HELP_PREFS_MONITOR                   "gimp-prefs-monitor"
-#define GIMP_HELP_PREFS_COLOR_MANAGEMENT          "gimp-prefs-color-management"
-#define GIMP_HELP_PREFS_WINDOW_MANAGEMENT         "gimp-prefs-window-management"
-#define GIMP_HELP_PREFS_SESSION                   "gimp-prefs-session"
-#define GIMP_HELP_PREFS_SYSTEM_RESOURCES          "gimp-prefs-system-resources"
+#define GIMP_HELP_PREFS_IMAGE_WINDOW_SNAPPING     "gimp-prefs-image-window-snapping"
+#define GIMP_HELP_PREFS_INPUT_DEVICES             "gimp-prefs-input-devices"
+#define GIMP_HELP_PREFS_INPUT_CONTROLLERS         "gimp-prefs-input-controllers"
 #define GIMP_HELP_PREFS_FOLDERS                   "gimp-prefs-folders"
 #define GIMP_HELP_PREFS_FOLDERS_BRUSHES           "gimp-prefs-folders-brushes"
 #define GIMP_HELP_PREFS_FOLDERS_DYNAMICS          "gimp-prefs-folders-dynamics"
@@ -645,6 +683,16 @@
 #define GIMP_HELP_SAMPLE_POINT_DIALOG             "gimp-sample-point-dialog"
 #define GIMP_HELP_SAMPLE_POINT_SAMPLE_MERGED      "gimp-sample-point-sample-merged"
 
+#define GIMP_HELP_DASHBOARD_DIALOG                "gimp-dashboard-dialog"
+#define GIMP_HELP_DASHBOARD_GROUPS                "gimp-dashboard-groups"
+#define GIMP_HELP_DASHBOARD_UPDATE_INTERVAL       "gimp-dashboard-update-interval"
+#define GIMP_HELP_DASHBOARD_HISTORY_DURATION      "gimp-dashboard-history-duration"
+#define GIMP_HELP_DASHBOARD_LOG_RECORD            "gimp-dashboard-log-record"
+#define GIMP_HELP_DASHBOARD_LOG_ADD_MARKER        "gimp-dashboard-log-add-marker"
+#define GIMP_HELP_DASHBOARD_LOG_ADD_EMPTY_MARKER  "gimp-dashboard-log-add-empty-marker"
+#define GIMP_HELP_DASHBOARD_RESET                 "gimp-dashboard-reset"
+#define GIMP_HELP_DASHBOARD_LOW_SWAP_SPACE_WARNING "gimp-dashboard-low-swap-space-warning"
+
 #define GIMP_HELP_DOCK                            "gimp-dock"
 #define GIMP_HELP_DOCK_CLOSE                      "gimp-dock-close"
 #define GIMP_HELP_DOCK_IMAGE_MENU                 "gimp-dock-image-menu"
@@ -676,6 +724,7 @@
 #define GIMP_HELP_TEXT_EDITOR_DIALOG              "gimp-text-editor-dialog"
 #define GIMP_HELP_TIPS_DIALOG                     "gimp-tips-dialog"
 #define GIMP_HELP_UNDO_DIALOG                     "gimp-undo-dialog"
+#define GIMP_HELP_WELCOME_DIALOG                  "gimp-welcome-dialog"
 
 #define GIMP_HELP_EXPORT_DIALOG                   "gimp-export-dialog"
 #define GIMP_HELP_EXPORT_CONFIRM_DIALOG           "gimp-export-confirm-dialog"
@@ -683,6 +732,7 @@
 
 #define GIMP_HELP_WINDOWS_SHOW_DOCK               "gimp-windows-show-dock"
 #define GIMP_HELP_WINDOWS_HIDE_DOCKS              "gimp-windows-hide-docks"
+#define GIMP_HELP_WINDOWS_SHOW_TABS               "gimp-windows-show-tabs"
 #define GIMP_HELP_WINDOWS_TABS_POSITION           "gimp-windows-tabs-position"
 #define GIMP_HELP_WINDOWS_USE_SINGLE_WINDOW_MODE  "gimp-windows-use-single-window-mode"
 #define GIMP_HELP_WINDOWS_OPEN_RECENT_DOCK        "gimp-windows-open-recent-dock"
@@ -706,5 +756,9 @@
 #define GIMP_HELP_CONTROLLER_MOUSE                "gimp-controller-mouse"
 #define GIMP_HELP_CONTROLLER_WHEEL                "gimp-controller-wheel"
 
+#define GIMP_HELP_EXTENSIONS_DIALOG               "gimp-extensions-dialog"
+#define GIMP_HELP_EXTENSIONS_INSTALLED            "gimp-extensions-installed"
+#define GIMP_HELP_EXTENSIONS_SYSTEM               "gimp-extensions-system"
+#define GIMP_HELP_EXTENSIONS_INSTALL              "gimp-extensions-install"
 
 #endif /* __GIMP_HELP_IDS_H__ */
