@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __WIDGETS_TYPES_H__
@@ -57,6 +57,7 @@ typedef struct _GimpDeviceStatus             GimpDeviceStatus;
 typedef struct _GimpEditor                   GimpEditor;
 typedef struct _GimpErrorConsole             GimpErrorConsole;
 typedef struct _GimpToolOptionsEditor        GimpToolOptionsEditor;
+typedef struct _GimpDashboard                GimpDashboard;
 
 
 /*  GimpDataEditor widgets  */
@@ -68,6 +69,7 @@ typedef struct _GimpGradientEditor           GimpGradientEditor;
 typedef struct _GimpPaletteEditor            GimpPaletteEditor;
 typedef struct _GimpToolPresetEditor         GimpToolPresetEditor;
 
+
 /*  GimpImageEditor widgets  */
 
 typedef struct _GimpColormapEditor           GimpColormapEditor;
@@ -76,6 +78,7 @@ typedef struct _GimpHistogramEditor          GimpHistogramEditor;
 typedef struct _GimpImageEditor              GimpImageEditor;
 typedef struct _GimpSamplePointEditor        GimpSamplePointEditor;
 typedef struct _GimpSelectionEditor          GimpSelectionEditor;
+typedef struct _GimpSymmetryEditor           GimpSymmetryEditor;
 typedef struct _GimpUndoEditor               GimpUndoEditor;
 
 
@@ -85,7 +88,6 @@ typedef struct _GimpChannelTreeView          GimpChannelTreeView;
 typedef struct _GimpContainerBox             GimpContainerBox;
 typedef struct _GimpContainerComboBox        GimpContainerComboBox;
 typedef struct _GimpContainerEntry           GimpContainerEntry;
-typedef struct _GimpContainerGridView        GimpContainerGridView;
 typedef struct _GimpContainerIconView        GimpContainerIconView;
 typedef struct _GimpContainerTreeStore       GimpContainerTreeStore;
 typedef struct _GimpContainerTreeView        GimpContainerTreeView;
@@ -115,28 +117,38 @@ typedef struct _GimpToolEditor               GimpToolEditor;
 typedef struct _GimpBrushFactoryView         GimpBrushFactoryView;
 typedef struct _GimpDataFactoryView          GimpDataFactoryView;
 typedef struct _GimpDynamicsFactoryView      GimpDynamicsFactoryView;
+typedef struct _GimpFontFactoryView          GimpFontFactoryView;
 typedef struct _GimpPatternFactoryView       GimpPatternFactoryView;
 typedef struct _GimpToolPresetFactoryView    GimpToolPresetFactoryView;
 
+
 /*  menus  */
 
 typedef struct _GimpAction                   GimpAction;
 typedef struct _GimpActionFactory            GimpActionFactory;
 typedef struct _GimpActionGroup              GimpActionGroup;
+typedef struct _GimpDoubleAction             GimpDoubleAction;
 typedef struct _GimpEnumAction               GimpEnumAction;
 typedef struct _GimpMenuFactory              GimpMenuFactory;
-typedef struct _GimpPlugInAction             GimpPlugInAction;
+typedef struct _GimpProcedureAction          GimpProcedureAction;
 typedef struct _GimpStringAction             GimpStringAction;
 typedef struct _GimpUIManager                GimpUIManager;
 
 
+/*  file dialogs  */
+
+typedef struct _GimpExportDialog             GimpExportDialog;
+typedef struct _GimpFileDialog               GimpFileDialog;
+typedef struct _GimpOpenDialog               GimpOpenDialog;
+typedef struct _GimpSaveDialog               GimpSaveDialog;
+
+
 /*  misc dialogs  */
 
 typedef struct _GimpColorDialog              GimpColorDialog;
+typedef struct _GimpCriticalDialog           GimpCriticalDialog;
 typedef struct _GimpErrorDialog              GimpErrorDialog;
-typedef struct _GimpFileDialog               GimpFileDialog;
 typedef struct _GimpMessageDialog            GimpMessageDialog;
-typedef struct _GimpProfileChooserDialog     GimpProfileChooserDialog;
 typedef struct _GimpProgressDialog           GimpProgressDialog;
 typedef struct _GimpTextEditor               GimpTextEditor;
 typedef struct _GimpViewableDialog           GimpViewableDialog;
@@ -154,27 +166,37 @@ typedef struct _GimpPdbDialog                GimpPdbDialog;
 
 /*  misc widgets  */
 
+typedef struct _GimpAccelLabel               GimpAccelLabel;
 typedef struct _GimpActionEditor             GimpActionEditor;
 typedef struct _GimpActionView               GimpActionView;
 typedef struct _GimpBlobEditor               GimpBlobEditor;
+typedef struct _GimpBufferSourceBox          GimpBufferSourceBox;
+typedef struct _GimpCircle                   GimpCircle;
 typedef struct _GimpColorBar                 GimpColorBar;
 typedef struct _GimpColorDisplayEditor       GimpColorDisplayEditor;
 typedef struct _GimpColorFrame               GimpColorFrame;
+typedef struct _GimpColorHistory             GimpColorHistory;
+typedef struct _GimpColormapSelection        GimpColormapSelection;
 typedef struct _GimpColorPanel               GimpColorPanel;
 typedef struct _GimpComboTagEntry            GimpComboTagEntry;
+typedef struct _GimpCompressionComboBox      GimpCompressionComboBox;
 typedef struct _GimpControllerEditor         GimpControllerEditor;
 typedef struct _GimpControllerList           GimpControllerList;
 typedef struct _GimpCurveView                GimpCurveView;
 typedef struct _GimpDashEditor               GimpDashEditor;
 typedef struct _GimpDeviceEditor             GimpDeviceEditor;
 typedef struct _GimpDeviceInfoEditor         GimpDeviceInfoEditor;
+typedef struct _GimpDial                     GimpDial;
 typedef struct _GimpDynamicsOutputEditor     GimpDynamicsOutputEditor;
+typedef struct _GimpExtensionDetails         GimpExtensionDetails;
+typedef struct _GimpExtensionList            GimpExtensionList;
 typedef struct _GimpFgBgEditor               GimpFgBgEditor;
 typedef struct _GimpFgBgView                 GimpFgBgView;
 typedef struct _GimpFileProcView             GimpFileProcView;
 typedef struct _GimpFillEditor               GimpFillEditor;
 typedef struct _GimpGridEditor               GimpGridEditor;
 typedef struct _GimpHandleBar                GimpHandleBar;
+typedef struct _GimpHighlightableButton      GimpHighlightableButton;
 typedef struct _GimpHistogramBox             GimpHistogramBox;
 typedef struct _GimpHistogramView            GimpHistogramView;
 typedef struct _GimpIconPicker               GimpIconPicker;
@@ -185,12 +207,24 @@ typedef struct _GimpImagePropView            GimpImagePropView;
 typedef struct _GimpLanguageComboBox         GimpLanguageComboBox;
 typedef struct _GimpLanguageEntry            GimpLanguageEntry;
 typedef struct _GimpLanguageStore            GimpLanguageStore;
+typedef struct _GimpLayerModeBox             GimpLayerModeBox;
+typedef struct _GimpLayerModeComboBox        GimpLayerModeComboBox;
 typedef struct _GimpMessageBox               GimpMessageBox;
+typedef struct _GimpMeter                    GimpMeter;
+typedef struct _GimpModifiersEditor          GimpModifiersEditor;
 typedef struct _GimpOverlayBox               GimpOverlayBox;
+typedef struct _GimpPickableButton           GimpPickableButton;
+typedef struct _GimpPickablePopup            GimpPickablePopup;
+typedef struct _GimpPivotSelector            GimpPivotSelector;
+typedef struct _GimpPlugInView               GimpPlugInView;
+typedef struct _GimpPolar                    GimpPolar;
+typedef struct _GimpPopup                    GimpPopup;
+typedef struct _GimpPrefsBox                 GimpPrefsBox;
 typedef struct _GimpProgressBox              GimpProgressBox;
 typedef struct _GimpScaleButton              GimpScaleButton;
 typedef struct _GimpSettingsBox              GimpSettingsBox;
 typedef struct _GimpSettingsEditor           GimpSettingsEditor;
+typedef struct _GimpShortcutButton           GimpShortcutButton;
 typedef struct _GimpSizeBox                  GimpSizeBox;
 typedef struct _GimpStrokeEditor             GimpStrokeEditor;
 typedef struct _GimpTagEntry                 GimpTagEntry;
@@ -198,6 +232,7 @@ typedef struct _GimpTagPopup                 GimpTagPopup;
 typedef struct _GimpTemplateEditor           GimpTemplateEditor;
 typedef struct _GimpTextStyleEditor          GimpTextStyleEditor;
 typedef struct _GimpThumbBox                 GimpThumbBox;
+typedef struct _GimpToolButton               GimpToolButton;
 typedef struct _GimpToolPalette              GimpToolPalette;
 typedef struct _GimpTranslationStore         GimpTranslationStore;
 typedef struct _GimpWindow                   GimpWindow;
@@ -226,6 +261,7 @@ typedef struct _GimpViewRendererVectors      GimpViewRendererVectors;
 
 /*  cell renderers  */
 
+typedef struct _GimpCellRendererButton       GimpCellRendererButton;
 typedef struct _GimpCellRendererDashes       GimpCellRendererDashes;
 typedef struct _GimpCellRendererViewable     GimpCellRendererViewable;
 
@@ -251,19 +287,22 @@ typedef struct _GimpSessionManaged           GimpSessionManaged;
 /*  structs  */
 
 typedef struct _GimpActionEntry              GimpActionEntry;
+typedef struct _GimpDoubleActionEntry        GimpDoubleActionEntry;
 typedef struct _GimpEnumActionEntry          GimpEnumActionEntry;
-typedef struct _GimpPlugInActionEntry        GimpPlugInActionEntry;
+typedef struct _GimpProcedureActionEntry     GimpProcedureActionEntry;
 typedef struct _GimpRadioActionEntry         GimpRadioActionEntry;
 typedef struct _GimpStringActionEntry        GimpStringActionEntry;
 typedef struct _GimpToggleActionEntry        GimpToggleActionEntry;
 
 typedef struct _GimpDialogFactoryEntry       GimpDialogFactoryEntry;
 
+typedef struct _GimpDashboardLogParams       GimpDashboardLogParams;
+
 
 /*  function types  */
 
 typedef GtkWidget * (* GimpDialogRestoreFunc)        (GimpDialogFactory *factory,
-                                                      GdkScreen         *screen,
+                                                      GdkMonitor        *monitor,
                                                       GimpSessionInfo   *info);
 typedef void        (* GimpActionGroupSetupFunc)     (GimpActionGroup   *group);
 typedef void        (* GimpActionGroupUpdateFunc)    (GimpActionGroup   *group,
@@ -276,22 +315,12 @@ typedef void        (* GimpMenuPositionFunc)         (GtkMenu           *menu,
                                                       gint              *x,
                                                       gint              *y,
                                                       gpointer           data);
-typedef gboolean    (* GimpPanedBoxDroppedFunc)      (GtkWidget         *source,
+typedef gboolean    (* GimpPanedBoxDroppedFunc)      (GtkWidget         *notebook,
+                                                      GtkWidget         *child,
                                                       gint               insert_index,
                                                       gpointer           data);
 
-
-/*  temp hack as replacement for GdkSegment  */
-
-typedef struct _GimpSegment GimpSegment;
-
-struct _GimpSegment
-{
-  gint x1;
-  gint y1;
-  gint x2;
-  gint y2;
-};
+typedef GtkWidget * (* GimpToolOptionsGUIFunc)       (GimpToolOptions   *tool_options);
 
 
 #endif /* __WIDGETS_TYPES_H__ */
