@@ -27,6 +27,8 @@
 #include "libgimpbase/gimpbase.h"
 #include "libgimpcolor/gimpcolor.h"
 
+#include "config/gimpgeglconfig.h"
+
 #include "core/core-types.h"
 
 #include "gegl/gimp-babl-compat.h"
@@ -73,6 +75,33 @@
 
 #include "gimp-intl.h"
 
+typedef void (* CompressTileFunc) (GeglRectangle  *tile_rect,
+                                   guchar         *tile_data,
+                                   const Babl     *format,
+                                   guchar         *out_data,
+                                   gint            out_data_max_len,
+                                   gint           *lenptr);
+
+/* Per thread data for xcf_save_tile_rle */
+typedef struct
+{
+  /* Common to all jobs. */
+  GeglBuffer       *buffer;
+  gint              file_version;
+  gint              max_out_data_len;
+  CompressTileFunc  compress;
+
+  /* Job specific. */
+  gint              tile;
+  gint              batch_size;
+
+  /* Temp data to avoid too many allocations. */
+  guchar           *tile_data;
+
+  /* Return data. */
+  guchar           *out_data;
+  gint              out_data_len[XCF_TILE_SAVE_BATCH_SIZE];
+} XcfJobData;
 
 static gboolean xcf_save_image_props   (XcfInfo           *info,
                                         GimpImage         *image,
@@ -85,6 +114,10 @@ static gboolean xcf_save_channel_props (XcfInfo           *info,
                                         GimpImage         *image,
                                         GimpChannel       *channel,
                                         GError           **error);
+static gboolean xcf_save_path_props    (XcfInfo           *info,
+                                        GimpImage         *image,
+                                        GimpVectors       *vectors,
+                                        GError           **error);
 static gboolean xcf_save_prop          (XcfInfo           *info,
                                         GimpImage         *image,
                                         PropType           prop_type,
@@ -98,10 +131,16 @@ static gboolean xcf_save_channel       (XcfInfo           *info,
                                         GimpImage         *image,
                                         GimpChannel       *channel,
                                         GError           **error);
+static gboolean xcf_save_path          (XcfInfo           *info,
+                                        GimpImage         *image,
+                                        GimpVectors       *vectors,
+                                        GError           **error);
 static gboolean xcf_save_buffer        (XcfInfo           *info,
+                                        GimpImage         *image,
                                         GeglBuffer        *buffer,
                                         GError           **error);
 static gboolean xcf_save_level         (XcfInfo           *info,
+                                        GimpImage         *image,
                                         GeglBuffer        *buffer,
                                         GError           **error);
 static gboolean xcf_save_tile          (XcfInfo           *info,
@@ -109,17 +148,24 @@ static gboolean xcf_save_tile          (XcfInfo           *info,
                                         GeglRectangle     *tile_rect,
                                         const Babl        *format,
                                         GError           **error);
-static gboolean xcf_save_tile_rle      (XcfInfo           *info,
-                                        GeglBuffer        *buffer,
-                                        GeglRectangle     *tile_rect,
+static void     xcf_save_free_job_data (XcfJobData        *data);
+static gint     xcf_save_sort_job_data (XcfJobData        *data1,
+                                        XcfJobData        *data2,
+                                        gpointer           user_data);
+static void     xcf_save_tile_parallel (XcfJobData        *job_data,
+                                        GAsyncQueue       *queue);
+static void     xcf_save_tile_rle      (GeglRectangle     *tile_rect,
+                                        guchar            *tile_data,
                                         const Babl        *format,
                                         guchar            *rlebuf,
-                                        GError           **error);
-static gboolean xcf_save_tile_zlib     (XcfInfo           *info,
-                                        GeglBuffer        *buffer,
-                                        GeglRectangle     *tile_rect,
+                                        gint               rlebuf_max_len,
+                                        gint              *lenptr);
+static void     xcf_save_tile_zlib     (GeglRectangle     *tile_rect,
+                                        guchar            *tile_data,
                                         const Babl        *format,
-                                        GError           **error);
+                                        guchar            *zlib_data,
+                                        gint               zlib_data_max_len,
+                                        gint              *lenptr);
 static gboolean xcf_save_parasite      (XcfInfo           *info,
                                         GimpParasite      *parasite,
                                         GError           **error);
@@ -129,7 +175,7 @@ static gboolean xcf_save_parasite_list (XcfInfo           *info,
 static gboolean xcf_save_old_paths     (XcfInfo           *info,
                                         GimpImage         *image,
                                         GError           **error);
-static gboolean xcf_save_vectors       (XcfInfo           *info,
+static gboolean xcf_save_old_vectors   (XcfInfo           *info,
                                         GimpImage         *image,
                                         GError           **error);
 
@@ -224,16 +270,19 @@ xcf_save_image (XcfInfo    *info,
 {
   GList   *all_layers;
   GList   *all_channels;
+  GList   *all_paths = NULL;
   GList   *list;
   goffset  saved_pos;
   goffset  offset;
   guint32  value;
   guint    n_layers;
   guint    n_channels;
+  guint    n_paths  = 0;
   guint    progress = 0;
   guint    max_progress;
   gchar    version_tag[16];
-  GError  *tmp_error = NULL;
+  gboolean write_paths = FALSE;
+  GError  *tmp_error   = NULL;
 
   /* write out the tag information for the image */
   if (info->file_version > 0)
@@ -264,6 +313,9 @@ xcf_save_image (XcfInfo    *info,
       xcf_write_int32_check_error (info, &value, 1);
     }
 
+  if (info->file_version >= 18)
+    write_paths = TRUE;
+
   /* determine the number of layers and channels in the image */
   all_layers   = gimp_image_get_layer_list (image);
   all_channels = gimp_image_get_channel_list (image);
@@ -277,7 +329,13 @@ xcf_save_image (XcfInfo    *info,
   n_layers   = (guint) g_list_length (all_layers);
   n_channels = (guint) g_list_length (all_channels);
 
-  max_progress = 1 + n_layers + n_channels;
+  if (write_paths)
+    {
+      all_paths = gimp_image_get_vectors_list (image);
+      n_paths   = (guint) g_list_length (all_paths);
+    }
+
+  max_progress = 1 + n_layers + n_channels + n_paths;
 
   /* write the property information for the image */
   xcf_check_error (xcf_save_image_props (info, image, error));
@@ -288,7 +346,9 @@ xcf_save_image (XcfInfo    *info,
   saved_pos = info->cp;
 
   /* write an empty offset table */
-  xcf_write_zero_offset_check_error (info, n_layers + n_channels + 2);
+  xcf_write_zero_offset_check_error (info,
+                                     n_layers + n_channels + 2 +
+                                     (write_paths ? n_paths + 1 : 0));
 
   /* 'offset' is where we will write the next layer or channel */
   offset = info->cp;
@@ -344,12 +404,44 @@ xcf_save_image (XcfInfo    *info,
       xcf_progress_update (info);
     }
 
+  if (write_paths)
+    {
+      /* skip a '0' in the offset table to indicate the end of the channel
+       * offsets
+       */
+      saved_pos += info->bytes_per_offset;
+
+      for (list = all_paths; list; list = g_list_next (list))
+        {
+          GimpVectors *vectors = list->data;
+
+          /* seek back to the next slot in the offset table and write the
+           * offset of the channel
+           */
+          xcf_check_error (xcf_seek_pos (info, saved_pos, error));
+          xcf_write_offset_check_error (info, &offset, 1);
+
+          /* remember the next slot in the offset table */
+          saved_pos = info->cp;
+
+          /* seek to the channel offset and save the channel */
+          xcf_check_error (xcf_seek_pos (info, offset, error));
+          xcf_check_error (xcf_save_path (info, image, vectors, error));
+
+          /* the next channels's offset is after the channel we just wrote */
+          offset = info->cp;
+
+          xcf_progress_update (info);
+        }
+    }
+
   /* there is already a '0' at the end of the offset table to indicate
    * the end of the channel offsets
    */
 
   g_list_free (all_layers);
   g_list_free (all_channels);
+  g_list_free (all_paths);
 
   return ! g_output_stream_is_closed (info->output);
 }
@@ -408,7 +500,8 @@ xcf_save_image_props (XcfInfo    *info,
   if (unit < gimp_unit_get_number_of_built_in_units ())
     xcf_check_error (xcf_save_prop (info, image, PROP_UNIT, error, unit));
 
-  if (gimp_container_get_n_children (gimp_image_get_vectors (image)) > 0)
+  if (gimp_container_get_n_children (gimp_image_get_vectors (image)) > 0 &&
+      info->file_version < 18)
     {
       if (gimp_vectors_compat_is_compatible (image))
         xcf_check_error (xcf_save_prop (info, image, PROP_PATHS, error));
@@ -714,6 +807,60 @@ xcf_save_channel_props (XcfInfo      *info,
   return TRUE;
 }
 
+static gboolean
+xcf_save_path_props (XcfInfo      *info,
+                     GimpImage    *image,
+                     GimpVectors  *vectors,
+                     GError      **error)
+{
+  GimpParasiteList *parasites;
+
+  if (g_list_find (gimp_image_get_selected_vectors (image), vectors))
+    xcf_check_error (xcf_save_prop (info, image, PROP_SELECTED_PATH, error));
+
+  xcf_check_error (xcf_save_prop (info, image, PROP_VISIBLE, error,
+                                  gimp_item_get_visible (GIMP_ITEM (vectors))));
+  xcf_check_error (xcf_save_prop (info, image, PROP_COLOR_TAG, error,
+                                  gimp_item_get_color_tag (GIMP_ITEM (vectors))));
+  xcf_check_error (xcf_save_prop (info, image, PROP_LOCK_CONTENT, error,
+                                  gimp_item_get_lock_content (GIMP_ITEM (vectors))));
+  xcf_check_error (xcf_save_prop (info, image, PROP_LOCK_POSITION, error,
+                                  gimp_item_get_lock_position (GIMP_ITEM (vectors))));
+
+  xcf_check_error (xcf_save_prop (info, image, PROP_TATTOO, error,
+                                  gimp_item_get_tattoo (GIMP_ITEM (vectors))));
+
+  parasites = gimp_item_get_parasites (GIMP_ITEM (vectors));
+
+  if (gimp_parasite_list_length (parasites) > 0)
+    {
+      xcf_check_error (xcf_save_prop (info, image, PROP_PARASITES, error,
+                                      parasites));
+    }
+
+#if 0
+  for (iter = info->vectors_sets; iter; iter = iter->next)
+    {
+      GimpItemList *set = iter->data;
+
+      if (! gimp_item_list_is_pattern (set, NULL))
+        {
+          GList *items = gimp_item_list_get_items (set, NULL);
+
+          if (g_list_find (items, GIMP_ITEM (vectors)))
+            xcf_check_error (xcf_save_prop (info, image, PROP_ITEM_SET_ITEM, error,
+                                            g_list_position (info->layer_sets, iter)));
+
+          g_list_free (items);
+        }
+    }
+#endif
+
+  xcf_check_error (xcf_save_prop (info, image, PROP_END, error));
+
+  return TRUE;
+}
+
 static gboolean
 xcf_save_prop (XcfInfo    *info,
                GimpImage  *image,
@@ -753,6 +900,7 @@ xcf_save_prop (XcfInfo    *info,
 
     case PROP_ACTIVE_LAYER:
     case PROP_ACTIVE_CHANNEL:
+    case PROP_SELECTED_PATH:
     case PROP_SELECTION:
     case PROP_GROUP_ITEM:
       size = 0;
@@ -1352,7 +1500,7 @@ xcf_save_prop (XcfInfo    *info,
 
         base = info->cp;
 
-        xcf_check_error (xcf_save_vectors (info, image, error));
+        xcf_check_error (xcf_save_old_vectors (info, image, error));
 
         size = info->cp - base;
 
@@ -1518,7 +1666,7 @@ xcf_save_layer (XcfInfo    *info,
   /* write a zero layer mask offset */
   xcf_write_zero_offset_check_error (info, 1);
 
-  xcf_check_error (xcf_save_buffer (info,
+  xcf_check_error (xcf_save_buffer (info, image,
                                     gimp_drawable_get_buffer (GIMP_DRAWABLE (layer)),
                                     error));
 
@@ -1581,7 +1729,7 @@ xcf_save_channel (XcfInfo      *info,
   offset = info->cp + info->bytes_per_offset;
   xcf_write_offset_check_error (info, &offset, 1);
 
-  xcf_check_error (xcf_save_buffer (info,
+  xcf_check_error (xcf_save_buffer (info, image,
                                     gimp_drawable_get_buffer (GIMP_DRAWABLE (channel)),
                                     error));
 
@@ -1607,6 +1755,7 @@ xcf_calc_levels (gint size,
 
 static gboolean
 xcf_save_buffer (XcfInfo     *info,
+                 GimpImage   *image,
                  GeglBuffer  *buffer,
                  GError     **error)
 {
@@ -1661,7 +1810,7 @@ xcf_save_buffer (XcfInfo     *info,
       if (i == 0)
         {
           /* write out the level. */
-          xcf_check_error (xcf_save_level (info, buffer, error));
+          xcf_check_error (xcf_save_level (info, image, buffer, error));
         }
       else
         {
@@ -1696,6 +1845,7 @@ xcf_save_buffer (XcfInfo     *info,
 
 static gboolean
 xcf_save_level (XcfInfo     *info,
+                GimpImage   *image,
                 GeglBuffer  *buffer,
                 GError     **error)
 {
@@ -1711,10 +1861,12 @@ xcf_save_level (XcfInfo     *info,
   gint        n_tile_rows;
   gint        n_tile_cols;
   guint       ntiles;
-  gint        i;
-  guchar     *rlebuf    = NULL;
+  gint        num_processors;
+  gint        i, j, k;
   GError     *tmp_error = NULL;
 
+  num_processors = GIMP_GEGL_CONFIG (image->gimp->config)->num_processors;
+
   format = gegl_buffer_get_format (buffer);
 
   width  = gegl_buffer_get_width (buffer);
@@ -1733,12 +1885,6 @@ xcf_save_level (XcfInfo     *info,
   max_data_length = XCF_TILE_WIDTH * XCF_TILE_HEIGHT * bpp *
                     XCF_TILE_MAX_DATA_LENGTH_FACTOR /* = 1.5, currently */;
 
-  /* allocate a temporary buffer to store the rle data before it is
-   * written to disk
-   */
-  if (info->compression == COMPRESS_RLE)
-    rlebuf = g_alloca (max_data_length);
-
   n_tile_rows = gimp_gegl_buffer_get_n_tile_rows (buffer, XCF_TILE_HEIGHT);
   n_tile_cols = gimp_gegl_buffer_get_n_tile_cols (buffer, XCF_TILE_WIDTH);
 
@@ -1762,51 +1908,209 @@ xcf_save_level (XcfInfo     *info,
   /* 'offset' is where we will write the next tile */
   offset = info->cp;
 
-  for (i = 0; i < ntiles; i++)
+  if (info->compression == COMPRESS_RLE ||
+      info->compression == COMPRESS_ZLIB)
     {
-      GeglRectangle rect;
+      /* parallel implementation */
+      XcfJobData  *job_data;
+      guchar      *switch_out_data;
+      gint         out_data_len[XCF_TILE_SAVE_BATCH_SIZE];
 
-      /* store the offset in the table and increment the next pointer */
-      *next_offset++ = offset;
+      GThreadPool *pool;
+      GAsyncQueue *queue;
+      gint         num_tasks = num_processors * 2;
+      gint         tile_size = XCF_TILE_WIDTH * XCF_TILE_HEIGHT * bpp;
+      gint         out_data_max_size;
+      gint         next_tile = 0;
 
-      gimp_gegl_buffer_get_tile_rect (buffer,
-                                      XCF_TILE_WIDTH, XCF_TILE_HEIGHT,
-                                      i, &rect);
+      out_data_max_size = tile_size * XCF_TILE_MAX_DATA_LENGTH_FACTOR;
+      /* Prepare an additional out_data to quickly switch. */
+      switch_out_data   = g_malloc (out_data_max_size * XCF_TILE_SAVE_BATCH_SIZE);
 
-      /* write out the tile. */
-      switch (info->compression)
-        {
-        case COMPRESS_NONE:
-          xcf_check_error (xcf_save_tile (info, buffer, &rect, format,
-                                          error));
-          break;
-        case COMPRESS_RLE:
-          xcf_check_error (xcf_save_tile_rle (info, buffer, &rect, format,
-                                              rlebuf, error));
-          break;
-        case COMPRESS_ZLIB:
-          xcf_check_error (xcf_save_tile_zlib (info, buffer, &rect, format,
-                                               error));
-          break;
-        case COMPRESS_FRACTAL:
-          g_warning ("xcf: fractal compression unimplemented");
-          g_free (offset_table);
-          return FALSE;
-        }
-
-      /* make sure the on-disk tile data didn't end up being too big.
-       * xcf_load_level() would refuse to load the file if it did.
+      /* The free function passed to the queue and thread pool will likely never
+       * be used. It would mean the thread pool is unfinidhed or the result
+       * queue still has data which would mean we had to interrupt the save,
+       * i.e. there is a bug in our code.
        */
-      if (info->cp < offset || info->cp - offset > max_data_length)
+      queue = g_async_queue_new_full ((GDestroyNotify) xcf_save_free_job_data);
+      pool  = g_thread_pool_new_full ((GFunc) xcf_save_tile_parallel,
+                                      queue,
+                                      (GDestroyNotify) xcf_save_free_job_data,
+                                      num_processors, TRUE, NULL);
+
+      i = 0;
+      /* We push more tasks than there are threads, ensuring threads always have
+       * something to do!
+       */
+      for (j = 0; j < num_tasks && i < ntiles; j++)
         {
-          g_message ("xcf: invalid tile data length: %" G_GOFFSET_FORMAT,
-                     info->cp - offset);
-          g_free (offset_table);
-          return FALSE;
+          job_data = g_malloc (sizeof (XcfJobData ));
+          job_data->buffer        = buffer;
+          job_data->file_version  = info->file_version;
+          job_data->max_out_data_len = out_data_max_size;
+          job_data->compress      = (info->compression == COMPRESS_RLE) ?
+                                      xcf_save_tile_rle : xcf_save_tile_zlib;
+          job_data->tile_data     = g_malloc (tile_size);
+          job_data->out_data      = g_malloc (out_data_max_size * XCF_TILE_SAVE_BATCH_SIZE);
+
+          job_data->tile       = i;
+          job_data->batch_size = MIN (XCF_TILE_SAVE_BATCH_SIZE, ntiles - i);
+          i += job_data->batch_size;
+
+          g_thread_pool_push (pool, job_data, NULL);
         }
 
-      /* the next tile's offset is after the tile we just wrote */
-      offset = info->cp;
+      /* Continue pushing tasks and writing tasks as long as we have tiles to
+       * process.
+       */
+      while (i < ntiles)
+        {
+          while ((job_data = g_async_queue_pop (queue)))
+            {
+              if (next_tile == job_data->tile)
+                {
+                  guchar *tmp_out_data;
+                  gint    batch_size;
+
+                  tmp_out_data = job_data->out_data;
+                  job_data->out_data = switch_out_data;
+                  switch_out_data = tmp_out_data;
+
+                  batch_size = job_data->batch_size;
+
+                  for (k = 0; k < batch_size; k++)
+                    out_data_len[k] = job_data->out_data_len[k];
+
+                  /* First immediately push a new task for the thread pool,
+                   * ensuring it always has work to do.
+                   */
+                  job_data->tile       = i;
+                  job_data->batch_size = MIN (XCF_TILE_SAVE_BATCH_SIZE, ntiles - i);
+                  i += job_data->batch_size;
+
+                  g_thread_pool_push (pool, job_data, NULL);
+
+                  /* Now write the data. */
+                  for (k = 0; k < batch_size; k++)
+                    {
+                      *next_offset++ = offset;
+                      xcf_write_int8_check_error (info,
+                                                  switch_out_data + out_data_max_size * k,
+                                                  out_data_len[k]);
+                      if (info->cp < offset || info->cp - offset > max_data_length)
+                        {
+                          g_message ("xcf: invalid tile data length: %" G_GOFFSET_FORMAT,
+                                     info->cp - offset);
+                          g_thread_pool_free (pool, TRUE, TRUE);
+                          g_async_queue_unref (queue);
+                          g_free (offset_table);
+                          return FALSE;
+                        }
+                      offset = info->cp;
+                    }
+                  next_tile += batch_size;
+                  break;
+                }
+              else
+                {
+                  g_async_queue_push_sorted (queue, job_data,
+                                             (GCompareDataFunc) xcf_save_sort_job_data,
+                                             NULL);
+                }
+            }
+        }
+      g_free (switch_out_data);
+
+      /* Finally wait for all remaining tasks to write. */
+      while ((job_data = g_async_queue_pop (queue)))
+        {
+          if (next_tile == job_data->tile)
+            {
+              gboolean done = FALSE;
+
+              for (k = 0; k < job_data->batch_size; k++)
+                {
+                  *next_offset++ = offset;
+                  xcf_write_int8_check_error (info,
+                                              job_data->out_data + out_data_max_size * k,
+                                              job_data->out_data_len[k]);
+                  if (info->cp < offset || info->cp - offset > max_data_length)
+                    {
+                      g_message ("xcf: invalid tile data length: %" G_GOFFSET_FORMAT,
+                                 info->cp - offset);
+                      g_thread_pool_free (pool, TRUE, TRUE);
+                      g_async_queue_unref (queue);
+                      g_free (offset_table);
+                      return FALSE;
+                    }
+                  offset = info->cp;
+                }
+              next_tile += job_data->batch_size;
+
+              if (job_data->tile + job_data->batch_size >= ntiles)
+                done = TRUE;
+
+              xcf_save_free_job_data (job_data);
+
+              if (done)
+                break;
+            }
+          else
+            {
+              g_async_queue_push_sorted (queue, job_data,
+                                         (GCompareDataFunc) xcf_save_sort_job_data,
+                                         NULL);
+            }
+        }
+
+      g_thread_pool_free (pool, FALSE, TRUE);
+      g_async_queue_unref (queue);
+    }
+  else
+    {
+      /* non parallel implementation */
+      for (i = 0; i < ntiles; i++)
+        {
+          GeglRectangle rect;
+
+          /* store the offset in the table and increment the next pointer */
+          *next_offset++ = offset;
+
+          gimp_gegl_buffer_get_tile_rect (buffer,
+                                          XCF_TILE_WIDTH, XCF_TILE_HEIGHT,
+                                          i, &rect);
+
+          /* write out the tile. */
+          switch (info->compression)
+            {
+            case COMPRESS_NONE:
+              xcf_check_error (xcf_save_tile (info, buffer, &rect, format,
+                                              error));
+              break;
+            case COMPRESS_FRACTAL:
+              g_warning ("xcf: fractal compression unimplemented");
+              g_free (offset_table);
+              return FALSE;
+            default:
+              g_warning ("xcf: unsupported compression algorithm");
+              g_free (offset_table);
+              return FALSE;
+            }
+
+          /* make sure the on-disk tile data didn't end up being too big.
+           * xcf_load_level() would refuse to load the file if it did.
+           */
+          if (info->cp < offset || info->cp - offset > max_data_length)
+            {
+              g_message ("xcf: invalid tile data length: %" G_GOFFSET_FORMAT,
+                         info->cp - offset);
+              g_free (offset_table);
+              return FALSE;
+            }
+
+          /* the next tile's offset is after the tile we just wrote */
+          offset = info->cp;
+        }
     }
 
   /* seek back to the offset table and write it  */
@@ -1851,31 +2155,81 @@ xcf_save_tile (XcfInfo        *info,
   return TRUE;
 }
 
-static gboolean
-xcf_save_tile_rle (XcfInfo        *info,
-                   GeglBuffer     *buffer,
-                   GeglRectangle  *tile_rect,
+static void
+xcf_save_free_job_data (XcfJobData *data)
+{
+  g_free (data->out_data);
+  g_free (data->tile_data);
+  g_free (data);
+}
+
+static gint
+xcf_save_sort_job_data (XcfJobData *data1,
+                        XcfJobData *data2,
+                        gpointer    user_data)
+{
+  if (data1->tile < data2->tile)
+    return -1;
+  else if (data1->tile > data2->tile)
+    return 1;
+  else
+    return 0;
+}
+
+static void
+xcf_save_tile_parallel (XcfJobData  *job_data,
+                        GAsyncQueue *queue)
+{
+  const Babl    *format;
+  GeglRectangle  tile_rect;
+  gint           bpp;
+
+  format = gegl_buffer_get_format (job_data->buffer);
+  bpp    = babl_format_get_bytes_per_pixel (format);
+
+  for (gint i = 0; i < job_data->batch_size; ++i)
+    {
+      gimp_gegl_buffer_get_tile_rect (job_data->buffer,
+                                      XCF_TILE_WIDTH,
+                                      XCF_TILE_HEIGHT,
+                                      job_data->tile + i,
+                                      &tile_rect);
+      /* only single thread can create tile data when cache miss */
+      gegl_buffer_get (job_data->buffer, &tile_rect, 1.0, format,
+                       job_data->tile_data,
+                       GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
+
+      if (job_data->file_version >= 12)
+        {
+          gint n_components = babl_format_get_n_components (format);
+          gint tile_size    = bpp * tile_rect.width * tile_rect.height;
+
+          xcf_write_to_be (bpp / n_components, job_data->tile_data,
+                           tile_size / bpp * n_components);
+        }
+
+      job_data->compress (&tile_rect, job_data->tile_data, format,
+                          job_data->out_data + job_data->max_out_data_len * i,
+                          job_data->max_out_data_len,
+                          job_data->out_data_len + i);
+    }
+
+  g_async_queue_push_sorted (queue, job_data,
+                             (GCompareDataFunc) xcf_save_sort_job_data,
+                             NULL);
+}
+
+static void
+xcf_save_tile_rle (GeglRectangle  *tile_rect,
+                   guchar         *tile_data,
                    const Babl     *format,
                    guchar         *rlebuf,
-                   GError        **error)
+                   gint            rlebuf_max_len,
+                   gint           *lenptr)
 {
-  gint    bpp       = babl_format_get_bytes_per_pixel (format);
-  gint    tile_size = bpp * tile_rect->width * tile_rect->height;
-  guchar *tile_data = g_alloca (tile_size);
-  gint    len       = 0;
-  gint    i, j;
-  GError *tmp_error  = NULL;
-
-  gegl_buffer_get (buffer, tile_rect, 1.0, format, tile_data,
-                   GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
-
-  if (info->file_version >= 12)
-    {
-      gint n_components = babl_format_get_n_components (format);
-
-      xcf_write_to_be (bpp / n_components, tile_data,
-                       tile_size / bpp * n_components);
-    }
+  gint bpp = babl_format_get_bytes_per_pixel (format);
+  gint len = 0;
+  gint i, j;
 
   for (i = 0; i < bpp; i++)
     {
@@ -1980,38 +2334,25 @@ xcf_save_tile_rle (XcfInfo        *info,
         g_message ("xcf: uh oh! xcf rle tile saving error: %d", count);
     }
 
-  xcf_write_int8_check_error (info, rlebuf, len);
-
-  return TRUE;
+  *lenptr = len;
 }
 
-static gboolean
-xcf_save_tile_zlib (XcfInfo        *info,
-                    GeglBuffer     *buffer,
-                    GeglRectangle  *tile_rect,
+static void
+xcf_save_tile_zlib (GeglRectangle  *tile_rect,
+                    guchar         *tile_data,
                     const Babl     *format,
-                    GError        **error)
+                    guchar         *zlib_data,
+                    gint            zlib_data_max_len,
+                    gint           *lenptr)
 {
   gint      bpp       = babl_format_get_bytes_per_pixel (format);
   gint      tile_size = bpp * tile_rect->width * tile_rect->height;
-  guchar   *tile_data = g_alloca (tile_size);
   /* The buffer for compressed data. */
-  guchar   *buf       = g_alloca (tile_size);
-  GError   *tmp_error = NULL;
   z_stream  strm;
   int       action;
   int       status;
 
-  gegl_buffer_get (buffer, tile_rect, 1.0, format, tile_data,
-                   GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
-
-  if (info->file_version >= 12)
-    {
-      gint n_components = babl_format_get_n_components (format);
-
-      xcf_write_to_be (bpp / n_components, tile_data,
-                       tile_size / bpp * n_components);
-    }
+  *lenptr = 0;
 
   /* allocate deflate state */
   strm.zalloc = Z_NULL;
@@ -2020,12 +2361,12 @@ xcf_save_tile_zlib (XcfInfo        *info,
 
   status = deflateInit (&strm, Z_DEFAULT_COMPRESSION);
   if (status != Z_OK)
-    return FALSE;
+    return;
 
   strm.next_in   = tile_data;
   strm.avail_in  = tile_size;
-  strm.next_out  = buf;
-  strm.avail_out = tile_size;
+  strm.next_out  = zlib_data;
+  strm.avail_out = zlib_data_max_len;
 
   action = Z_NO_FLUSH;
 
@@ -2041,24 +2382,23 @@ xcf_save_tile_zlib (XcfInfo        *info,
 
       if (status == Z_STREAM_END || status == Z_BUF_ERROR)
         {
-          size_t write_size = tile_size - strm.avail_out;
+          size_t write_size = zlib_data_max_len - strm.avail_out;
 
-          xcf_write_int8_check_error (info, buf, write_size);
+          *lenptr = write_size;
 
           /* Reset next_out and avail_out. */
-          strm.next_out  = buf;
-          strm.avail_out = tile_size;
+          strm.next_out  = zlib_data;
+          strm.avail_out = zlib_data_max_len;
         }
       else if (status != Z_OK)
         {
           g_printerr ("xcf: tile compression failed: %s", zError (status));
           deflateEnd (&strm);
-          return FALSE;
+          return;
         }
     }
 
   deflateEnd (&strm);
-  return TRUE;
 }
 
 static gboolean
@@ -2124,12 +2464,13 @@ xcf_save_parasite_list (XcfInfo           *info,
   return TRUE;
 }
 
+/* This is the oldest way to save paths. */
 static gboolean
 xcf_save_old_paths (XcfInfo    *info,
                     GimpImage  *image,
                     GError    **error)
 {
-  GimpVectors *active_vectors;
+  GimpVectors *active_vectors = NULL;
   guint32      num_paths;
   guint32      active_index = 0;
   GList       *list;
@@ -2145,7 +2486,16 @@ xcf_save_old_paths (XcfInfo    *info,
 
   num_paths = gimp_container_get_n_children (gimp_image_get_vectors (image));
 
-  active_vectors = gimp_image_get_active_vectors (image);
+  if (gimp_image_get_selected_vectors (image))
+    {
+      active_vectors = gimp_image_get_selected_vectors (image)->data;
+      /* Having more than 1 selected vectors should not have happened in this
+       * code path but let's not break saving, only produce a critical.
+       */
+      if (g_list_length (gimp_image_get_selected_vectors (image)) > 1)
+        g_critical ("%s: this code path should not happen with multiple paths selected",
+                    G_STRFUNC);
+    }
 
   if (active_vectors)
     active_index = gimp_container_get_child_index (gimp_image_get_vectors (image),
@@ -2233,14 +2583,18 @@ xcf_save_old_paths (XcfInfo    *info,
   return TRUE;
 }
 
+/* This is an older way to save paths, though more recent than
+ * xcf_save_old_paths(). It used to be the normal path storing format until all
+ * 2.10 versions. It changed with GIMP 3.0.
+ */
 static gboolean
-xcf_save_vectors (XcfInfo    *info,
-                  GimpImage  *image,
-                  GError    **error)
+xcf_save_old_vectors (XcfInfo    *info,
+                      GimpImage  *image,
+                      GError    **error)
 {
-  GimpVectors *active_vectors;
-  guint32      version      = 1;
-  guint32      active_index = 0;
+  GimpVectors *active_vectors = NULL;
+  guint32      version        = 1;
+  guint32      active_index   = 0;
   guint32      num_paths;
   GList       *list;
   GList       *stroke_list;
@@ -2255,7 +2609,16 @@ xcf_save_vectors (XcfInfo    *info,
    * then each path:-
    */
 
-  active_vectors = gimp_image_get_active_vectors (image);
+  if (gimp_image_get_selected_vectors (image))
+    {
+      active_vectors = gimp_image_get_selected_vectors (image)->data;
+      /* Having more than 1 selected vectors should not have happened in this
+       * code path but let's not break saving, only produce a critical.
+       */
+      if (g_list_length (gimp_image_get_selected_vectors (image)) > 1)
+        g_critical ("%s: this code path should not happen with multiple paths selected",
+                    G_STRFUNC);
+    }
 
   if (active_vectors)
     active_index = gimp_container_get_child_index (gimp_image_get_vectors (image),
@@ -2388,3 +2751,128 @@ xcf_save_vectors (XcfInfo    *info,
 
   return TRUE;
 }
+
+static gboolean
+xcf_save_path (XcfInfo      *info,
+               GimpImage    *image,
+               GimpVectors  *vectors,
+               GError      **error)
+{
+  const gchar *string;
+  GList       *stroke_list;
+  GError      *tmp_error = NULL;
+  /* Version of the path format is always 1 for now. */
+  guint32      version   = 1;
+  guint32      num_strokes;
+  guint32      size;
+  goffset      base;
+  goffset      pos;
+
+  /* write out the path name */
+  string = gimp_object_get_name (vectors);
+  xcf_write_string_check_error (info, (gchar **) &string, 1);
+
+  /* Payload size */
+  size = 0;
+  pos = info->cp;
+  xcf_write_int32_check_error (info, &size, 1);
+  base = info->cp;
+
+  /* write out the path properties */
+  xcf_save_path_props (info, image, vectors, error);
+
+  /* Path version */
+  xcf_write_int32_check_error (info, &version, 1);
+
+  /* Write out the number of strokes. */
+  num_strokes = g_queue_get_length (vectors->strokes);
+  xcf_write_int32_check_error  (info, &num_strokes, 1);
+
+  for (stroke_list = g_list_first (vectors->strokes->head);
+       stroke_list;
+       stroke_list = g_list_next (stroke_list))
+    {
+      GimpStroke *stroke = stroke_list->data;
+      guint32     stroke_type;
+      guint32     closed;
+      guint32     num_axes;
+      GArray     *control_points;
+      gint        i;
+
+      guint32     type;
+      gfloat      coords[6];
+
+      /*
+       * stroke_type (gint)
+       * closed (gint)
+       * num_axes (gint)
+       * num_control_points (gint)
+       *
+       * then each control point.
+       */
+
+      if (GIMP_IS_BEZIER_STROKE (stroke))
+        {
+          stroke_type = XCF_STROKETYPE_BEZIER_STROKE;
+          num_axes = 2;   /* hardcoded, might be increased later */
+        }
+      else
+        {
+          g_printerr ("Skipping unknown stroke type!\n");
+          continue;
+        }
+
+      control_points = gimp_stroke_control_points_get (stroke,
+                                                       (gint32 *) &closed);
+
+      /* Stroke type. */
+      xcf_write_int32_check_error (info, &stroke_type,         1);
+      /* close path or not? */
+      xcf_write_int32_check_error (info, &closed,              1);
+      /* Number of floats given for each point. */
+      xcf_write_int32_check_error (info, &num_axes,            1);
+      /* Number of control points. */
+      xcf_write_int32_check_error (info, &control_points->len, 1);
+
+      for (i = 0; i < control_points->len; i++)
+        {
+          GimpAnchor *anchor;
+
+          anchor = & (g_array_index (control_points, GimpAnchor, i));
+
+          type      = anchor->type;
+          coords[0] = anchor->position.x;
+          coords[1] = anchor->position.y;
+          coords[2] = anchor->position.pressure;
+          coords[3] = anchor->position.xtilt;
+          coords[4] = anchor->position.ytilt;
+          coords[5] = anchor->position.wheel;
+
+          /*
+           * type (gint)
+           *
+           * the first num_axis elements of:
+           * [0] x (gfloat)
+           * [1] y (gfloat)
+           * [2] pressure (gfloat)
+           * [3] xtilt (gfloat)
+           * [4] ytilt (gfloat)
+           * [5] wheel (gfloat)
+           */
+
+          xcf_write_int32_check_error (info, &type,  1);
+          xcf_write_float_check_error (info, coords, num_axes);
+        }
+
+      g_array_free (control_points, TRUE);
+    }
+
+  /* go back to the saved position and write the length */
+  size = info->cp - base;
+  xcf_check_error (xcf_seek_pos (info, pos, error));
+  xcf_write_int32_check_error (info, &size, 1);
+
+  xcf_check_error (xcf_seek_pos (info, base + size, error));
+
+  return TRUE;
+}
