@@ -16,11 +16,12 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library.  If not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 
 #include "libgimpwidgets/gimpwidgets.h"
@@ -46,27 +47,6 @@
 #define CELL_SIZE 20
 
 
-#define GIMP_BRUSH_SELECT_BUTTON_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GIMP_TYPE_BRUSH_SELECT_BUTTON, GimpBrushSelectButtonPrivate))
-
-typedef struct _GimpBrushSelectButtonPrivate GimpBrushSelectButtonPrivate;
-
-struct _GimpBrushSelectButtonPrivate
-{
-  gchar                *title;
-
-  gchar                *brush_name;      /* Local copy */
-  gdouble               opacity;
-  gint                  spacing;
-  GimpLayerModeEffects  paint_mode;
-  gint                  width;
-  gint                  height;
-  guchar               *mask_data;       /* local copy */
-
-  GtkWidget            *inside;
-  GtkWidget            *preview;
-  GtkWidget            *popup;
-};
-
 enum
 {
   BRUSH_SET,
@@ -80,7 +60,26 @@ enum
   PROP_BRUSH_NAME,
   PROP_BRUSH_OPACITY,
   PROP_BRUSH_SPACING,
-  PROP_BRUSH_PAINT_MODE
+  PROP_BRUSH_PAINT_MODE,
+  N_PROPS
+};
+
+
+struct _GimpBrushSelectButtonPrivate
+{
+  gchar         *title;
+
+  gchar         *brush_name;      /* Local copy */
+  gdouble        opacity;
+  gint           spacing;
+  GimpLayerMode  paint_mode;
+  gint           width;
+  gint           height;
+  guchar        *mask_data;       /* local copy */
+
+  GtkWidget     *inside;
+  GtkWidget     *preview;
+  GtkWidget     *popup;
 };
 
 
@@ -102,9 +101,10 @@ static void   gimp_brush_select_button_clicked  (GimpBrushSelectButton *button);
 static void   gimp_brush_select_button_callback (const gchar          *brush_name,
                                                  gdouble               opacity,
                                                  gint                  spacing,
-                                                 GimpLayerModeEffects  paint_mode,
+                                                 GimpLayerMode         paint_mode,
                                                  gint                  width,
                                                  gint                  height,
+                                                 gint                  mask_size,
                                                  const guchar         *mask_data,
                                                  gboolean              dialog_closing,
                                                  gpointer              user_data);
@@ -144,10 +144,11 @@ static GtkWidget * gimp_brush_select_button_create_inside (GimpBrushSelectButton
 static const GtkTargetEntry target = { "application/x-gimp-brush-name", 0 };
 
 static guint brush_button_signals[LAST_SIGNAL] = { 0 };
+static GParamSpec *brush_button_props[N_PROPS] = { NULL, };
 
 
-G_DEFINE_TYPE (GimpBrushSelectButton, gimp_brush_select_button,
-               GIMP_TYPE_SELECT_BUTTON)
+G_DEFINE_TYPE_WITH_PRIVATE (GimpBrushSelectButton, gimp_brush_select_button,
+                            GIMP_TYPE_SELECT_BUTTON)
 
 
 static void
@@ -169,72 +170,69 @@ gimp_brush_select_button_class_init (GimpBrushSelectButtonClass *klass)
    *
    * The title to be used for the brush selection popup dialog.
    *
-   * Since: GIMP 2.4
+   * Since: 2.4
    */
-  g_object_class_install_property (object_class, PROP_TITLE,
-                                   g_param_spec_string ("title",
+  brush_button_props[PROP_TITLE] = g_param_spec_string ("title",
                                                         "Title",
                                                         "The title to be used for the brush selection popup dialog",
                                                         _("Brush Selection"),
                                                         GIMP_PARAM_READWRITE |
-                                                        G_PARAM_CONSTRUCT_ONLY));
+                                                        G_PARAM_CONSTRUCT_ONLY);
 
   /**
    * GimpBrushSelectButton:brush-name:
    *
    * The name of the currently selected brush.
    *
-   * Since: GIMP 2.4
+   * Since: 2.4
    */
-  g_object_class_install_property (object_class, PROP_BRUSH_NAME,
-                                   g_param_spec_string ("brush-name",
-                                                        "Brush name",
-                                                        "The name of the currently selected brush",
-                                                        NULL,
-                                                        GIMP_PARAM_READWRITE));
+  brush_button_props[PROP_BRUSH_NAME] = g_param_spec_string ("brush-name",
+                                                             "Brush name",
+                                                             "The name of the currently selected brush",
+                                                             NULL,
+                                                             GIMP_PARAM_READWRITE);
 
   /**
    * GimpBrushSelectButton:opacity:
    *
    * The opacity of the currently selected brush.
    *
-   * Since: GIMP 2.4
+   * Since: 2.4
    */
-  g_object_class_install_property (object_class, PROP_BRUSH_OPACITY,
-                                   g_param_spec_double ("brush-opacity",
-                                                        "Brush opacity",
-                                                        "The opacity of the currently selected brush",
-                                                        -1.0, 100.0, -1.0,
-                                                        GIMP_PARAM_READWRITE));
+  brush_button_props[PROP_BRUSH_OPACITY] = g_param_spec_double ("brush-opacity",
+                                                                "Brush opacity",
+                                                                "The opacity of the currently selected brush",
+                                                                -1.0, 100.0, -1.0,
+                                                                GIMP_PARAM_READWRITE);
 
   /**
    * GimpBrushSelectButton:spacing:
    *
    * The spacing of the currently selected brush.
    *
-   * Since: GIMP 2.4
+   * Since: 2.4
    */
-  g_object_class_install_property (object_class, PROP_BRUSH_SPACING,
-                                   g_param_spec_int ("brush-spacing",
-                                                     "Brush spacing",
-                                                     "The spacing of the currently selected brush",
-                                                     -G_MAXINT, 1000, -1,
-                                                     GIMP_PARAM_READWRITE));
+  brush_button_props[PROP_BRUSH_SPACING] = g_param_spec_int ("brush-spacing",
+                                                             "Brush spacing",
+                                                             "The spacing of the currently selected brush",
+                                                             -G_MAXINT, 1000, -1,
+                                                             GIMP_PARAM_READWRITE);
 
   /**
    * GimpBrushSelectButton:paint-mode:
    *
-   * The name of the currently selected brush.
+   * The paint mode.
    *
-   * Since: GIMP 2.4
+   * Since: 2.4
    */
-  g_object_class_install_property (object_class, PROP_BRUSH_PAINT_MODE,
-                                   g_param_spec_int ("brush-paint-mode",
-                                                     "Brush paint mode",
-                                                     "The paint mode of the currently selected brush",
-                                                     -1, GIMP_COLOR_ERASE_MODE,
-                                                     -1,
-                                                     GIMP_PARAM_READWRITE));
+  brush_button_props[PROP_BRUSH_PAINT_MODE] = g_param_spec_int ("brush-paint-mode",
+                                                                "Brush paint mode",
+                                                                "The paint mode of the currently selected brush",
+                                                                -1, GIMP_LAYER_MODE_LUMINANCE,
+                                                                -1,
+                                                                GIMP_PARAM_READWRITE);
+
+  g_object_class_install_properties (object_class, N_PROPS, brush_button_props);
 
   /**
    * GimpBrushSelectButton::brush-set:
@@ -245,12 +243,12 @@ gimp_brush_select_button_class_init (GimpBrushSelectButtonClass *klass)
    * @paint_mode: paint mode of the brush
    * @width: width of the brush
    * @height: height of the brush
-   * @mask_data: brush mask data
+   * @mask_data: (array) (element-type guchar): brush mask data
    * @dialog_closing: whether the dialog was closed or not.
    *
    * The ::brush-set signal is emitted when the user selects a brush.
    *
-   * Since: GIMP 2.4
+   * Since: 2.4
    */
   brush_button_signals[BRUSH_SET] =
     g_signal_new ("brush-set",
@@ -268,9 +266,6 @@ gimp_brush_select_button_class_init (GimpBrushSelectButtonClass *klass)
                   G_TYPE_INT,
                   G_TYPE_POINTER,
                   G_TYPE_BOOLEAN);
-
-  g_type_class_add_private (object_class,
-                            sizeof (GimpBrushSelectButtonPrivate));
 }
 
 static void
@@ -283,7 +278,9 @@ gimp_brush_select_button_init (GimpBrushSelectButton *button)
   gint                          color_data_size;
   guint8                       *color_data;
 
-  priv = GIMP_BRUSH_SELECT_BUTTON_GET_PRIVATE (button);
+  button->priv = gimp_brush_select_button_get_instance_private (button);
+
+  priv = button->priv;
 
   priv->brush_name = gimp_context_get_brush ();
   gimp_brush_get_pixels (priv->brush_name,
@@ -301,33 +298,31 @@ gimp_brush_select_button_init (GimpBrushSelectButton *button)
 
   priv->inside = gimp_brush_select_button_create_inside (button);
   gtk_container_add (GTK_CONTAINER (button), priv->inside);
-
-  priv->popup = NULL;
 }
 
 /**
  * gimp_brush_select_button_new:
- * @title:      Title of the dialog to use or %NULL means to use the default
+ * @title: (nullable): Title of the dialog to use or %NULL means to use the default
  *              title.
- * @brush_name: Initial brush name or %NULL to use current selection.
+ * @brush_name: (nullable): Initial brush name or %NULL to use current selection.
  * @opacity:    Initial opacity. -1 means to use current opacity.
  * @spacing:    Initial spacing. -1 means to use current spacing.
  * @paint_mode: Initial paint mode.  -1 means to use current paint mode.
  *
  * Creates a new #GtkWidget that completely controls the selection of
- * a #GimpBrush.  This widget is suitable for placement in a table in
- * a plug-in dialog.
+ * a brush. This widget is suitable for placement in a table in a
+ * plug-in dialog.
  *
  * Returns: A #GtkWidget that you can use in your UI.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  */
 GtkWidget *
-gimp_brush_select_button_new (const gchar          *title,
-                              const gchar          *brush_name,
-                              gdouble               opacity,
-                              gint                  spacing,
-                              GimpLayerModeEffects  paint_mode)
+gimp_brush_select_button_new (const gchar   *title,
+                              const gchar   *brush_name,
+                              gdouble        opacity,
+                              gint           spacing,
+                              GimpLayerMode  paint_mode)
 {
   GtkWidget *button;
 
@@ -352,45 +347,41 @@ gimp_brush_select_button_new (const gchar          *title,
 
 /**
  * gimp_brush_select_button_get_brush:
- * @button: A #GimpBrushSelectButton
- * @opacity: Opacity of the selected brush.
- * @spacing: Spacing of the selected brush.
- * @paint_mode: Paint mode of the selected brush.
+ * @button:                        A #GimpBrushSelectButton
+ * @opacity:    (out) (optional):  Opacity of the selected brush.
+ * @spacing:    (out) (optional):  Spacing of the selected brush.
+ * @paint_mode: (out) (optional):  Paint mode of the selected brush.
  *
  * Retrieves the properties of currently selected brush.
  *
  * Returns: an internal copy of the brush name which must not be freed.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  */
 const gchar *
 gimp_brush_select_button_get_brush (GimpBrushSelectButton *button,
                                     gdouble               *opacity,
                                     gint                  *spacing,
-                                    GimpLayerModeEffects  *paint_mode)
+                                    GimpLayerMode         *paint_mode)
 {
-  GimpBrushSelectButtonPrivate *priv;
-
   g_return_val_if_fail (GIMP_IS_BRUSH_SELECT_BUTTON (button), NULL);
 
-  priv = GIMP_BRUSH_SELECT_BUTTON_GET_PRIVATE (button);
-
   if (opacity)
-    *opacity = priv->opacity;
+    *opacity = button->priv->opacity;
 
   if (spacing)
-    *spacing = priv->spacing;
+    *spacing = button->priv->spacing;
 
   if (paint_mode)
-    *paint_mode = priv->paint_mode;
+    *paint_mode = button->priv->paint_mode;
 
-  return priv->brush_name;
+  return button->priv->brush_name;
 }
 
 /**
  * gimp_brush_select_button_set_brush:
  * @button: A #GimpBrushSelectButton
- * @brush_name: Brush name to set; %NULL means no change.
+ * @brush_name: (nullable): Brush name to set; %NULL means no change.
  * @opacity:    Opacity to set. -1.0 means no change.
  * @spacing:    Spacing to set. -1 means no change.
  * @paint_mode: Paint mode to set.  -1 means no change.
@@ -398,14 +389,14 @@ gimp_brush_select_button_get_brush (GimpBrushSelectButton *button,
  * Sets the current brush and other values for the brush select
  * button.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  */
 void
 gimp_brush_select_button_set_brush (GimpBrushSelectButton *button,
                                     const gchar           *brush_name,
                                     gdouble                opacity,
                                     gint                   spacing,
-                                    GimpLayerModeEffects   paint_mode)
+                                    GimpLayerMode          paint_mode)
 {
   GimpSelectButton *select_button;
 
@@ -459,7 +450,8 @@ gimp_brush_select_button_set_brush (GimpBrushSelectButton *button,
 
           gimp_brush_select_button_callback (name,
                                              opacity, spacing, paint_mode,
-                                             width, height, mask_data,
+                                             width, height, width * height,
+                                             mask_data,
                                              FALSE, button);
 
           g_free (mask_data);
@@ -475,18 +467,11 @@ gimp_brush_select_button_set_brush (GimpBrushSelectButton *button,
 static void
 gimp_brush_select_button_finalize (GObject *object)
 {
-  GimpBrushSelectButtonPrivate *priv;
+  GimpBrushSelectButton *button = GIMP_BRUSH_SELECT_BUTTON (object);
 
-  priv = GIMP_BRUSH_SELECT_BUTTON_GET_PRIVATE (object);
-
-  g_free (priv->brush_name);
-  priv->brush_name = NULL;
-
-  g_free (priv->mask_data);
-  priv->mask_data = NULL;
-
-  g_free (priv->title);
-  priv->title = NULL;
+  g_clear_pointer (&button->priv->brush_name, g_free);
+  g_clear_pointer (&button->priv->mask_data,  g_free);
+  g_clear_pointer (&button->priv->title,      g_free);
 
   G_OBJECT_CLASS (gimp_brush_select_button_parent_class)->finalize (object);
 }
@@ -497,39 +482,41 @@ gimp_brush_select_button_set_property (GObject      *object,
                                        const GValue *value,
                                        GParamSpec   *pspec)
 {
-  GimpBrushSelectButton        *button = GIMP_BRUSH_SELECT_BUTTON (object);
-  GimpBrushSelectButtonPrivate *priv;
-  gdouble opacity;
-  gint32 spacing;
-  gint32 paint_mode;
-
-  priv = GIMP_BRUSH_SELECT_BUTTON_GET_PRIVATE (button);
+  GimpBrushSelectButton *button = GIMP_BRUSH_SELECT_BUTTON (object);
+  gdouble                opacity;
+  gint32                 spacing;
+  gint32                 paint_mode;
 
   switch (property_id)
     {
     case PROP_TITLE:
-      priv->title = g_value_dup_string (value);
+      button->priv->title = g_value_dup_string (value);
       break;
+
     case PROP_BRUSH_NAME:
       gimp_brush_select_button_set_brush (button,
                                           g_value_get_string (value),
                                           -1.0, -1, -1);
       break;
+
     case PROP_BRUSH_OPACITY:
       opacity = g_value_get_double (value);
       if (opacity >= 0.0)
-        priv->opacity = opacity;
+        button->priv->opacity = opacity;
       break;
+
     case PROP_BRUSH_SPACING:
       spacing = g_value_get_int (value);
       if (spacing != -1)
-        priv->spacing = spacing;
+        button->priv->spacing = spacing;
       break;
+
     case PROP_BRUSH_PAINT_MODE:
       paint_mode = g_value_get_int (value);
       if (paint_mode != -1)
-        priv->paint_mode = paint_mode;
+        button->priv->paint_mode = paint_mode;
       break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -542,28 +529,30 @@ gimp_brush_select_button_get_property (GObject    *object,
                                        GValue     *value,
                                        GParamSpec *pspec)
 {
-  GimpBrushSelectButton        *button = GIMP_BRUSH_SELECT_BUTTON (object);
-  GimpBrushSelectButtonPrivate *priv;
-
-  priv = GIMP_BRUSH_SELECT_BUTTON_GET_PRIVATE (button);
+  GimpBrushSelectButton *button = GIMP_BRUSH_SELECT_BUTTON (object);
 
   switch (property_id)
     {
     case PROP_TITLE:
-      g_value_set_string (value, priv->title);
+      g_value_set_string (value, button->priv->title);
       break;
+
     case PROP_BRUSH_NAME:
-      g_value_set_string (value, priv->brush_name);
+      g_value_set_string (value, button->priv->brush_name);
       break;
+
     case PROP_BRUSH_OPACITY:
-      g_value_set_double (value, priv->opacity);
+      g_value_set_double (value, button->priv->opacity);
       break;
+
     case PROP_BRUSH_SPACING:
-      g_value_set_int (value, priv->spacing);
+      g_value_set_int (value, button->priv->spacing);
       break;
+
     case PROP_BRUSH_PAINT_MODE:
-      g_value_set_int (value, priv->paint_mode);
+      g_value_set_int (value, button->priv->paint_mode);
       break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -571,37 +560,32 @@ gimp_brush_select_button_get_property (GObject    *object,
 }
 
 static void
-gimp_brush_select_button_callback (const gchar          *name,
-                                   gdouble               opacity,
-                                   gint                  spacing,
-                                   GimpLayerModeEffects  paint_mode,
-                                   gint                  width,
-                                   gint                  height,
-                                   const guchar         *mask_data,
-                                   gboolean              dialog_closing,
-                                   gpointer              data)
+gimp_brush_select_button_callback (const gchar   *name,
+                                   gdouble        opacity,
+                                   gint           spacing,
+                                   GimpLayerMode  paint_mode,
+                                   gint           width,
+                                   gint           height,
+                                   gint           mask_size,
+                                   const guchar  *mask_data,
+                                   gboolean       dialog_closing,
+                                   gpointer       data)
 {
-  GimpBrushSelectButton        *button;
-  GimpBrushSelectButtonPrivate *priv;
-  GimpSelectButton             *select_button;
+  GimpBrushSelectButton *button        = GIMP_BRUSH_SELECT_BUTTON (data);
+  GimpSelectButton      *select_button = GIMP_SELECT_BUTTON (button);
 
-  button = GIMP_BRUSH_SELECT_BUTTON (data);
+  g_free (button->priv->brush_name);
+  g_free (button->priv->mask_data);
 
-  priv = GIMP_BRUSH_SELECT_BUTTON_GET_PRIVATE (button);
-  select_button = GIMP_SELECT_BUTTON (button);
+  button->priv->brush_name = g_strdup (name);
+  button->priv->width      = width;
+  button->priv->height     = height;
+  button->priv->mask_data  = g_memdup2 (mask_data, width * height);
+  button->priv->opacity    = opacity;
+  button->priv->spacing    = spacing;
+  button->priv->paint_mode = paint_mode;
 
-  g_free (priv->brush_name);
-  g_free (priv->mask_data);
-
-  priv->brush_name = g_strdup (name);
-  priv->width      = width;
-  priv->height     = height;
-  priv->mask_data  = g_memdup (mask_data, width * height);
-  priv->opacity    = opacity;
-  priv->spacing    = spacing;
-  priv->paint_mode = paint_mode;
-
-  gimp_brush_select_preview_update (priv->preview,
+  gimp_brush_select_preview_update (button->priv->preview,
                                     width, height, mask_data);
 
   if (dialog_closing)
@@ -610,49 +594,45 @@ gimp_brush_select_button_callback (const gchar          *name,
   g_signal_emit (button, brush_button_signals[BRUSH_SET], 0,
                  name, opacity, spacing, paint_mode, width, height, mask_data,
                  dialog_closing);
-  g_object_notify (G_OBJECT (button), "brush-name");
+  g_object_notify_by_pspec (G_OBJECT (button), brush_button_props[PROP_BRUSH_NAME]);
 }
 
 static void
 gimp_brush_select_button_clicked (GimpBrushSelectButton *button)
 {
-  GimpBrushSelectButtonPrivate *priv;
-  GimpSelectButton             *select_button;
-
-  priv = GIMP_BRUSH_SELECT_BUTTON_GET_PRIVATE (button);
-  select_button = GIMP_SELECT_BUTTON (button);
+  GimpSelectButton *select_button = GIMP_SELECT_BUTTON (button);
 
   if (select_button->temp_callback)
     {
       /*  calling gimp_brushes_set_popup() raises the dialog  */
       gimp_brushes_set_popup (select_button->temp_callback,
-                              priv->brush_name,
-                              priv->opacity,
-                              priv->spacing,
-                              priv->paint_mode);
+                              button->priv->brush_name,
+                              button->priv->opacity,
+                              button->priv->spacing,
+                              button->priv->paint_mode);
     }
   else
     {
       select_button->temp_callback =
-        gimp_brush_select_new (priv->title, priv->brush_name,
-                               priv->opacity, priv->spacing, priv->paint_mode,
+        gimp_brush_select_new (button->priv->title,
+                               button->priv->brush_name,
+                               button->priv->opacity,
+                               button->priv->spacing,
+                               button->priv->paint_mode,
                                gimp_brush_select_button_callback,
-                               button);
+                               button, NULL);
     }
 }
 
 static void
 gimp_brush_select_preview_resize (GimpBrushSelectButton *button)
 {
-  GimpBrushSelectButtonPrivate *priv;
-
-  priv = GIMP_BRUSH_SELECT_BUTTON_GET_PRIVATE (button);
-
-  if (priv->width > 0 && priv->height > 0)
-    gimp_brush_select_preview_update (priv->preview,
-                                      priv->width,
-                                      priv->height,
-                                      priv->mask_data);
+  if (button->priv->width  > 0 &&
+      button->priv->height > 0)
+    gimp_brush_select_preview_update (button->priv->preview,
+                                      button->priv->width,
+                                      button->priv->height,
+                                      button->priv->mask_data);
 }
 
 static gboolean
@@ -660,12 +640,9 @@ gimp_brush_select_preview_events (GtkWidget             *widget,
                                   GdkEvent              *event,
                                   GimpBrushSelectButton *button)
 {
-  GimpBrushSelectButtonPrivate *priv;
-  GdkEventButton               *bevent;
+  GdkEventButton *bevent;
 
-  priv = GIMP_BRUSH_SELECT_BUTTON_GET_PRIVATE (button);
-
-  if (priv->mask_data)
+  if (button->priv->mask_data)
     {
       switch (event->type)
         {
@@ -772,16 +749,13 @@ gimp_brush_select_button_open_popup (GimpBrushSelectButton *button,
                                      gint                   x,
                                      gint                   y)
 {
-  GimpBrushSelectButtonPrivate *priv;
+  GimpBrushSelectButtonPrivate *priv = button->priv;
   GtkWidget                    *frame;
   GtkWidget                    *preview;
-  GdkScreen                    *screen;
+  GdkMonitor                   *monitor;
+  GdkRectangle                  workarea;
   gint                          x_org;
   gint                          y_org;
-  gint                          scr_w;
-  gint                          scr_h;
-
-  priv = GIMP_BRUSH_SELECT_BUTTON_GET_PRIVATE (button);
 
   if (priv->popup)
     gimp_brush_select_button_close_popup (button);
@@ -789,11 +763,10 @@ gimp_brush_select_button_open_popup (GimpBrushSelectButton *button,
   if (priv->width <= CELL_SIZE && priv->height <= CELL_SIZE)
     return;
 
-  screen = gtk_widget_get_screen (GTK_WIDGET (button));
-
   priv->popup = gtk_window_new (GTK_WINDOW_POPUP);
   gtk_window_set_type_hint (GTK_WINDOW (priv->popup), GDK_WINDOW_TYPE_HINT_DND);
-  gtk_window_set_screen (GTK_WINDOW (priv->popup), screen);
+  gtk_window_set_screen (GTK_WINDOW (priv->popup),
+                         gtk_widget_get_screen (GTK_WIDGET (button)));
 
   frame = gtk_frame_new (NULL);
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
@@ -809,15 +782,14 @@ gimp_brush_select_button_open_popup (GimpBrushSelectButton *button,
   gdk_window_get_origin (gtk_widget_get_window (priv->preview),
                          &x_org, &y_org);
 
-  scr_w = gdk_screen_get_width (screen);
-  scr_h = gdk_screen_get_height (screen);
+  monitor = gimp_widget_get_monitor (GTK_WIDGET (button));
+  gdk_monitor_get_workarea (monitor, &workarea);
 
   x = x_org + x - (priv->width  / 2);
   y = y_org + y - (priv->height / 2);
-  x = (x < 0) ? 0 : x;
-  y = (y < 0) ? 0 : y;
-  x = (x + priv->width  > scr_w) ? scr_w - priv->width  : x;
-  y = (y + priv->height > scr_h) ? scr_h - priv->height : y;
+
+  x = CLAMP (x, workarea.x, workarea.x + workarea.width  - priv->width);
+  y = CLAMP (y, workarea.y, workarea.y + workarea.height - priv->height);
 
   gtk_window_move (GTK_WINDOW (priv->popup), x, y);
 
@@ -832,15 +804,7 @@ gimp_brush_select_button_open_popup (GimpBrushSelectButton *button,
 static void
 gimp_brush_select_button_close_popup (GimpBrushSelectButton *button)
 {
-  GimpBrushSelectButtonPrivate *priv;
-
-  priv = GIMP_BRUSH_SELECT_BUTTON_GET_PRIVATE (button);
-
-  if (priv->popup)
-    {
-      gtk_widget_destroy (priv->popup);
-      priv->popup = NULL;
-    }
+  g_clear_pointer (&button->priv->popup, gtk_widget_destroy);
 }
 
 static void
@@ -885,14 +849,10 @@ gimp_brush_select_drag_data_received (GimpBrushSelectButton *button,
 static GtkWidget *
 gimp_brush_select_button_create_inside (GimpBrushSelectButton *brush_button)
 {
+  GimpBrushSelectButtonPrivate *priv = brush_button->priv;
   GtkWidget                    *hbox;
   GtkWidget                    *frame;
   GtkWidget                    *button;
-  GimpBrushSelectButtonPrivate *priv;
-
-  priv = GIMP_BRUSH_SELECT_BUTTON_GET_PRIVATE (brush_button);
-
-  gtk_widget_push_composite_child ();
 
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 
@@ -933,7 +893,5 @@ gimp_brush_select_button_create_inside (GimpBrushSelectButton *brush_button)
 
   gtk_widget_show_all (hbox);
 
-  gtk_widget_pop_composite_child ();
-
   return hbox;
 }
