@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library.  If not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -46,6 +46,7 @@ static void   gimp_select_button_dispose (GObject *object);
 
 G_DEFINE_TYPE (GimpSelectButton, gimp_select_button, GTK_TYPE_BOX)
 
+
 static void
 gimp_select_button_class_init (GimpSelectButtonClass *klass)
 {
@@ -59,8 +60,6 @@ gimp_select_button_init (GimpSelectButton *select_button)
 {
   gtk_orientable_set_orientation (GTK_ORIENTABLE (select_button),
                                   GTK_ORIENTATION_HORIZONTAL);
-
-  select_button->temp_callback = NULL;
 }
 
 static void
@@ -77,7 +76,7 @@ gimp_select_button_dispose (GObject *object)
  *
  * Closes the popup window associated with @button.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  */
 void
 gimp_select_button_close_popup (GimpSelectButton *button)
