@@ -15,13 +15,15 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library.  If not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 
+#include "libgimpbase/gimpbase.h"
 #include "libgimpmath/gimpmath.h"
 
 #include "gimpwidgets.h"
@@ -58,16 +60,28 @@ enum
   PROP_UPDATE
 };
 
-typedef struct
+
+struct _GimpPreviewPrivate
 {
+  GtkWidget *area;
+  GtkWidget *grid;
+  GtkWidget *frame;
+  GtkWidget *toggle;
   GtkWidget *controls;
-} GimpPreviewPrivate;
+  GdkCursor *cursor_busy;
+  GdkCursor *default_cursor;
 
-#define GIMP_PREVIEW_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GIMP_TYPE_PREVIEW, GimpPreviewPrivate))
+  gint       xoff, yoff;
+  gint       xmin, xmax, ymin, ymax;
+  gint       width, height;
+
+  gboolean   update_preview;
+  guint      timeout_id;
+};
+
+#define GET_PRIVATE(obj) (((GimpPreview *) (obj))->priv)
 
 
-static void      gimp_preview_class_init          (GimpPreviewClass *klass);
-static void      gimp_preview_init                (GimpPreview      *preview);
 static void      gimp_preview_dispose             (GObject          *object);
 static void      gimp_preview_get_property        (GObject          *object,
                                                    guint             property_id,
@@ -113,39 +127,12 @@ static void      gimp_preview_real_untransform    (GimpPreview      *preview,
                                                    gint             *dest_y);
 
 
+G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GimpPreview, gimp_preview, GTK_TYPE_BOX)
+
+#define parent_class gimp_preview_parent_class
+
 static guint preview_signals[LAST_SIGNAL] = { 0 };
 
-static GtkBoxClass *parent_class = NULL;
-
-
-GType
-gimp_preview_get_type (void)
-{
-  static GType preview_type = 0;
-
-  if (! preview_type)
-    {
-      const GTypeInfo preview_info =
-      {
-        sizeof (GimpPreviewClass),
-        (GBaseInitFunc) NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc) gimp_preview_class_init,
-        NULL,           /* class_finalize */
-        NULL,           /* class_data     */
-        sizeof (GimpPreview),
-        0,              /* n_preallocs    */
-        (GInstanceInitFunc) gimp_preview_init,
-      };
-
-      preview_type = g_type_register_static (GTK_TYPE_BOX,
-                                             "GimpPreview",
-                                             &preview_info,
-                                             G_TYPE_FLAG_ABSTRACT);
-    }
-
-  return preview_type;
-}
 
 static void
 gimp_preview_class_init (GimpPreviewClass *klass)
@@ -160,8 +147,7 @@ gimp_preview_class_init (GimpPreviewClass *klass)
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (GimpPreviewClass, invalidated),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
   object_class->dispose           = gimp_preview_dispose;
@@ -178,19 +164,19 @@ gimp_preview_class_init (GimpPreviewClass *klass)
   klass->transform                = gimp_preview_real_transform;
   klass->untransform              = gimp_preview_real_untransform;
 
-  g_type_class_add_private (object_class, sizeof (GimpPreviewPrivate));
-
   g_object_class_install_property (object_class,
                                    PROP_UPDATE,
                                    g_param_spec_boolean ("update",
-                                                         NULL, NULL,
+                                                         "Update",
+                                                         "Whether the preview should update automatically",
                                                          TRUE,
                                                          GIMP_PARAM_READWRITE |
                                                          G_PARAM_CONSTRUCT));
 
   gtk_widget_class_install_style_property (widget_class,
                                            g_param_spec_int ("size",
-                                                             NULL, NULL,
+                                                             "Size",
+                                                             "The preview's size",
                                                              1, 1024,
                                                              DEFAULT_SIZE,
                                                              GIMP_PARAM_READABLE));
@@ -199,10 +185,14 @@ gimp_preview_class_init (GimpPreviewClass *klass)
 static void
 gimp_preview_init (GimpPreview *preview)
 {
-  GimpPreviewPrivate *priv = GIMP_PREVIEW_GET_PRIVATE (preview);
+  GimpPreviewPrivate *priv;
   GtkWidget          *frame;
   gdouble             xalign = 0.0;
 
+  preview->priv = gimp_preview_get_instance_private (preview);
+
+  priv = preview->priv;
+
   gtk_orientable_set_orientation (GTK_ORIENTABLE (preview),
                                   GTK_ORIENTATION_VERTICAL);
 
@@ -212,88 +202,96 @@ gimp_preview_init (GimpPreview *preview)
   if (gtk_widget_get_direction (GTK_WIDGET (preview)) == GTK_TEXT_DIR_RTL)
     xalign = 1.0;
 
-  preview->frame = gtk_aspect_frame_new (NULL, xalign, 0.0, 1.0, TRUE);
-  gtk_frame_set_shadow_type (GTK_FRAME (preview->frame), GTK_SHADOW_NONE);
-  gtk_box_pack_start (GTK_BOX (preview), preview->frame, TRUE, TRUE, 0);
-  gtk_widget_show (preview->frame);
+  priv->frame = gtk_aspect_frame_new (NULL, xalign, 0.0, 1.0, TRUE);
+  gtk_frame_set_shadow_type (GTK_FRAME (priv->frame), GTK_SHADOW_NONE);
+  gtk_box_pack_start (GTK_BOX (preview), priv->frame, TRUE, TRUE, 0);
+  gtk_widget_show (priv->frame);
 
-  preview->table = gtk_table_new (3, 2, FALSE);
-  gtk_table_set_row_spacing (GTK_TABLE (preview->table), 1, 3);
-  gtk_container_add (GTK_CONTAINER (preview->frame), preview->table);
-  gtk_widget_show (preview->table);
+  priv->grid = gtk_grid_new ();
+  gtk_container_add (GTK_CONTAINER (priv->frame), priv->grid);
+  gtk_widget_show (priv->grid);
 
-  preview->timeout_id = 0;
+  priv->timeout_id = 0;
 
-  preview->xmin   = preview->ymin = 0;
-  preview->xmax   = preview->ymax = 1;
-  preview->width  = preview->xmax - preview->xmin;
-  preview->height = preview->ymax - preview->ymin;
+  priv->xmin   = priv->ymin = 0;
+  priv->xmax   = priv->ymax = 1;
+  priv->width  = priv->xmax - priv->xmin;
+  priv->height = priv->ymax - priv->ymin;
 
-  preview->xoff   = 0;
-  preview->yoff   = 0;
+  priv->xoff   = 0;
+  priv->yoff   = 0;
 
-  preview->default_cursor = NULL;
+  priv->default_cursor = NULL;
 
   /*  preview area  */
   frame = gtk_frame_new (NULL);
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
-  gtk_table_attach (GTK_TABLE (preview->table), frame, 0, 1, 0, 1,
-                    GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 0, 0);
+  gtk_widget_set_hexpand (frame, TRUE);
+  gtk_widget_set_vexpand (frame, TRUE);
+  gtk_grid_attach (GTK_GRID (priv->grid), frame, 0, 0, 1, 1);
   gtk_widget_show (frame);
 
-  preview->area = gimp_preview_area_new ();
-  gtk_container_add (GTK_CONTAINER (frame), preview->area);
-  gtk_widget_show (preview->area);
+  priv->area = gimp_preview_area_new ();
+  gtk_container_add (GTK_CONTAINER (frame), priv->area);
+  gtk_widget_show (priv->area);
 
-  g_signal_connect_swapped (preview->area, "notify::check-size",
+  g_signal_connect_swapped (priv->area, "notify::check-size",
                             G_CALLBACK (gimp_preview_notify_checks),
                             preview);
-  g_signal_connect_swapped (preview->area, "notify::check-type",
+  g_signal_connect_swapped (priv->area, "notify::check-type",
+                            G_CALLBACK (gimp_preview_notify_checks),
+                            preview);
+  g_signal_connect_swapped (priv->area, "notify::check-custom-color1",
+                            G_CALLBACK (gimp_preview_notify_checks),
+                            preview);
+  g_signal_connect_swapped (priv->area, "notify::check-custom-color2",
                             G_CALLBACK (gimp_preview_notify_checks),
                             preview);
 
-  gtk_widget_add_events (preview->area,
+  gtk_widget_add_events (priv->area,
                          GDK_BUTTON_PRESS_MASK        |
                          GDK_BUTTON_RELEASE_MASK      |
+                         GDK_SCROLL_MASK              |
+                         GDK_SMOOTH_SCROLL_MASK       |
                          GDK_POINTER_MOTION_HINT_MASK |
                          GDK_BUTTON_MOTION_MASK);
 
-  g_signal_connect (preview->area, "event",
+  g_signal_connect (priv->area, "event",
                     G_CALLBACK (gimp_preview_area_event),
                     preview);
 
-  g_signal_connect (preview->area, "realize",
+  g_signal_connect (priv->area, "realize",
                     G_CALLBACK (gimp_preview_area_realize),
                     preview);
-  g_signal_connect (preview->area, "unrealize",
+  g_signal_connect (priv->area, "unrealize",
                     G_CALLBACK (gimp_preview_area_unrealize),
                     preview);
 
-  g_signal_connect_data (preview->area, "realize",
+  g_signal_connect_data (priv->area, "realize",
                          G_CALLBACK (gimp_preview_area_set_cursor),
                          preview, NULL, G_CONNECT_AFTER | G_CONNECT_SWAPPED);
 
-  g_signal_connect (preview->area, "size-allocate",
+  g_signal_connect (priv->area, "size-allocate",
                     G_CALLBACK (gimp_preview_area_size_allocate),
                     preview);
 
-  g_signal_connect_data (preview->area, "size-allocate",
+  g_signal_connect_data (priv->area, "size-allocate",
                          G_CALLBACK (gimp_preview_area_set_cursor),
                          preview, NULL, G_CONNECT_AFTER | G_CONNECT_SWAPPED);
 
   priv->controls = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-  gtk_table_attach (GTK_TABLE (preview->table), priv->controls, 0, 2, 2, 3,
-                    GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
+  gtk_widget_set_margin_top (priv->controls, 3);
+  gtk_grid_attach (GTK_GRID (priv->grid), priv->controls, 0, 2, 2, 1);
   gtk_widget_show (priv->controls);
 
   /*  toggle button to (de)activate the instant preview  */
-  preview->toggle = gtk_check_button_new_with_mnemonic (_("_Preview"));
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (preview->toggle),
-                                preview->update_preview);
-  gtk_box_pack_start (GTK_BOX (priv->controls), preview->toggle, TRUE, TRUE, 0);
-  gtk_widget_show (preview->toggle);
+  priv->toggle = gtk_check_button_new_with_mnemonic (_("_Preview"));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->toggle),
+                                priv->update_preview);
+  gtk_box_pack_start (GTK_BOX (priv->controls), priv->toggle, TRUE, TRUE, 0);
+  gtk_widget_show (priv->toggle);
 
-  g_signal_connect (preview->toggle, "toggled",
+  g_signal_connect (priv->toggle, "toggled",
                     G_CALLBACK (gimp_preview_toggle_callback),
                     preview);
 }
@@ -301,12 +299,12 @@ gimp_preview_init (GimpPreview *preview)
 static void
 gimp_preview_dispose (GObject *object)
 {
-  GimpPreview *preview = GIMP_PREVIEW (object);
+  GimpPreviewPrivate *priv = GET_PRIVATE (object);
 
-  if (preview->timeout_id)
+  if (priv->timeout_id)
     {
-      g_source_remove (preview->timeout_id);
-      preview->timeout_id = 0;
+      g_source_remove (priv->timeout_id);
+      priv->timeout_id = 0;
     }
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
@@ -318,12 +316,12 @@ gimp_preview_get_property (GObject    *object,
                            GValue     *value,
                            GParamSpec *pspec)
 {
-  GimpPreview *preview = GIMP_PREVIEW (object);
+  GimpPreviewPrivate *priv = GET_PRIVATE (object);
 
   switch (property_id)
     {
     case PROP_UPDATE:
-      g_value_set_boolean (value, preview->update_preview);
+      g_value_set_boolean (value, priv->update_preview);
       break;
 
     default:
@@ -338,12 +336,12 @@ gimp_preview_set_property (GObject      *object,
                            const GValue *value,
                            GParamSpec   *pspec)
 {
-  GimpPreview *preview = GIMP_PREVIEW (object);
+  GimpPreviewPrivate *priv = GET_PRIVATE (object);
 
   switch (property_id)
     {
     case PROP_UPDATE:
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (preview->toggle),
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->toggle),
                                     g_value_get_boolean (value));
       break;
 
@@ -357,22 +355,22 @@ static void
 gimp_preview_direction_changed (GtkWidget        *widget,
                                 GtkTextDirection  prev_dir)
 {
-  GimpPreview *preview = GIMP_PREVIEW (widget);
-  gdouble      xalign  = 0.0;
+  GimpPreviewPrivate *priv = GET_PRIVATE (widget);
+  gdouble             xalign  = 0.0;
 
   if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
     xalign = 1.0;
 
-  gtk_aspect_frame_set (GTK_ASPECT_FRAME (preview->frame),
+  gtk_aspect_frame_set (GTK_ASPECT_FRAME (priv->frame),
                         xalign, 0.0, 1.0, TRUE);
 }
 
 static gboolean
 gimp_preview_popup_menu (GtkWidget *widget)
 {
-  GimpPreview *preview = GIMP_PREVIEW (widget);
+  GimpPreviewPrivate *priv = GET_PRIVATE (widget);
 
-  gimp_preview_area_menu_popup (GIMP_PREVIEW_AREA (preview->area), NULL);
+  gimp_preview_area_menu_popup (GIMP_PREVIEW_AREA (priv->area), NULL);
 
   return TRUE;
 }
@@ -381,11 +379,12 @@ static void
 gimp_preview_area_realize (GtkWidget   *widget,
                            GimpPreview *preview)
 {
-  GdkDisplay *display = gtk_widget_get_display (widget);
+  GimpPreviewPrivate *priv    = GET_PRIVATE (preview);
+  GdkDisplay         *display = gtk_widget_get_display (widget);
 
-  g_return_if_fail (preview->cursor_busy == NULL);
+  g_return_if_fail (priv->cursor_busy == NULL);
 
-  preview->cursor_busy = gdk_cursor_new_for_display (display, GDK_WATCH);
+  priv->cursor_busy = gdk_cursor_new_for_display (display, GDK_WATCH);
 
 }
 
@@ -393,11 +392,9 @@ static void
 gimp_preview_area_unrealize (GtkWidget   *widget,
                              GimpPreview *preview)
 {
-  if (preview->cursor_busy)
-    {
-      gdk_cursor_unref (preview->cursor_busy);
-      preview->cursor_busy = NULL;
-    }
+  GimpPreviewPrivate *priv = GET_PRIVATE (preview);
+
+  g_clear_object (&priv->cursor_busy);
 }
 
 static void
@@ -405,11 +402,12 @@ gimp_preview_area_size_allocate (GtkWidget     *widget,
                                  GtkAllocation *allocation,
                                  GimpPreview   *preview)
 {
-  gint width  = preview->xmax - preview->xmin;
-  gint height = preview->ymax - preview->ymin;
+  GimpPreviewPrivate *priv   = GET_PRIVATE (preview);
+  gint                width  = priv->xmax - priv->xmin;
+  gint                height = priv->ymax - priv->ymin;
 
-  preview->width  = MIN (width,  allocation->width);
-  preview->height = MIN (height, allocation->height);
+  priv->width  = MIN (width,  allocation->width);
+  priv->height = MIN (height, allocation->height);
 
   gimp_preview_draw (preview);
   gimp_preview_invalidate (preview);
@@ -450,20 +448,22 @@ static void
 gimp_preview_toggle_callback (GtkWidget   *toggle,
                               GimpPreview *preview)
 {
+  GimpPreviewPrivate *priv = GET_PRIVATE (preview);
+
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (toggle)))
     {
-      preview->update_preview = TRUE;
+      priv->update_preview = TRUE;
 
       g_object_notify (G_OBJECT (preview), "update");
 
-      if (preview->timeout_id)
-        g_source_remove (preview->timeout_id);
+      if (priv->timeout_id)
+        g_source_remove (priv->timeout_id);
 
       gimp_preview_invalidate_now (preview);
     }
   else
     {
-      preview->update_preview = FALSE;
+      priv->update_preview = FALSE;
 
       g_object_notify (G_OBJECT (preview), "update");
 
@@ -481,21 +481,22 @@ gimp_preview_notify_checks (GimpPreview *preview)
 static gboolean
 gimp_preview_invalidate_now (GimpPreview *preview)
 {
-  GtkWidget        *toplevel = gtk_widget_get_toplevel (GTK_WIDGET (preview));
-  GimpPreviewClass *class    = GIMP_PREVIEW_GET_CLASS (preview);
+  GimpPreviewPrivate *priv     = GET_PRIVATE (preview);
+  GtkWidget          *toplevel = gtk_widget_get_toplevel (GTK_WIDGET (preview));
+  GimpPreviewClass   *class    = GIMP_PREVIEW_GET_CLASS (preview);
 
   gimp_preview_draw (preview);
 
-  preview->timeout_id = 0;
+  priv->timeout_id = 0;
 
   if (toplevel && gtk_widget_get_realized (toplevel))
     {
       gdk_window_set_cursor (gtk_widget_get_window (toplevel),
-                             preview->cursor_busy);
-      gdk_window_set_cursor (gtk_widget_get_window (preview->area),
-                             preview->cursor_busy);
+                             priv->cursor_busy);
+      gdk_window_set_cursor (gtk_widget_get_window (priv->area),
+                             priv->cursor_busy);
 
-      gdk_flush ();
+      gdk_display_flush (gtk_widget_get_display (toplevel));
 
       g_signal_emit (preview, preview_signals[INVALIDATED], 0);
 
@@ -513,9 +514,11 @@ gimp_preview_invalidate_now (GimpPreview *preview)
 static void
 gimp_preview_real_set_cursor (GimpPreview *preview)
 {
-  if (gtk_widget_get_realized (preview->area))
-    gdk_window_set_cursor (gtk_widget_get_window (preview->area),
-                           preview->default_cursor);
+  GimpPreviewPrivate *priv = GET_PRIVATE (preview);
+
+  if (gtk_widget_get_realized (priv->area))
+    gdk_window_set_cursor (gtk_widget_get_window (priv->area),
+                           priv->default_cursor);
 }
 
 static void
@@ -525,8 +528,10 @@ gimp_preview_real_transform (GimpPreview *preview,
                              gint        *dest_x,
                              gint        *dest_y)
 {
-  *dest_x = src_x - preview->xoff - preview->xmin;
-  *dest_y = src_y - preview->yoff - preview->ymin;
+  GimpPreviewPrivate *priv = GET_PRIVATE (preview);
+
+  *dest_x = src_x - priv->xoff - priv->xmin;
+  *dest_y = src_y - priv->yoff - priv->ymin;
 }
 
 static void
@@ -536,8 +541,10 @@ gimp_preview_real_untransform (GimpPreview *preview,
                                gint        *dest_x,
                                gint        *dest_y)
 {
-  *dest_x = src_x + preview->xoff + preview->xmin;
-  *dest_y = src_y + preview->yoff + preview->ymin;
+  GimpPreviewPrivate *priv = GET_PRIVATE (preview);
+
+  *dest_x = src_x + priv->xoff + priv->xmin;
+  *dest_y = src_y + priv->yoff + priv->ymin;
 }
 
 /**
@@ -548,7 +555,7 @@ gimp_preview_real_untransform (GimpPreview *preview,
  *
  * Sets the state of the "Preview" check button.
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 void
 gimp_preview_set_update (GimpPreview *preview,
@@ -565,16 +572,16 @@ gimp_preview_set_update (GimpPreview *preview,
  * gimp_preview_get_update:
  * @preview: a #GimpPreview widget
  *
- * Return value: the state of the "Preview" check button.
+ * Returns: the state of the "Preview" check button.
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 gboolean
 gimp_preview_get_update (GimpPreview *preview)
 {
   g_return_val_if_fail (GIMP_IS_PREVIEW (preview), FALSE);
 
-  return preview->update_preview;
+  return GET_PRIVATE (preview)->update_preview;
 }
 
 /**
@@ -589,7 +596,7 @@ gimp_preview_get_update (GimpPreview *preview)
  * difference between the upper and lower value is used to set the
  * maximum size of the #GimpPreviewArea used in the @preview.
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 void
 gimp_preview_set_bounds (GimpPreview *preview,
@@ -598,62 +605,148 @@ gimp_preview_set_bounds (GimpPreview *preview,
                          gint         xmax,
                          gint         ymax)
 {
+  GimpPreviewPrivate *priv;
+
   g_return_if_fail (GIMP_IS_PREVIEW (preview));
   g_return_if_fail (xmax > xmin);
   g_return_if_fail (ymax > ymin);
 
-  preview->xmin = xmin;
-  preview->ymin = ymin;
-  preview->xmax = xmax;
-  preview->ymax = ymax;
+  priv = GET_PRIVATE (preview);
 
-  gimp_preview_area_set_max_size (GIMP_PREVIEW_AREA (preview->area),
+  priv->xmin = xmin;
+  priv->ymin = ymin;
+  priv->xmax = xmax;
+  priv->ymax = ymax;
+
+  gimp_preview_area_set_max_size (GIMP_PREVIEW_AREA (priv->area),
                                   xmax - xmin,
                                   ymax - ymin);
 }
 
+/**
+ * gimp_preview_get_bounds:
+ * @preview: A #GimpPreview:
+ * @xmin: (out) (optional): Lower bound for x
+ * @ymin: (out) (optional): Lower bound for y
+ * @xmax: (out) (optional): Higher bound for x
+ * @ymax: (out) (optional): Higher bound for y
+ */
+void
+gimp_preview_get_bounds (GimpPreview *preview,
+                         gint        *xmin,
+                         gint        *ymin,
+                         gint        *xmax,
+                         gint        *ymax)
+{
+  GimpPreviewPrivate *priv;
+
+  g_return_if_fail (GIMP_IS_PREVIEW (preview));
+
+  priv = GET_PRIVATE (preview);
+
+  if (xmin) *xmin = priv->xmin;
+  if (ymin) *ymin = priv->ymin;
+  if (xmax) *xmax = priv->xmax;
+  if (ymax) *ymax = priv->ymax;
+}
+
+void
+gimp_preview_set_size (GimpPreview *preview,
+                       gint         width,
+                       gint         height)
+{
+  GimpPreviewPrivate *priv;
+
+  g_return_if_fail (GIMP_IS_PREVIEW (preview));
+
+  priv = GET_PRIVATE (preview);
+
+  priv->width  = width;
+  priv->height = height;
+
+  gtk_widget_set_size_request (priv->area, width, height);
+}
+
 /**
  * gimp_preview_get_size:
  * @preview: a #GimpPreview widget
- * @width:   return location for the preview area width
- * @height:  return location for the preview area height
+ * @width: (out) (optional): return location for the preview area width
+ * @height: (out) (optional): return location for the preview area height
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 void
 gimp_preview_get_size (GimpPreview *preview,
                        gint        *width,
                        gint        *height)
 {
+  GimpPreviewPrivate *priv;
+
   g_return_if_fail (GIMP_IS_PREVIEW (preview));
 
-  if (width)
-    *width = preview->width;
+  priv = GET_PRIVATE (preview);
 
-  if (height)
-    *height = preview->height;
+  if (width)  *width  = priv->width;
+  if (height) *height = priv->height;
+}
+
+void
+gimp_preview_set_offsets (GimpPreview *preview,
+                          gint         xoff,
+                          gint         yoff)
+{
+  GimpPreviewPrivate *priv;
+
+  g_return_if_fail (GIMP_IS_PREVIEW (preview));
+
+  priv = GET_PRIVATE (preview);
+
+  priv->xoff = xoff;
+  priv->yoff = yoff;
+}
+
+/**
+ * gimp_preview_get_offsets:
+ * @preview: A #GimpPreview:
+ * @xoff: (out) (optional): X offset
+ * @yoff: (out) (optional): Y offset
+ */
+void
+gimp_preview_get_offsets (GimpPreview *preview,
+                          gint        *xoff,
+                          gint        *yoff)
+{
+  GimpPreviewPrivate *priv;
+
+  g_return_if_fail (GIMP_IS_PREVIEW (preview));
+
+  priv = GET_PRIVATE (preview);
+
+  if (xoff) *xoff = priv->xoff;
+  if (yoff) *yoff = priv->yoff;
 }
 
 /**
  * gimp_preview_get_position:
  * @preview: a #GimpPreview widget
- * @x:       return location for the horizontal offset
- * @y:       return location for the vertical offset
+ * @x: (out) (optional): return location for the horizontal offset
+ * @y: (out) (optional): return location for the vertical offset
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 void
 gimp_preview_get_position (GimpPreview *preview,
                            gint        *x,
                            gint        *y)
 {
+  GimpPreviewPrivate *priv;
+
   g_return_if_fail (GIMP_IS_PREVIEW (preview));
 
-  if (x)
-    *x = preview->xoff + preview->xmin;
+  priv = GET_PRIVATE (preview);
 
-  if (y)
-    *y = preview->yoff + preview->ymin;
+  if (x) *x = priv->xoff + priv->xmin;
+  if (y) *y = priv->yoff + priv->ymin;
 }
 
 /**
@@ -661,12 +754,12 @@ gimp_preview_get_position (GimpPreview *preview,
  * @preview: a #GimpPreview widget
  * @src_x:   horizontal position on the previewed image
  * @src_y:   vertical position on the previewed image
- * @dest_x:  returns the transformed horizontal position
- * @dest_y:  returns the transformed vertical position
+ * @dest_x: (out): returns the transformed horizontal position
+ * @dest_y: (out): returns the transformed vertical position
  *
  * Transforms from image to widget coordinates.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  **/
 void
 gimp_preview_transform (GimpPreview *preview,
@@ -687,12 +780,12 @@ gimp_preview_transform (GimpPreview *preview,
  * @preview: a #GimpPreview widget
  * @src_x:   horizontal position relative to the preview area's origin
  * @src_y:   vertical position relative to  preview area's origin
- * @dest_x:  returns the untransformed horizontal position
- * @dest_y:  returns the untransformed vertical position
+ * @dest_x: (out): returns the untransformed horizontal position
+ * @dest_y: (out): returns the untransformed vertical position
  *
  * Transforms from widget to image coordinates.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  **/
 void
 gimp_preview_untransform (GimpPreview *preview,
@@ -708,6 +801,40 @@ gimp_preview_untransform (GimpPreview *preview,
                                                  src_x, src_y, dest_x, dest_y);
 }
 
+/**
+ * gimp_preview_get_frame:
+ * @preview: a #GimpPreview widget
+ *
+ * Returns: (transfer none) (type GtkAspectFrame): a pointer to the
+ * #GtkAspectFrame used in the @preview.
+ *
+ * Since: 3.0
+ **/
+GtkWidget *
+gimp_preview_get_frame (GimpPreview  *preview)
+{
+  g_return_val_if_fail (GIMP_IS_PREVIEW (preview), NULL);
+
+  return GET_PRIVATE (preview)->frame;
+}
+
+/**
+ * gimp_preview_get_grid:
+ * @preview: a #GimpPreview widget
+ *
+ * Returns: (transfer none) (type GtkGrid): a pointer to
+ * the #GtkGrid used in the @preview.
+ *
+ * Since: 3.0
+ **/
+GtkWidget *
+gimp_preview_get_grid (GimpPreview  *preview)
+{
+  g_return_val_if_fail (GIMP_IS_PREVIEW (preview), NULL);
+
+  return GET_PRIVATE (preview)->grid;
+}
+
 /**
  * gimp_preview_get_area:
  * @preview: a #GimpPreview widget
@@ -717,16 +844,17 @@ gimp_preview_untransform (GimpPreview *preview,
  * For example if you want to receive mouse events from the area. In
  * such cases, use gimp_preview_get_area().
  *
- * Return value: a pointer to the #GimpPreviewArea used in the @preview.
+ * Returns: (transfer none) (type GimpPreviewArea): a pointer to the
+ * #GimpPreviewArea used in the @preview.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  **/
 GtkWidget *
 gimp_preview_get_area (GimpPreview  *preview)
 {
   g_return_val_if_fail (GIMP_IS_PREVIEW (preview), NULL);
 
-  return preview->area;
+  return GET_PRIVATE (preview)->area;
 }
 
 /**
@@ -740,7 +868,7 @@ gimp_preview_get_area (GimpPreview  *preview)
  * #GimpDrawablePreview implements gimp_preview_draw() by drawing the
  * original, unmodified drawable to the @preview.
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 void
 gimp_preview_draw (GimpPreview *preview)
@@ -754,14 +882,14 @@ gimp_preview_draw (GimpPreview *preview)
 /**
  * gimp_preview_draw_buffer:
  * @preview:   a #GimpPreview widget
- * @buffer:    a pixel buffer the size of the preview
+ * @buffer: (array): a pixel buffer the size of the preview
  * @rowstride: the @buffer's rowstride
  *
  * Calls the GimpPreview::draw_buffer method. GimpPreview itself
  * doesn't implement this method so the behaviour is determined by the
  * derived class implementing this method.
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 void
 gimp_preview_draw_buffer (GimpPreview  *preview,
@@ -788,19 +916,23 @@ gimp_preview_draw_buffer (GimpPreview  *preview,
  * toplevel window containing the @preview and on the preview area
  * itself.
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 void
 gimp_preview_invalidate (GimpPreview *preview)
 {
+  GimpPreviewPrivate *priv;
+
   g_return_if_fail (GIMP_IS_PREVIEW (preview));
 
-  if (preview->update_preview)
-    {
-      if (preview->timeout_id)
-        g_source_remove (preview->timeout_id);
+  priv = GET_PRIVATE (preview);
 
-      preview->timeout_id =
+  if (priv->update_preview)
+    {
+      if (priv->timeout_id)
+        g_source_remove (priv->timeout_id);
+
+      priv->timeout_id =
         g_timeout_add_full (G_PRIORITY_DEFAULT_IDLE, PREVIEW_TIMEOUT,
                             (GSourceFunc) gimp_preview_invalidate_now,
                             preview, NULL);
@@ -813,42 +945,59 @@ gimp_preview_invalidate (GimpPreview *preview)
  * @cursor:  a #GdkCursor or %NULL
  *
  * Sets the default mouse cursor for the preview.  Note that this will
- * be overriden by a %GDK_FLEUR if the preview has scrollbars, or by a
+ * be overridden by a %GDK_FLEUR if the preview has scrollbars, or by a
  * %GDK_WATCH when the preview is invalidated.
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 void
 gimp_preview_set_default_cursor (GimpPreview *preview,
                                  GdkCursor   *cursor)
 {
+  GimpPreviewPrivate *priv;
+
   g_return_if_fail (GIMP_IS_PREVIEW (preview));
 
-  if (preview->default_cursor)
-    gdk_cursor_unref (preview->default_cursor);
+  priv = GET_PRIVATE (preview);
 
-  if (cursor)
-    gdk_cursor_ref (cursor);
+  g_set_object (&priv->default_cursor, cursor);
+}
 
-  preview->default_cursor = cursor;
+/**
+ * gimp_preview_get_default_cursor:
+ * @preview: a #GimpPreview widget
+ *
+ * See gimp_preview_set_default_cursor():
+ *
+ * Returns: (transfer none): The @preview's default #GdkCursor.
+ *
+ * Since: 3.0
+ **/
+GdkCursor *
+gimp_preview_get_default_cursor (GimpPreview *preview)
+{
+  g_return_val_if_fail (GIMP_IS_PREVIEW (preview), NULL);
+
+  return GET_PRIVATE (preview)->default_cursor;
 }
 
 /**
  * gimp_preview_get_controls:
  * @preview: a #GimpPreview widget
  *
- * Gives access to the #GtkHBox at the bottom of the preview that
- * contains the update toggle. Derived widgets can use this function
+ * Gives access to the horizontal #GtkBox at the bottom of the preview
+ * that contains the update toggle. Derived widgets can use this function
  * if they need to add controls to this area.
  *
- * Return value: the #GtkHBox at the bottom of the preview.
+ * Returns: (transfer none) (type GtkBox): the horizontal #GtkBox at the
+ * bottom of the preview.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  **/
 GtkWidget *
 gimp_preview_get_controls (GimpPreview *preview)
 {
   g_return_val_if_fail (GIMP_IS_PREVIEW (preview), NULL);
 
-  return GIMP_PREVIEW_GET_PRIVATE (preview)->controls;
+  return GET_PRIVATE (preview)->controls;
 }
