@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -28,11 +27,10 @@
 
 #define PLUG_IN_PROC        "plug-in-retinex"
 #define PLUG_IN_BINARY      "contrast-retinex"
+#define PLUG_IN_ROLE        "gimp-contrast-retinex"
 #define MAX_RETINEX_SCALES    8
 #define MIN_GAUSSIAN_SCALE   16
 #define MAX_GAUSSIAN_SCALE  250
-#define SCALE_WIDTH         150
-#define ENTRY_WIDTH           4
 
 
 typedef struct
@@ -70,20 +68,42 @@ typedef struct
 } gauss3_coefs;
 
 
-/*
- * Declare local functions.
- */
-static void     query                       (void);
-static void     run                         (const gchar      *name,
-                                             gint              nparams,
-                                             const GimpParam  *param,
-                                             gint             *nreturn_vals,
-                                             GimpParam       **return_vals);
+typedef struct _Retinex      Retinex;
+typedef struct _RetinexClass RetinexClass;
+
+struct _Retinex
+{
+  GimpPlugIn parent_instance;
+};
+
+struct _RetinexClass
+{
+  GimpPlugInClass parent_class;
+};
+
+
+#define RETINEX_TYPE  (retinex_get_type ())
+#define RETINEX (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), RETINEX_TYPE, Retinex))
+
+GType                   retinex_get_type         (void) G_GNUC_CONST;
+
+static GList          * retinex_query_procedures (GimpPlugIn           *plug_in);
+static GimpProcedure  * retinex_create_procedure (GimpPlugIn           *plug_in,
+                                                  const gchar          *name);
+
+static GimpValueArray * retinex_run              (GimpProcedure        *procedure,
+                                                  GimpRunMode           run_mode,
+                                                  GimpImage            *image,
+                                                  gint                  n_drawables,
+                                                  GimpDrawable        **drawables,
+                                                  const GimpValueArray *args,
+                                                  gpointer              run_data);
 
-/* Gimp */
 static gboolean retinex_dialog              (GimpDrawable *drawable);
 static void     retinex                     (GimpDrawable *drawable,
                                              GimpPreview  *preview);
+static void     retinex_preview             (GimpDrawable *drawable,
+                                             GimpPreview  *preview);
 
 static void     retinex_scales_distribution (gfloat       *scales,
                                              gint          nscales,
@@ -95,9 +115,7 @@ static void     compute_mean_var            (gfloat       *src,
                                              gfloat       *var,
                                              gint          size,
                                              gint          bytes);
-/*
- * Gauss
- */
+
 static void     compute_coefs3              (gauss3_coefs *c,
                                              gfloat        sigma);
 
@@ -107,6 +125,11 @@ static void     gausssmooth                 (gfloat       *in,
                                              gint          rowtride,
                                              gauss3_coefs *c);
 
+static void contrast_retinex_scale_entry_update_int   (GimpLabelSpin *entry,
+                                                       gint          *value);
+static void contrast_retinex_scale_entry_update_float (GimpLabelSpin *entry,
+                                                       gfloat        *value);
+
 /*
  * MSRCR = MultiScale Retinex with Color Restoration
  */
@@ -117,9 +140,12 @@ static void     MSRCR                       (guchar       *src,
                                              gboolean      preview_mode);
 
 
-/*
- * Private variables.
- */
+G_DEFINE_TYPE (Retinex, retinex, GIMP_TYPE_PLUG_IN)
+
+GIMP_MAIN (RETINEX_TYPE)
+DEFINE_STD_SET_I18N
+
+
 static RetinexParams rvals =
 {
   240,             /* Scale */
@@ -128,112 +154,150 @@ static RetinexParams rvals =
   1.2              /* A voir */
 };
 
-static GimpPlugInInfo PLUG_IN_INFO =
-{
-  NULL,  /* init_proc  */
-  NULL,  /* quit_proc  */
-  query, /* query_proc */
-  run,   /* run_proc   */
-};
-
-MAIN ()
 
 static void
-query (void)
+retinex_class_init (RetinexClass *klass)
 {
-  static const GimpParamDef args[] =
-  {
-    { GIMP_PDB_INT32,    "run-mode",    "Interactive, non-interactive"        },
-    { GIMP_PDB_IMAGE,    "image",       "Input image (unused)"                },
-    { GIMP_PDB_DRAWABLE, "drawable",    "Input drawable"                      },
-    { GIMP_PDB_INT32,    "scale",       "Biggest scale value"                 },
-    { GIMP_PDB_INT32,    "nscales",     "Number of scales"                    },
-    { GIMP_PDB_INT32,    "scales-mode", "Retinex distribution through scales" },
-    { GIMP_PDB_FLOAT,    "cvar",        "Variance value"                      }
-  };
+  GimpPlugInClass *plug_in_class = GIMP_PLUG_IN_CLASS (klass);
 
-  gimp_install_procedure (PLUG_IN_PROC,
-                          N_("Enhance contrast using the Retinex method"),
-                          "The Retinex Image Enhancement Algorithm is an "
-                          "automatic image enhancement method that enhances "
-                          "a digital image in terms of dynamic range "
-                          "compression, color independence from the spectral "
-                          "distribution of the scene illuminant, and "
-                          "color/lightness rendition.",
-                          "Fabien Pelisson",
-                          "Fabien Pelisson",
-                          "2003",
-                          N_("Retine_x..."),
-                          "RGB*",
-                          GIMP_PLUGIN,
-                          G_N_ELEMENTS (args), 0,
-                          args, NULL);
-
-  gimp_plugin_menu_register (PLUG_IN_PROC, "<Image>/Colors/Modify");
+  plug_in_class->query_procedures = retinex_query_procedures;
+  plug_in_class->create_procedure = retinex_create_procedure;
+  plug_in_class->set_i18n         = STD_SET_I18N;
 }
 
 static void
-run (const gchar      *name,
-     gint              nparams,
-     const GimpParam  *param,
-     gint             *nreturn_vals,
-     GimpParam       **return_vals)
+retinex_init (Retinex *retinex)
 {
-  static GimpParam   values[1];
-  GimpDrawable      *drawable;
-  GimpRunMode        run_mode;
-  GimpPDBStatusType  status = GIMP_PDB_SUCCESS;
-  gint               x, y, width, height;
+}
 
-  run_mode = param[0].data.d_int32;
+static GList *
+retinex_query_procedures (GimpPlugIn *plug_in)
+{
+  return g_list_append (NULL, g_strdup (PLUG_IN_PROC));
+}
 
-  INIT_I18N ();
+static GimpProcedure *
+retinex_create_procedure (GimpPlugIn  *plug_in,
+                               const gchar *name)
+{
+  GimpProcedure *procedure = NULL;
 
-  *nreturn_vals = 1;
-  *return_vals  = values;
+  if (! strcmp (name, PLUG_IN_PROC))
+    {
+      procedure = gimp_image_procedure_new (plug_in, name,
+                                            GIMP_PDB_PROC_TYPE_PLUGIN,
+                                            retinex_run, NULL, NULL);
 
-  values[0].type          = GIMP_PDB_STATUS;
-  values[0].data.d_status = status;
+      gimp_procedure_set_image_types (procedure, "RGB*");
+      gimp_procedure_set_sensitivity_mask (procedure,
+                                           GIMP_PROCEDURE_SENSITIVE_DRAWABLE);
 
-  drawable = gimp_drawable_get (param[2].data.d_drawable);
+      gimp_procedure_set_menu_label (procedure, _("Retine_x..."));
+      gimp_procedure_add_menu_path (procedure, "<Image>/Colors/Tone Mapping");
 
-  if (! gimp_drawable_mask_intersect (drawable->drawable_id,
-                                      &x, &y, &width, &height) ||
+      gimp_procedure_set_documentation (procedure,
+                                        _("Enhance contrast using the "
+                                          "Retinex method"),
+                                        "The Retinex Image Enhancement "
+                                        "Algorithm is an automatic image "
+                                        "enhancement method that enhances "
+                                        "a digital image in terms of dynamic "
+                                        "range compression, color independence "
+                                        "from the spectral distribution of the "
+                                        "scene illuminant, and color/lightness "
+                                        "rendition.",
+                                        name);
+      gimp_procedure_set_attribution (procedure,
+                                      "Fabien Pelisson",
+                                      "Fabien Pelisson",
+                                      "2003");
+
+      GIMP_PROC_ARG_INT (procedure, "scale",
+                         "Scale",
+                         "Biggest scale value",
+                         MIN_GAUSSIAN_SCALE, MAX_GAUSSIAN_SCALE, 240,
+                         G_PARAM_READWRITE);
+
+      GIMP_PROC_ARG_INT (procedure, "nscales",
+                         "N scales",
+                         "Number of scales",
+                         0, MAX_RETINEX_SCALES, 3,
+                         G_PARAM_READWRITE);
+
+      GIMP_PROC_ARG_INT (procedure, "scales-mode",
+                         "Scales mode",
+                         "Retinex distribution through scales",
+                         RETINEX_UNIFORM, RETINEX_HIGH, RETINEX_UNIFORM,
+                         G_PARAM_READWRITE);
+
+      GIMP_PROC_ARG_DOUBLE (procedure, "cvar",
+                            "Cvar",
+                            "Variance value",
+                            0, 4, 1.2,
+                            G_PARAM_READWRITE);
+    }
+
+  return procedure;
+}
+
+static GimpValueArray *
+retinex_run (GimpProcedure        *procedure,
+             GimpRunMode           run_mode,
+             GimpImage            *image,
+             gint                  n_drawables,
+             GimpDrawable        **drawables,
+             const GimpValueArray *args,
+             gpointer              run_data)
+{
+  GimpDrawable *drawable;
+  gint          x, y, width, height;
+
+  gegl_init (NULL, NULL);
+
+  if (n_drawables != 1)
+    {
+      GError *error = NULL;
+
+      g_set_error (&error, GIMP_PLUG_IN_ERROR, 0,
+                   _("Procedure '%s' only works with one drawable."),
+                   PLUG_IN_PROC);
+
+      return gimp_procedure_new_return_values (procedure,
+                                               GIMP_PDB_CALLING_ERROR,
+                                               error);
+    }
+  else
+    {
+      drawable = drawables[0];
+    }
+
+  if (! gimp_drawable_mask_intersect (drawable, &x, &y, &width, &height) ||
       width  < MIN_GAUSSIAN_SCALE ||
       height < MIN_GAUSSIAN_SCALE)
     {
-      status = GIMP_PDB_EXECUTION_ERROR;
-      gimp_drawable_detach (drawable);
-      values[0].data.d_status = status;
-      return;
+      return gimp_procedure_new_return_values (procedure,
+                                               GIMP_PDB_EXECUTION_ERROR,
+                                               NULL);
     }
 
-  gimp_tile_cache_ntiles (2 * (drawable->width / gimp_tile_width () + 1));
-
   switch (run_mode)
     {
     case GIMP_RUN_INTERACTIVE:
-      /*  Possibly retrieve data  */
       gimp_get_data (PLUG_IN_PROC, &rvals);
 
-      /*  First acquire information with a dialog  */
       if (! retinex_dialog (drawable))
-        return;
+        {
+          return gimp_procedure_new_return_values (procedure,
+                                                   GIMP_PDB_CANCEL,
+                                                   NULL);
+        }
       break;
 
     case GIMP_RUN_NONINTERACTIVE:
-      /*  Make sure all the arguments are there!  */
-      if (nparams != 7)
-        {
-          status = GIMP_PDB_CALLING_ERROR;
-        }
-      else
-        {
-          rvals.scale        = (param[3].data.d_int32);
-          rvals.nscales      = (param[4].data.d_int32);
-          rvals.scales_mode  = (param[5].data.d_int32);
-          rvals.cvar         = (param[6].data.d_float);
-        }
+      rvals.scale        = GIMP_VALUES_GET_INT    (args, 0);
+      rvals.nscales      = GIMP_VALUES_GET_INT    (args, 1);
+      rvals.scales_mode  = GIMP_VALUES_GET_INT    (args, 2);
+      rvals.cvar         = GIMP_VALUES_GET_DOUBLE (args, 3);
       break;
 
     case GIMP_RUN_WITH_LAST_VALS:
@@ -244,8 +308,7 @@ run (const gchar      *name,
       break;
     }
 
-  if ((status == GIMP_PDB_SUCCESS) &&
-      (gimp_drawable_is_rgb (drawable->drawable_id)))
+  if (gimp_drawable_is_rgb (drawable))
     {
       gimp_progress_init (_("Retinex"));
 
@@ -254,18 +317,17 @@ run (const gchar      *name,
       if (run_mode != GIMP_RUN_NONINTERACTIVE)
         gimp_displays_flush ();
 
-      /*  Store data  */
       if (run_mode == GIMP_RUN_INTERACTIVE)
         gimp_set_data (PLUG_IN_PROC, &rvals, sizeof (RetinexParams));
     }
   else
     {
-      status = GIMP_PDB_EXECUTION_ERROR;
+      return gimp_procedure_new_return_values (procedure,
+                                               GIMP_PDB_EXECUTION_ERROR,
+                                               NULL);
     }
 
-  gimp_drawable_detach (drawable);
-
-  values[0].data.d_status = status;
+  return gimp_procedure_new_return_values (procedure, GIMP_PDB_SUCCESS, NULL);
 }
 
 
@@ -275,48 +337,48 @@ retinex_dialog (GimpDrawable *drawable)
   GtkWidget *dialog;
   GtkWidget *main_vbox;
   GtkWidget *preview;
-  GtkWidget *table;
+  GtkWidget *grid;
   GtkWidget *combo;
   GtkWidget *scale;
-  GtkObject *adj;
   gboolean   run;
 
-  gimp_ui_init (PLUG_IN_BINARY, FALSE);
+  gimp_ui_init (PLUG_IN_BINARY);
 
-  dialog = gimp_dialog_new (_("Retinex Image Enhancement"), PLUG_IN_BINARY,
+  dialog = gimp_dialog_new (_("Retinex Image Enhancement"), PLUG_IN_ROLE,
                             NULL, 0,
                             gimp_standard_help_func, PLUG_IN_PROC,
 
-                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                            GTK_STOCK_OK,     GTK_RESPONSE_OK,
+                            _("_Cancel"), GTK_RESPONSE_CANCEL,
+                            _("_OK"),     GTK_RESPONSE_OK,
 
                             NULL);
 
-  gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
+  gimp_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
                                            GTK_RESPONSE_OK,
                                            GTK_RESPONSE_CANCEL,
                                            -1);
 
   gimp_window_set_transient (GTK_WINDOW (dialog));
 
-  main_vbox = gtk_vbox_new (FALSE, 12);
+  main_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
   gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 12);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), main_vbox);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+                      main_vbox, TRUE, TRUE, 0);
   gtk_widget_show (main_vbox);
 
-  preview = gimp_zoom_preview_new (drawable);
+  preview = gimp_zoom_preview_new_from_drawable (drawable);
   gtk_box_pack_start (GTK_BOX (main_vbox), preview, TRUE, TRUE, 0);
   gtk_widget_show (preview);
 
   g_signal_connect_swapped (preview, "invalidated",
-                            G_CALLBACK (retinex),
+                            G_CALLBACK (retinex_preview),
                             drawable);
 
-  table = gtk_table_new (4, 3, FALSE);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 6);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 6);
-  gtk_box_pack_start (GTK_BOX (main_vbox), table, FALSE, FALSE, 0);
-  gtk_widget_show (table);
+  grid = gtk_grid_new ();
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 6);
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
+  gtk_box_pack_start (GTK_BOX (main_vbox), grid, FALSE, FALSE, 0);
+  gtk_widget_show (grid);
 
   combo = gimp_int_combo_box_new (_("Uniform"), filter_uniform,
                                   _("Low"),     filter_low,
@@ -325,59 +387,48 @@ retinex_dialog (GimpDrawable *drawable)
 
   gimp_int_combo_box_connect (GIMP_INT_COMBO_BOX (combo), rvals.scales_mode,
                               G_CALLBACK (gimp_int_combo_box_get_active),
-                              &rvals.scales_mode);
+                              &rvals.scales_mode, NULL);
   g_signal_connect_swapped (combo, "changed",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
 
-  gimp_table_attach_aligned (GTK_TABLE (table), 0, 0,
-                             _("_Level:"), 0.0, 0.5,
-                             combo, 2, FALSE);
+  gimp_grid_attach_aligned (GTK_GRID (grid), 0, 0,
+                            _("_Level:"), 0.0, 0.5,
+                            combo, 2);
   gtk_widget_show (combo);
 
-  adj = gimp_scale_entry_new (GTK_TABLE (table), 0, 1,
-                              _("_Scale:"), SCALE_WIDTH, ENTRY_WIDTH,
-                              rvals.scale,
-                              MIN_GAUSSIAN_SCALE, MAX_GAUSSIAN_SCALE, 1, 1, 0,
-                              TRUE, 0, 0, NULL, NULL);
-  scale = GIMP_SCALE_ENTRY_SCALE (adj);
-  gtk_range_set_update_policy (GTK_RANGE (scale), GTK_UPDATE_DISCONTINUOUS);
+  scale = gimp_scale_entry_new (_("_Scale:"), rvals.scale, MIN_GAUSSIAN_SCALE, MAX_GAUSSIAN_SCALE, 0);
 
-  g_signal_connect (adj, "value-changed",
-                    G_CALLBACK (gimp_int_adjustment_update),
+  g_signal_connect (scale, "value-changed",
+                    G_CALLBACK (contrast_retinex_scale_entry_update_int),
                     &rvals.scale);
-  g_signal_connect_swapped (adj, "value-changed",
+  g_signal_connect_swapped (scale, "value-changed",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
+  gtk_grid_attach (GTK_GRID (grid), scale, 0, 1, 3, 1);
+  gtk_widget_show (scale);
 
-  adj = gimp_scale_entry_new (GTK_TABLE (table), 0, 2,
-                              _("Scale _division:"), SCALE_WIDTH, ENTRY_WIDTH,
-                              rvals.nscales,
-                              0, MAX_RETINEX_SCALES, 1, 1, 0,
-                              TRUE, 0, 0, NULL, NULL);
-  scale = GIMP_SCALE_ENTRY_SCALE (adj);
-  gtk_range_set_update_policy (GTK_RANGE (scale), GTK_UPDATE_DISCONTINUOUS);
+  scale = gimp_scale_entry_new (_("Scale _division:"), rvals.nscales, 0, MAX_RETINEX_SCALES, 0);
 
-  g_signal_connect (adj, "value-changed",
-                    G_CALLBACK (gimp_int_adjustment_update),
+  g_signal_connect (scale, "value-changed",
+                    G_CALLBACK (contrast_retinex_scale_entry_update_int),
                     &rvals.nscales);
-  g_signal_connect_swapped (adj, "value-changed",
+  g_signal_connect_swapped (scale, "value-changed",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
+  gtk_grid_attach (GTK_GRID (grid), scale, 0, 2, 3, 1);
+  gtk_widget_show (scale);
 
-  adj = gimp_scale_entry_new (GTK_TABLE (table), 0, 3,
-                              _("Dy_namic:"), SCALE_WIDTH, ENTRY_WIDTH,
-                              rvals.cvar, 0, 4, 0.1, 0.1, 1,
-                              TRUE, 0, 0, NULL, NULL);
-  scale = GIMP_SCALE_ENTRY_SCALE (adj);
-  gtk_range_set_update_policy (GTK_RANGE (scale), GTK_UPDATE_DISCONTINUOUS);
+  scale = gimp_scale_entry_new (_("Dy_namic:"), rvals.cvar, 0, 4, 1);
 
-  g_signal_connect (adj, "value-changed",
-                    G_CALLBACK (gimp_float_adjustment_update),
+  g_signal_connect (scale, "value-changed",
+                    G_CALLBACK (contrast_retinex_scale_entry_update_float),
                     &rvals.cvar);
-  g_signal_connect_swapped (adj, "value-changed",
+  g_signal_connect_swapped (scale, "value-changed",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
+  gtk_grid_attach (GTK_GRID (grid), scale, 0, 3, 3, 1);
+  gtk_widget_show (scale);
 
   gtk_widget_show (dialog);
 
@@ -393,15 +444,15 @@ retinex_dialog (GimpDrawable *drawable)
  */
 static void
 retinex (GimpDrawable *drawable,
-         GimpPreview  *preview)
+         GimpPreview *preview)
 {
-  gint          x, y, width, height;
-  gint          size, bytes;
-  guchar       *src  = NULL;
-  guchar       *psrc = NULL;
-  GimpPixelRgn  dst_rgn, src_rgn;
-
-  bytes = drawable->bpp;
+  GeglBuffer *src_buffer;
+  GeglBuffer *dest_buffer;
+  const Babl *format;
+  guchar     *src  = NULL;
+  guchar     *psrc = NULL;
+  gint        x, y, width, height;
+  gint        size, bytes;
 
   /*
    * Get the size of the current image or its selection.
@@ -413,13 +464,20 @@ retinex (GimpDrawable *drawable,
     }
   else
     {
-      if (! gimp_drawable_mask_intersect (drawable->drawable_id,
+      if (! gimp_drawable_mask_intersect (drawable,
                                           &x, &y, &width, &height))
         return;
 
+      if (gimp_drawable_has_alpha (drawable))
+        format = babl_format ("R'G'B'A u8");
+      else
+        format = babl_format ("R'G'B' u8");
+
+      bytes = babl_format_get_bytes_per_pixel (format);
+
       /* Allocate memory */
       size = width * height * bytes;
-      src = g_try_malloc (sizeof (guchar) * size);
+      src  = g_try_malloc (sizeof (guchar) * size);
 
       if (src == NULL)
         {
@@ -430,10 +488,11 @@ retinex (GimpDrawable *drawable,
       memset (src, 0, sizeof (guchar) * size);
 
       /* Fill allocated memory with pixel data */
-      gimp_pixel_rgn_init (&src_rgn, drawable,
-                           x, y, width, height,
-                           FALSE, FALSE);
-      gimp_pixel_rgn_get_rect (&src_rgn, src, x, y, width, height);
+      src_buffer = gimp_drawable_get_buffer (drawable);
+
+      gegl_buffer_get (src_buffer, GEGL_RECTANGLE (x, y, width, height), 1.0,
+                       format, src,
+                       GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
     }
 
   /*
@@ -448,34 +507,46 @@ retinex (GimpDrawable *drawable,
     }
   else
     {
-      gimp_pixel_rgn_init (&dst_rgn, drawable,
-                           x, y, width, height,
-                           TRUE, TRUE);
-      gimp_pixel_rgn_set_rect (&dst_rgn, psrc, x, y, width, height);
+      dest_buffer = gimp_drawable_get_shadow_buffer (drawable);
+
+      gegl_buffer_set (dest_buffer, GEGL_RECTANGLE (x, y, width, height), 0,
+                       format, psrc,
+                       GEGL_AUTO_ROWSTRIDE);
+
+      g_object_unref (src_buffer);
+      g_object_unref (dest_buffer);
 
       gimp_progress_update (1.0);
 
-      gimp_drawable_flush (drawable);
-      gimp_drawable_merge_shadow (drawable->drawable_id, TRUE);
-      gimp_drawable_update (drawable->drawable_id, x, y, width, height);
+      gimp_drawable_merge_shadow (drawable, TRUE);
+      gimp_drawable_update (drawable, x, y, width, height);
     }
 
   g_free (src);
 }
 
+static void
+retinex_preview (GimpDrawable *drawable,
+                 GimpPreview  *preview)
+{
+  retinex (drawable, preview);
+}
 
 /*
  * calculate scale values for desired distribution.
  */
 static void
-retinex_scales_distribution(gfloat* scales, gint nscales, gint mode, gint s)
+retinex_scales_distribution (gfloat *scales,
+                             gint    nscales,
+                             gint    mode,
+                             gint    s)
 {
   if (nscales == 1)
     { /* For one filter we choose the median scale */
       scales[0] = (gint) s / 2;
     }
   else if (nscales == 2)
-    { /* For two filters whe choose the median and maximum scale */
+    { /* For two filters we choose the median and maximum scale */
       scales[0] = (gint) s / 2;
       scales[1] = (gint) s;
     }
@@ -525,8 +596,6 @@ compute_coefs3 (gauss3_coefs *c, gfloat sigma)
    */
   gfloat q, q2, q3;
 
-  q = 0;
-
   if (sigma >= 2.5)
     {
       q = 0.98711 * sigma - 0.96330;
@@ -599,7 +668,7 @@ gausssmooth (gfloat *in, gfloat *out, gint size, gint rowstride, gauss3_coefs *c
   w2[size+3]= w1[size+3];
   for (i = size, n = i; i >= 0; i--, n--)
     {
-      w2[n]= out[i * rowstride] = (gfloat)(c->B*w1[n] +
+      w2[n]= out[i * rowstride] = (gfloat)(c->B*w1[n+3] +
                                            ((c->b[1]*w2[n+1] +
                                              c->b[2]*w2[n+2] +
                                              c->b[3]*w2[n+3] ) / c->b[0]));
@@ -621,7 +690,6 @@ MSRCR (guchar *src, gint width, gint height, gint bytes, gboolean preview_mode)
   gint          scale,row,col;
   gint          i,j;
   gint          size;
-  gint          pos;
   gint          channel;
   guchar       *psrc = NULL;            /* backup pointer for src buffer */
   gfloat       *dst  = NULL;            /* float buffer for algorithm */
@@ -691,9 +759,10 @@ MSRCR (guchar *src, gint width, gint height, gint bytes, gboolean preview_mode)
     The recursive filtering algorithm needs different coefficients according
     to the selected scale (~ = standard deviation of Gaussian).
    */
-  pos = 0;
   for (channel = 0; channel < 3; channel++)
     {
+      gint pos;
+
       for (i = 0, pos = channel; i < channelsize ; i++, pos += bytes)
          {
             /* 0-255 => 1-256 */
@@ -829,3 +898,17 @@ compute_mean_var (gfloat *src, gfloat *mean, gfloat *var, gint size, gint bytes)
   *var = ( vsquared - (*mean * *mean) );
   *var = sqrt(*var); /* var */
 }
+
+static void
+contrast_retinex_scale_entry_update_int (GimpLabelSpin *entry,
+                                         gint          *value)
+{
+  *value = (gint) gimp_label_spin_get_value (entry);
+}
+
+static void
+contrast_retinex_scale_entry_update_float (GimpLabelSpin *entry,
+                                           gfloat        *value)
+{
+  *value = (gfloat) gimp_label_spin_get_value (entry);
+}
