@@ -4,9 +4,9 @@
  * GIMP Plug-in for Windows Icon files.
  * Copyright (C) 2002 Christian Kreibich <christian@whoop.org>.
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -121,25 +120,29 @@ ico_read_int8 (FILE   *fp,
 }
 
 
-static guint32
+static IcoFileHeader
 ico_read_init (FILE *fp)
 {
   IcoFileHeader header;
+
   /* read and check file header */
-  if (!ico_read_int16 (fp, &header.reserved, 1)
-      || !ico_read_int16 (fp, &header.resource_type, 1)
-      || !ico_read_int16 (fp, &header.icon_count, 1)
-      || header.reserved != 0
-      || header.resource_type != 1)
+  if (! ico_read_int16 (fp, &header.reserved, 1)      ||
+      ! ico_read_int16 (fp, &header.resource_type, 1) ||
+      ! ico_read_int16 (fp, &header.icon_count, 1)    ||
+      header.reserved != 0 ||
+      (header.resource_type != 1 && header.resource_type != 2))
     {
-      return 0;
+      header.icon_count = 0;
+      return header;
     }
-  return header.icon_count;
+
+  return header;
 }
 
 
 static gboolean
 ico_read_size (FILE        *fp,
+               gint32       file_offset,
                IcoLoadInfo *info)
 {
   png_structp png_ptr;
@@ -149,22 +152,25 @@ ico_read_size (FILE        *fp,
   gint32      color_type;
   guint32     magic;
 
-  if ( fseek (fp, info->offset, SEEK_SET) < 0 )
+  if (fseek (fp, info->offset + file_offset, SEEK_SET) < 0)
     return FALSE;
 
   ico_read_int32 (fp, &magic, 1);
+
   if (magic == ICO_PNG_MAGIC)
     {
       png_ptr = png_create_read_struct (PNG_LIBPNG_VER_STRING, NULL, NULL,
                                         NULL);
-      if (! png_ptr )
+      if (! png_ptr)
         return FALSE;
+
       info_ptr = png_create_info_struct (png_ptr);
-      if (! info_ptr )
+      if (! info_ptr)
         {
           png_destroy_read_struct (&png_ptr, NULL, NULL);
           return FALSE;
         }
+
       if (setjmp (png_jmpbuf (png_ptr)))
         {
           png_destroy_read_struct (&png_ptr, NULL, NULL);
@@ -183,8 +189,8 @@ ico_read_size (FILE        *fp,
     }
   else if (magic == 40)
     {
-      if (ico_read_int32 (fp, &info->width, 1)
-          && ico_read_int32 (fp, &info->height, 1))
+      if (ico_read_int32 (fp, &info->width, 1) &&
+          ico_read_int32 (fp, &info->height, 1))
         {
           info->height /= 2;
           D(("ico_read_size: ICO: %ix%i\n", info->width, info->height));
@@ -201,8 +207,10 @@ ico_read_size (FILE        *fp,
 }
 
 static IcoLoadInfo*
-ico_read_info (FILE *fp,
-               gint  icon_count)
+ico_read_info (FILE    *fp,
+               gint     icon_count,
+               gint32   file_offset,
+               GError **error)
 {
   gint            i;
   IcoFileEntry   *entries;
@@ -210,8 +218,11 @@ ico_read_info (FILE *fp,
 
   /* read icon entries */
   entries = g_new (IcoFileEntry, icon_count);
-  if ( fread (entries, sizeof(IcoFileEntry), icon_count, fp) <= 0 )
+  if (fread (entries, sizeof (IcoFileEntry), icon_count, fp) <= 0)
     {
+      g_set_error (error, G_FILE_ERROR, 0,
+                   _("Could not read '%lu' bytes"),
+                   (long unsigned int) sizeof (IcoFileEntry));
       g_free (entries);
       return NULL;
     }
@@ -219,30 +230,41 @@ ico_read_info (FILE *fp,
   info = g_new (IcoLoadInfo, icon_count);
   for (i = 0; i < icon_count; i++)
     {
-      info[i].width = entries[i].width;
+      info[i].width  = entries[i].width;
       info[i].height = entries[i].height;
-      info[i].bpp = GUINT16_FROM_LE (entries[i].bpp);
-      info[i].size = GUINT32_FROM_LE (entries[i].size);
+      info[i].planes = entries[i].planes;
+      info[i].bpp    = GUINT16_FROM_LE (entries[i].bpp);
+      info[i].size   = GUINT32_FROM_LE (entries[i].size);
       info[i].offset = GUINT32_FROM_LE (entries[i].offset);
 
       if (info[i].width == 0 || info[i].height == 0)
         {
-          ico_read_size (fp, info+i);
+          ico_read_size (fp, file_offset, info + i);
         }
 
       D(("ico_read_info: %ix%i (%i bits, size: %i, offset: %i)\n",
          info[i].width, info[i].height, info[i].bpp,
          info[i].size, info[i].offset));
+
+      if (info[i].width == 0 || info[i].height == 0)
+        {
+          g_set_error (error, G_FILE_ERROR, 0,
+                       _("Icon #%d has zero width or height"), i);
+          g_free (info);
+          g_free (entries);
+          return NULL;
+        }
     }
 
   g_free (entries);
+
   return info;
 }
 
 static gboolean
 ico_read_png (FILE    *fp,
               guint32  header,
-              guchar  *buffer,
+              guchar  *buf,
               gint     maxsize,
               gint    *width,
               gint    *height)
@@ -257,10 +279,10 @@ ico_read_png (FILE    *fp,
   gint          i;
 
   png_ptr = png_create_read_struct (PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
-  if (! png_ptr )
+  if (! png_ptr)
     return FALSE;
   info = png_create_info_struct (png_ptr);
-  if (! info )
+  if (! info)
     {
       png_destroy_read_struct (&png_ptr, NULL, NULL);
       return FALSE;
@@ -287,15 +309,15 @@ ico_read_png (FILE    *fp,
   switch (color_type)
     {
     case PNG_COLOR_TYPE_GRAY:
-      png_set_gray_1_2_4_to_8 (png_ptr);
-      if ( bit_depth == 16 )
+      png_set_expand_gray_1_2_4_to_8 (png_ptr);
+      if (bit_depth == 16)
         png_set_strip_16 (png_ptr);
       png_set_gray_to_rgb (png_ptr);
       png_set_add_alpha (png_ptr, 0xff, PNG_FILLER_AFTER);
       break;
     case PNG_COLOR_TYPE_GRAY_ALPHA:
-      png_set_gray_1_2_4_to_8 (png_ptr);
-      if ( bit_depth == 16 )
+      png_set_expand_gray_1_2_4_to_8 (png_ptr);
+      if (bit_depth == 16)
         png_set_strip_16 (png_ptr);
       png_set_gray_to_rgb (png_ptr);
       break;
@@ -320,7 +342,7 @@ ico_read_png (FILE    *fp,
   *width = w;
   *height = h;
   rows = g_new (guint32*, h);
-  rows[0] = (guint32*) buffer;
+  rows[0] = (guint32*) buf;
   for (i = 1; i < h; i++)
     rows[i] = rows[i-1] + w;
   png_read_image (png_ptr, (png_bytepp) rows);
@@ -394,7 +416,7 @@ ico_get_byte_from_data (const guint8 *data,
 static gboolean
 ico_read_icon (FILE    *fp,
                guint32  header_size,
-               guchar  *buffer,
+               guchar  *buf,
                gint     maxsize,
                gint    *width,
                gint    *height)
@@ -428,16 +450,18 @@ ico_read_icon (FILE    *fp,
      data.planes, data.image_size, data.bpp,
      data.used_clrs, data.important_clrs));
 
-  if (data.planes != 1
-      || data.compression != 0)
+  if (data.planes      != 1 ||
+      data.compression != 0)
     {
       D(("skipping image: invalid header\n"));
       return FALSE;
     }
 
-  if (data.bpp != 1 && data.bpp != 4
-      && data.bpp != 8 && data.bpp != 24
-      && data.bpp != 32)
+  if (data.bpp != 1  &&
+      data.bpp != 4  &&
+      data.bpp != 8  &&
+      data.bpp != 24 &&
+      data.bpp != 32)
     {
       D(("skipping image: invalid depth: %i\n", data.bpp));
       return FALSE;
@@ -473,7 +497,7 @@ ico_read_icon (FILE    *fp,
   ico_read_int8 (fp, and_map, length);
   D(("  length of and_map: %i\n", length));
 
-  dest_vec = (guint32 *) buffer;
+  dest_vec = (guint32 *) buf;
   switch (data.bpp)
     {
     case 1:
@@ -577,96 +601,105 @@ ico_read_icon (FILE    *fp,
   return TRUE;
 }
 
-static gint32
+static GimpLayer *
 ico_load_layer (FILE        *fp,
-                gint32       image,
+                GimpImage   *image,
                 gint32       icon_num,
-                guchar      *buffer,
+                guchar      *buf,
                 gint         maxsize,
+                gint32       file_offset,
                 IcoLoadInfo *info)
 {
-  gint          width, height;
-  gint32        layer;
-  guint32       first_bytes;
-  GimpDrawable *drawable;
-  GimpPixelRgn  pixel_rgn;
-  gchar         buf [ICO_MAXBUF];
+  gint        width, height;
+  GimpLayer  *layer;
+  guint32     first_bytes;
+  GeglBuffer *buffer;
+  gchar       name[ICO_MAXBUF];
 
-  if ( fseek (fp, info->offset, SEEK_SET) < 0
-       || !ico_read_int32 (fp, &first_bytes, 1) )
-    return -1;
+  if (fseek (fp, info->offset + file_offset, SEEK_SET) < 0 ||
+      ! ico_read_int32 (fp, &first_bytes, 1))
+    return NULL;
 
   if (first_bytes == ICO_PNG_MAGIC)
     {
-      if (!ico_read_png (fp, first_bytes, buffer, maxsize, &width, &height))
-        return -1;
+      if (!ico_read_png (fp, first_bytes, buf, maxsize, &width, &height))
+        return NULL;
     }
   else if (first_bytes == 40)
     {
-      if (!ico_read_icon (fp, first_bytes, buffer, maxsize, &width, &height))
-        return -1;
+      if (!ico_read_icon (fp, first_bytes, buf, maxsize, &width, &height))
+        return NULL;
     }
   else
     {
-      return -1;
+      return NULL;
     }
 
-
   /* read successfully. add to image */
-  g_snprintf (buf, sizeof (buf), _("Icon #%i"), icon_num+1);
-  layer = gimp_layer_new (image, buf, width, height,
-                          GIMP_RGBA_IMAGE, 100, GIMP_NORMAL_MODE);
-  gimp_image_add_layer (image, layer, icon_num);
-  drawable = gimp_drawable_get (layer);
-  gimp_pixel_rgn_init (&pixel_rgn, drawable, 0, 0,
-                       drawable->width, drawable->height, TRUE, FALSE);
-  gimp_pixel_rgn_set_rect (&pixel_rgn, (const guchar*) buffer,
-                           0, 0, drawable->width,
-                           drawable->height);
-  gimp_drawable_detach (drawable);
+  g_snprintf (name, sizeof (name), _("Icon #%i"), icon_num+1);
+  layer = gimp_layer_new (image, name, width, height,
+                          GIMP_RGBA_IMAGE,
+                          100,
+                          gimp_image_get_default_new_layer_mode (image));
+  gimp_image_insert_layer (image, layer, NULL, icon_num);
+
+  buffer = gimp_drawable_get_buffer (GIMP_DRAWABLE (layer));
+
+  gegl_buffer_set (buffer, GEGL_RECTANGLE (0, 0, width, height), 0,
+                   NULL, buf, GEGL_AUTO_ROWSTRIDE);
+
+  g_object_unref (buffer);
 
   return layer;
 }
 
 
-gint32
-ico_load_image (const gchar  *filename,
+GimpImage *
+ico_load_image (GFile        *file,
+                gint32       *file_offset,
                 GError      **error)
 {
-  FILE        *fp;
-  IcoLoadInfo *info;
-  gint         max_width, max_height;
-  gint         i;
-  gint32       image;
-  gint32       layer;
-  guchar      *buffer;
-  guint        icon_count;
-  gint         maxsize;
+  FILE          *fp;
+  IcoFileHeader  header;
+  IcoLoadInfo   *info;
+  gint           max_width, max_height;
+  gint           i;
+  GimpImage     *image;
+  guchar        *buf;
+  guint          icon_count;
+  gint           maxsize;
+  gchar         *str;
 
-  gimp_progress_init_printf (_("Opening '%s'"),
-                             gimp_filename_to_utf8 (filename));
+  if (! file_offset)
+    gimp_progress_init_printf (_("Opening '%s'"),
+                               gimp_file_get_utf8_name (file));
 
-  fp = g_fopen (filename, "rb");
-  if (! fp )
+  fp = g_fopen (g_file_peek_path (file), "rb");
+
+  if (! fp)
     {
       g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno),
                    _("Could not open '%s' for reading: %s"),
-                   gimp_filename_to_utf8 (filename), g_strerror (errno));
-      return -1;
+                   gimp_file_get_utf8_name (file), g_strerror (errno));
+      return NULL;
     }
 
-  icon_count = ico_read_init (fp);
+  if (file_offset)
+    fseek (fp, *file_offset, SEEK_SET);
+
+  header = ico_read_init (fp);
+  icon_count = header.icon_count;
   if (!icon_count)
     {
       fclose (fp);
-      return -1;
+      return NULL;
     }
 
-  info = ico_read_info (fp, icon_count);
-  if (!info)
+  info = ico_read_info (fp, icon_count, file_offset ? *file_offset : 0, error);
+  if (! info)
     {
       fclose (fp);
-      return -1;
+      return NULL;
     }
 
   /* find width and height of image */
@@ -674,81 +707,278 @@ ico_load_image (const gchar  *filename,
   max_height = 0;
   for (i = 0; i < icon_count; i++)
     {
-      if ( info[i].width > max_width )
+      if (info[i].width > max_width)
         max_width = info[i].width;
-      if ( info[i].height > max_height )
+      if (info[i].height > max_height)
         max_height = info[i].height;
     }
-  if ( max_width <= 0 || max_height <= 0 )
+  if (max_width <= 0 || max_height <= 0)
     {
       g_free (info);
       fclose (fp);
-      return -1;
+      return NULL;
     }
   D(("image size: %ix%i\n", max_width, max_height));
 
   image = gimp_image_new (max_width, max_height, GIMP_RGB);
-  gimp_image_set_filename (image, filename);
+  if (! file_offset)
+    gimp_image_set_file (image, file);
 
   maxsize = max_width * max_height * 4;
-  buffer = g_new (guchar, max_width * max_height * 4);
+  buf = g_new (guchar, max_width * max_height * 4);
   for (i = 0; i < icon_count; i++)
     {
-      layer = ico_load_layer (fp, image, i, buffer, maxsize, info+i);
+      GimpLayer *layer;
+
+      layer = ico_load_layer (fp, image, i, buf, maxsize, file_offset ? *file_offset : 0, info+i);
+
+      /* Save CUR hot spot information */
+      if (header.resource_type == 2)
+        {
+          GimpParasite *parasite;
+
+          str = g_strdup_printf ("%d %d", info[i].planes, info[i].bpp);
+          parasite = gimp_parasite_new ("cur-hot-spot",
+                                        GIMP_PARASITE_PERSISTENT,
+                                        strlen (str) + 1, (gpointer) str);
+          g_free (str);
+          gimp_item_attach_parasite (GIMP_ITEM (layer), parasite);
+          gimp_parasite_free (parasite);
+        }
     }
-  g_free (buffer);
+
+  if (file_offset)
+    *file_offset = ftell (fp);
+
+  g_free (buf);
   g_free (info);
   fclose (fp);
 
+  /* Don't update progress here if .ani file */
+  if (! file_offset)
+    gimp_progress_update (1.0);
+
+  return image;
+}
+
+/* Ported from James Huang's ani.c code, under the GPL license, version 3
+ * or any later version of the license */
+GimpImage *
+ani_load_image (GFile   *file,
+                gboolean load_thumb,
+                gint    *width,
+                gint    *height,
+                GError **error)
+{
+  FILE         *fp;
+  GimpImage    *image = NULL;
+  GimpParasite *parasite;
+  gchar         id[4];
+  guint32       size;
+  guint8        padding;
+  gint32        file_offset;
+  gint          frame = 1;
+  AniFileHeader header;
+  gchar         inam[G_MAXSHORT] = {0};
+  gchar         iart[G_MAXSHORT] = {0};
+  gchar        *str;
+
+  gimp_progress_init_printf (_("Opening '%s'"),
+                             gimp_file_get_utf8_name (file));
+
+  fp = g_fopen (g_file_peek_path (file), "rb");
+
+  if (! fp)
+    {
+      g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno),
+                   _("Could not open '%s' for reading: %s"),
+                   gimp_file_get_utf8_name (file), g_strerror (errno));
+      return NULL;
+    }
+
+  while (fread (id, 1, 4, fp) == 4)
+    {
+      if (memcmp (id, "RIFF", 4) == 0 )
+        {
+          fread (&size, sizeof (size), 1, fp);
+        }
+      else if (memcmp (id, "anih", 4) == 0)
+        {
+          fread (&size, sizeof (size), 1, fp);
+          fread (&header, sizeof (header), 1, fp);
+        }
+      else if (memcmp (id, "rate", 4) == 0)
+        {
+          fread (&size, sizeof (size), 1, fp);
+          fseek (fp, size, SEEK_CUR);
+        }
+      else if (memcmp (id, "seq ", 4) == 0)
+        {
+          fread (&size, sizeof (size), 1, fp);
+          fseek (fp, size, SEEK_CUR);
+        }
+      else if (memcmp (id, "LIST", 4) == 0)
+        {
+          fread (&size, sizeof (size), 1, fp);
+        }
+      else if (memcmp (id, "INAM", 4) == 0)
+        {
+          fread (&size, sizeof (size), 1, fp);
+          fread (&inam, sizeof (char), size, fp);
+          inam[size] = '\0';
+
+          /* Metadata length must be even. If data itself is odd,
+           * then an extra 0x00 is added for padding. We read in
+           * that extra byte to keep loading properly.
+           * See discussion in #8562.
+           */
+          if (size % 2 != 0)
+            fread (&padding, sizeof (padding), 1, fp);
+        }
+      else if (memcmp (id, "IART", 4) == 0)
+        {
+          fread (&size, sizeof (size), 1, fp);
+          fread (&iart, sizeof (char), size, fp);
+          iart[size] = '\0';
+
+          if (size % 2 != 0)
+            fread (&padding, sizeof (padding), 1, fp);
+        }
+      else if (memcmp (id, "icon", 4) == 0)
+        {
+          fread (&size, sizeof (size), 1, fp);
+          file_offset = ftell (fp);
+          if (load_thumb)
+            {
+              image = ico_load_thumbnail_image (file, width, height, file_offset, error);
+              break;
+            }
+           else
+             {
+               if (! image)
+                 {
+                   image = ico_load_image (file, &file_offset, error);
+                 }
+               else
+                 {
+                   GimpImage    *temp_image = NULL;
+                   GimpLayer   **layers;
+                   GimpLayer    *new_layer;
+                   gint          nlayers;
+
+                   temp_image = ico_load_image (file, &file_offset, error);
+                   layers = gimp_image_get_layers (temp_image, &nlayers);
+                   if (layers)
+                     {
+                       for (gint i = 0; i < nlayers; i++)
+                         {
+                           new_layer = gimp_layer_new_from_drawable (GIMP_DRAWABLE (layers[i]),
+                                                                     image);
+                           gimp_image_insert_layer (image, new_layer, NULL, frame);
+                           frame++;
+                         }
+                     }
+                   gimp_image_delete (temp_image);
+                 }
+
+               /* Update position after reading icon data */
+               fseek (fp, file_offset, SEEK_SET);
+               if (header.frames > 0)
+                 gimp_progress_update ((gdouble) frame /
+                                       (gdouble) header.frames);
+             }
+        }
+    }
+  fclose (fp);
+
+  /* Saving header metadata */
+  str = g_strdup_printf ("%d", header.jif_rate);
+  parasite = gimp_parasite_new ("ani-header",
+                                GIMP_PARASITE_PERSISTENT,
+                                strlen (str) + 1, (gpointer) str);
+  g_free (str);
+  gimp_image_attach_parasite (image, parasite);
+  gimp_parasite_free (parasite);
+
+  /* Saving INFO block */
+  if (strlen (inam) > 0)
+    {
+      str = g_strdup_printf ("%s", inam);
+      parasite = gimp_parasite_new ("ani-info-inam",
+                                    GIMP_PARASITE_PERSISTENT,
+                                    strlen (str) + 1, (gpointer) str);
+      g_free (str);
+      gimp_image_attach_parasite (image, parasite);
+      gimp_parasite_free (parasite);
+    }
+
+  if (strlen (iart) > 0)
+    {
+      str = g_strdup_printf ("%s", iart);
+      parasite = gimp_parasite_new ("ani-info-iart",
+                                    GIMP_PARASITE_PERSISTENT,
+                                    strlen (str) + 1, (gpointer) str);
+      g_free (str);
+      gimp_image_attach_parasite (image, parasite);
+      gimp_parasite_free (parasite);
+    }
+
+  gimp_image_set_file (image, file);
   gimp_progress_update (1.0);
 
   return image;
 }
 
-gint32
-ico_load_thumbnail_image (const gchar  *filename,
-                          gint         *width,
-                          gint         *height,
-                          GError      **error)
+GimpImage *
+ico_load_thumbnail_image (GFile   *file,
+                          gint    *width,
+                          gint    *height,
+                          gint32   file_offset,
+                          GError **error)
 {
-  FILE        *fp;
-  IcoLoadInfo *info;
-  gint32       image;
-  gint32       layer;
-  gint         w     = 0;
-  gint         h     = 0;
-  gint         bpp   = 0;
-  gint         match = 0;
-  gint         i, icon_count;
-  guchar      *buffer;
+  FILE          *fp;
+  IcoLoadInfo   *info;
+  IcoFileHeader  header;
+  GimpImage     *image;
+  gint           w     = 0;
+  gint           h     = 0;
+  gint           bpp   = 0;
+  gint           match = 0;
+  gint           i, icon_count;
+  guchar        *buf;
 
   gimp_progress_init_printf (_("Opening thumbnail for '%s'"),
-                             gimp_filename_to_utf8 (filename));
+                             gimp_file_get_utf8_name (file));
 
-  fp = g_fopen (filename, "rb");
-  if (! fp )
+  fp = g_fopen (g_file_peek_path (file), "rb");
+
+  if (! fp)
     {
       g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno),
                    _("Could not open '%s' for reading: %s"),
-                   gimp_filename_to_utf8 (filename), g_strerror (errno));
-      return -1;
+                   gimp_file_get_utf8_name (file), g_strerror (errno));
+      return NULL;
     }
 
-  icon_count = ico_read_init (fp);
-  if (! icon_count )
+  if (file_offset > 0)
+    fseek (fp, file_offset, SEEK_SET);
+
+  header = ico_read_init (fp);
+  icon_count = header.icon_count;
+  if (! icon_count)
     {
       fclose (fp);
-      return -1;
+      return NULL;
     }
 
   D(("*** %s: Microsoft icon file, containing %i icon(s)\n",
-     filename, icon_count));
+     gimp_file_get_utf8_name (file), icon_count));
 
-  info = ico_read_info (fp, icon_count);
-  if (! info )
+  info = ico_read_info (fp, icon_count, file_offset, error);
+  if (! info)
     {
       fclose (fp);
-      return -1;
+      return NULL;
     }
 
   /* Do a quick scan of the icons in the file to find the best match */
@@ -763,9 +993,9 @@ ico_load_thumbnail_image (const gchar  *filename,
 
           match = i;
         }
-      else if ( w == info[i].width
-                && h == info[i].height
-                && info[i].bpp > bpp )
+      else if (w == info[i].width  &&
+               h == info[i].height &&
+               info[i].bpp > bpp)
         {
           /* better quality */
           bpp = info[i].bpp;
@@ -774,12 +1004,12 @@ ico_load_thumbnail_image (const gchar  *filename,
     }
 
   if (w <= 0 || h <= 0)
-    return -1;
+    return NULL;
 
   image = gimp_image_new (w, h, GIMP_RGB);
-  buffer = g_new (guchar, w*h*4);
-  layer = ico_load_layer (fp, image, match, buffer, w*h*4, info+match);
-  g_free (buffer);
+  buf = g_new (guchar, w*h*4);
+  ico_load_layer (fp, image, match, buf, w*h*4, file_offset, info+match);
+  g_free (buf);
 
   *width  = w;
   *height = h;
