@@ -15,43 +15,30 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __PSD_H__
 #define __PSD_H__
 
-/* Temporary disable of save functionality */
-#ifdef PSD_SAVE
-#undef PSD_SAVE
-#else
-/* #define PSD_SAVE */
-#endif
 
 /* Set to the level of debugging output you want, 0 for none.
  *   Setting higher than 2 will result in a very large amount of debug
  *   output being produced. */
-#define PSD_DEBUG 0
+#define PSD_DEBUG 3
 #define IFDBG(level) if (PSD_DEBUG >= level)
 
-/* Set to FALSE to supress pop-up warnings about lossy file conversions */
+/* Set to FALSE to suppress pop-up warnings about lossy file conversions */
 #define CONVERSION_WARNINGS             FALSE
 
 #define LOAD_PROC                       "file-psd-load"
+#define LOAD_MERGED_PROC                "file-psd-load-merged"
 #define LOAD_THUMB_PROC                 "file-psd-load-thumb"
 #define SAVE_PROC                       "file-psd-save"
 #define PLUG_IN_BINARY                  "file-psd"
 #define PLUG_IN_ROLE                    "gimp-file-psd"
 
-#define DECODE_XMP_PROC                 "plug-in-metadata-decode-xmp"
-
 #define GIMP_PARASITE_COMMENT           "gimp-comment"
-#define GIMP_PARASITE_ICC_PROFILE       "icc-profile"
-#define GIMP_PARASITE_EXIF              "exif-data"
-#define GIMP_PARASITE_IPTC              "iptc-data"
-
-#define METADATA_PARASITE               "gimp-metadata"
-#define METADATA_MARKER                 "GIMP_XMP_1"
 
 #define PSD_PARASITE_DUOTONE_DATA       "psd-duotone-data"
 
@@ -61,7 +48,8 @@
 #define MAX_RAW_SIZE    0               /* FIXME all images are raw if 0 */
 
 /* PSD spec defines */
-#define MAX_CHANNELS    56              /* Photoshop CS to CS3 support 56 channels */
+/* Although the spec still says 56 there is a test image with more so let's go a little higher to 99 */
+#define MAX_CHANNELS    99              /* Photoshop CS to CS3 support 56 channels */
 
 /* PSD spec constants */
 
@@ -83,6 +71,8 @@
 #define PSD_LADJ_INVERT         "nvrt"          /* Adjustment layer - invert (PS4) */
 #define PSD_LADJ_THRESHOLD      "thrs"          /* Adjustment layer - threshold (PS4) */
 #define PSD_LADJ_POSTERIZE      "post"          /* Adjustment layer - posterize (PS4) */
+#define PSD_LADJ_VIBRANCE       "vibA"          /* Adjustment layer - vibrance (PS10) */
+#define PSD_LADJ_COLOR_LOOKUP   "clrL"          /* Adjustment layer - color lookup (PS13) */
 
 /* Fill Layer IDs */
 #define PSD_LFIL_SOLID          "SoCo"          /* Solid color sheet setting (PS6) */
@@ -107,6 +97,7 @@
 #define PSD_LPRP_PROTECT        "lspf"          /* Protected setting (PS6) */
 #define PSD_LPRP_COLOR          "lclr"          /* Sheet color setting (PS6) */
 #define PSD_LPRP_REF_POINT      "fxrp"          /* Reference point (PS6) */
+#define PSD_LPRP_VERSION        "lyvr"          /* Layer version (PS7) */
 
 /* Vector mask */
 #define PSD_LMSK_VMASK          "vmsk"          /* Vector mask setting (PS6) */
@@ -116,13 +107,24 @@
 
 /* Other */
 #define PSD_LOTH_SECTION        "lsct"          /* Section divider setting - Layer groups (PS6) */
+#define PSD_LOTH_SECTION2       "lsdk"          /* Nested section divider setting - Layer groups (CS5) */
 #define PSD_LOTH_PATTERN        "Patt"          /* Patterns (PS6) */
+#define PSD_LOTH_PATTERN_2      "Pat2"          /* Patterns 2nd key (PS6) */
+#define PSD_LOTH_PATTERN_3      "Pat3"          /* Patterns 3rd key (PS6) */
 #define PSD_LOTH_GRADIENT       "grdm"          /* Gradient settings (PS6) */
 #define PSD_LOTH_RESTRICT       "brst"          /* Channel blending restriction setting (PS6) */
 #define PSD_LOTH_FOREIGN_FX     "ffxi"          /* Foreign effect ID (PS6) */
 #define PSD_LOTH_PATT_DATA      "shpa"          /* Pattern data (PS6) */
 #define PSD_LOTH_META_DATA      "shmd"          /* Meta data setting (PS6) */
 #define PSD_LOTH_LAYER_DATA     "layr"          /* Layer data (PS6) */
+#define PSD_LOTH_CONTENT_GEN    "CgEd"          /* Content generator extra data (PS12) */
+#define PSD_LOTH_TEXT_ENGINE    "Txt2"          /* Text engine data (PS10) */
+#define PSD_LOTH_PATH_NAME      "pths"          /* Unicode path name (PS13) */
+#define PSD_LOTH_ANIMATION_FX   "anFX"          /* Animation effects (PS13) */
+#define PSD_LOTH_FILTER_MASK    "FMsk"          /* Filter mask (PS10) */
+#define PSD_LOTH_VECTOR_STROKE  "vscg"          /* Vector stroke data (PS13) */
+#define PSD_LOTH_ALIGN_RENDER   "sn2P"          /* Aligned rendering flag (?) */
+#define PSD_LOTH_USER_MASK      "LMsk"          /* User mask (?) */
 
 /* Effects layer resource IDs */
 #define PSD_LFX_COMMON          "cmnS"          /* Effects layer - common state (PS5) */
@@ -132,9 +134,27 @@
 #define PSD_LFX_INNER_GLW       "iglw"          /* Effects layer - inner glow (PS5) */
 #define PSD_LFX_BEVEL           "bevl"          /* Effects layer - bevel (PS5) */
 
+/* Placed Layer */
+#define PSD_LPL_PLACE_LAYER     "plLd"          /* Placed layer (?) */
+#define PSD_LPL_PLACE_LAYER_NEW "SoLd"          /* Placed layer (PS10) */
+
+/* Linked Layer */
+#define PSD_LLL_LINKED_LAYER    "lnkD"          /* Linked layer (?) */
+#define PSD_LLL_LINKED_LAYER_2  "lnk2"          /* Linked layer 2nd key */
+#define PSD_LLL_LINKED_LAYER_3  "lnk3"          /* Linked layer 3rd key */
+
+/* Merged Transparency */
+#define PSD_LMT_MERGE_TRANS     "Mtrn"          /* Merged transparency save flag (?) */
+#define PSD_LMT_MERGE_TRANS_16  "Mt16"          /* Merged transparency save flag 2 */
+#define PSD_LMT_MERGE_TRANS_32  "Mt32"          /* Merged transparency save flag 3 */
+
+/* Filter Effects */
+#define PSD_LFFX_FILTER_FX      "FXid"          /* Filter effects (?) */
+#define PSD_LFFX_FILTER_FX_2    "FEid"          /* Filter effects 2 */
+
 /* PSD spec enums */
 
-/* Image colour modes */
+/* Image color modes */
 typedef enum {
   PSD_BITMAP       = 0,                 /* Bitmap image */
   PSD_GRAYSCALE    = 1,                 /* Greyscale image */
@@ -146,13 +166,13 @@ typedef enum {
   PSD_LAB          = 9                  /* L*a*b image */
 } PSDColorMode;
 
-/* Image colour spaces */
+/* Image color spaces */
 typedef enum {
   PSD_CS_RGB       = 0,                 /* RGB */
   PSD_CS_HSB       = 1,                 /* Hue, Saturation, Brightness */
   PSD_CS_CMYK      = 2,                 /* CMYK */
   PSD_CS_PANTONE   = 3,                 /* Pantone matching system (Lab)*/
-  PSD_CS_FOCOLTONE = 4,                 /* Focoltone colour system (CMYK)*/
+  PSD_CS_FOCOLTONE = 4,                 /* Focoltone color system (CMYK)*/
   PSD_CS_TRUMATCH  = 5,                 /* Trumatch color (CMYK)*/
   PSD_CS_TOYO      = 6,                 /* Toyo 88 colorfinder 1050 (Lab)*/
   PSD_CS_LAB       = 7,                 /* L*a*b*/
@@ -166,19 +186,19 @@ typedef enum {
 typedef enum {
   PSD_PS2_IMAGE_INFO    = 1000,         /* 0x03e8 - Obsolete - ps 2.0 image info */
   PSD_MAC_PRINT_INFO    = 1001,         /* 0x03e9 - Optional - Mac print manager print info record */
-  PSD_PS2_COLOR_TAB     = 1003,         /* 0x03eb - Obsolete - ps 2.0 indexed colour table */
+  PSD_PS2_COLOR_TAB     = 1003,         /* 0x03eb - Obsolete - ps 2.0 indexed color table */
   PSD_RESN_INFO         = 1005,         /* 0x03ed - ResolutionInfo structure */
   PSD_ALPHA_NAMES       = 1006,         /* 0x03ee - Alpha channel names */
-  PSD_DISPLAY_INFO      = 1007,         /* 0x03ef - DisplayInfo structure */
+  PSD_DISPLAY_INFO      = 1007,         /* 0x03ef - Superceded by PSD_DISPLAY_INFO_NEW for ps CS3 and higher - DisplayInfo structure */
   PSD_CAPTION           = 1008,         /* 0x03f0 - Optional - Caption string */
   PSD_BORDER_INFO       = 1009,         /* 0x03f1 - Border info */
-  PSD_BACKGROUND_COL    = 1010,         /* 0x03f2 - Background colour */
+  PSD_BACKGROUND_COL    = 1010,         /* 0x03f2 - Background color */
   PSD_PRINT_FLAGS       = 1011,         /* 0x03f3 - Print flags */
   PSD_GREY_HALFTONE     = 1012,         /* 0x03f4 - Greyscale and multichannel halftoning info */
-  PSD_COLOR_HALFTONE    = 1013,         /* 0x03f5 - Colour halftoning info */
+  PSD_COLOR_HALFTONE    = 1013,         /* 0x03f5 - Color halftoning info */
   PSD_DUOTONE_HALFTONE  = 1014,         /* 0x03f6 - Duotone halftoning info */
   PSD_GREY_XFER         = 1015,         /* 0x03f7 - Greyscale and multichannel transfer functions */
-  PSD_COLOR_XFER        = 1016,         /* 0x03f8 - Colour transfer functions */
+  PSD_COLOR_XFER        = 1016,         /* 0x03f8 - Color transfer functions */
   PSD_DUOTONE_XFER      = 1017,         /* 0x03f9 - Duotone transfer functions */
   PSD_DUOTONE_INFO      = 1018,         /* 0x03fa - Duotone image information */
   PSD_EFFECTIVE_BW      = 1019,         /* 0x03fb - Effective black & white values for dot range */
@@ -198,8 +218,8 @@ typedef enum {
   PSD_COPYRIGHT_FLG     = 1034,         /* 0x040a - Copyright flag */
   PSD_URL               = 1035,         /* 0x040b - URL string */
   PSD_THUMB_RES2        = 1036,         /* 0x040c - Thumbnail resource */
-  PSD_GLOBAL_ANGLE      = 1037,         /* 0x040d - Global angle */
-  PSD_COLOR_SAMPLER     = 1038,         /* 0x040e - Colour samplers resource */
+  PSD_GLOBAL_ANGLE      = 1037,         /* 0x040d - Superceded by PSD_NEW_COLOR_SAMPLER for ps CS3 and higher - Global angle */
+  PSD_COLOR_SAMPLER     = 1038,         /* 0x040e - Superceded by PSD_NEW_COLOR_SAMPLER for ps CS3 and higher - Color samplers resource */
   PSD_ICC_PROFILE       = 1039,         /* 0x040f - ICC Profile */
   PSD_WATERMARK         = 1040,         /* 0x0410 - Watermark */
   PSD_ICC_UNTAGGED      = 1041,         /* 0x0411 - Do not use ICC profile flag */
@@ -207,8 +227,8 @@ typedef enum {
   PSD_SPOT_HALFTONE     = 1043,         /* 0x0413 - Spot halftone */
   PSD_DOC_IDS           = 1044,         /* 0x0414 - Document specific IDs */
   PSD_ALPHA_NAMES_UNI   = 1045,         /* 0x0415 - Unicode alpha names */
-  PSD_IDX_COL_TAB_CNT   = 1046,         /* 0x0416 - Indexed colour table count */
-  PSD_IDX_TRANSPARENT   = 1047,         /* 0x0417 - Index of transparent colour (if any) */
+  PSD_IDX_COL_TAB_CNT   = 1046,         /* 0x0416 - Indexed color table count */
+  PSD_IDX_TRANSPARENT   = 1047,         /* 0x0417 - Index of transparent color (if any) */
   PSD_GLOBAL_ALT        = 1049,         /* 0x0419 - Global altitude */
   PSD_SLICES            = 1050,         /* 0x041a - Slices */
   PSD_WORKFLOW_URL_UNI  = 1051,         /* 0x041b - Workflow URL - Unicode string */
@@ -216,11 +236,40 @@ typedef enum {
   PSD_ALPHA_ID          = 1053,         /* 0x041d - Alpha IDs */
   PSD_URL_LIST_UNI      = 1054,         /* 0x041e - URL list - unicode */
   PSD_VERSION_INFO      = 1057,         /* 0x0421 - Version info */
-  PSD_EXIF_DATA         = 1058,         /* 0x0422 - Exif data block */
+  PSD_EXIF_DATA         = 1058,         /* 0x0422 - Exif data block 1 */
+  PSD_EXIF_DATA_3       = 1059,         /* 0X0423 - Exif data block 3 (?) */
   PSD_XMP_DATA          = 1060,         /* 0x0424 - XMP data block */
+  PSD_CAPTION_DIGEST    = 1061,         /* 0x0425 - Caption digest */
+  PSD_PRINT_SCALE       = 1062,         /* 0x0426 - Print scale */
+  PSD_PIXEL_AR          = 1064,         /* 0x0428 - Pixel aspect ratio */
+  PSD_LAYER_COMPS       = 1065,         /* 0x0429 - Layer comps */
+  PSD_ALT_DUOTONE_COLOR = 1066,         /* 0x042A - Alternative Duotone colors */
+  PSD_ALT_SPOT_COLOR    = 1067,         /* 0x042B - Alternative Spot colors */
+  PSD_LAYER_SELECT_ID   = 1069,         /* 0x042D - Layer selection ID */
+  PSD_HDR_TONING_INFO   = 1070,         /* 0x042E - HDR toning information */
+  PSD_PRINT_INFO_SCALE  = 1071,         /* 0x042F - Print scale */
+  PSD_LAYER_GROUP_E_ID  = 1072,         /* 0x0430 - Layer group(s) enabled ID */
+  PSD_COLOR_SAMPLER_NEW = 1073,         /* 0x0431 - Color sampler resource for ps CS3 and higher PSD files */
+  PSD_MEASURE_SCALE     = 1074,         /* 0x0432 - Measurement scale */
+  PSD_TIMELINE_INFO     = 1075,         /* 0x0433 - Timeline information */
+  PSD_SHEET_DISCLOSE    = 1076,         /* 0x0434 - Sheet discloser */
+  PSD_DISPLAY_INFO_NEW  = 1077,         /* 0x0435 - DisplayInfo structure for ps CS3 and higher PSD files */
+  PSD_ONION_SKINS       = 1078,         /* 0x0436 - Onion skins */
+  PSD_COUNT_INFO        = 1080,         /* 0x0438 - Count information*/
+  PSD_PRINT_INFO        = 1082,         /* 0x043A - Print information added in ps CS5*/
+  PSD_PRINT_STYLE       = 1083,         /* 0x043B - Print style */
+  PSD_MAC_NSPRINTINFO   = 1084,         /* 0x043C - Mac NSPrintInfo*/
+  PSD_WIN_DEVMODE       = 1085,         /* 0x043D - Windows DEVMODE */
+  PSD_AUTO_SAVE_PATH    = 1086,         /* 0x043E - Auto save file path */
+  PSD_AUTO_SAVE_FORMAT  = 1087,         /* 0x043F - Auto save format */
   PSD_PATH_INFO_FIRST   = 2000,         /* 0x07d0 - First path info block */
   PSD_PATH_INFO_LAST    = 2998,         /* 0x0bb6 - Last path info block */
   PSD_CLIPPING_PATH     = 2999,         /* 0x0bb7 - Name of clipping path */
+  PSD_PLUGIN_R_FIRST    = 4000,         /* 0x0FA0 - First plugin resource */
+  PSD_PLUGIN_R_LAST     = 4999,         /* 0x1387 - Last plugin resource */
+  PSD_IMAGEREADY_VARS   = 7000,         /* 0x1B58 - Imageready variables */
+  PSD_IMAGEREADY_DATA   = 7001,         /* 0x1B59 - Imageready data sets */
+  PSD_LIGHTROOM_WORK    = 8000,         /* 0x1F40 - Lightroom workflow */
   PSD_PRINT_FLAGS_2     = 10000         /* 0x2710 - Print flags */
 } PSDImageResID;
 
@@ -260,6 +309,7 @@ typedef enum {
 
 /* Channel ID */
 typedef enum {
+  PSD_CHANNEL_EXTRA_MASK= -3,           /* User supplied extra layer mask */
   PSD_CHANNEL_MASK      = -2,           /* User supplied layer mask */
   PSD_CHANNEL_ALPHA     = -1,           /* Transparency mask */
   PSD_CHANNEL_RED       =  0,           /* Red channel data */
@@ -294,7 +344,7 @@ typedef enum {
 typedef gint32  Fixed;                  /* Represents a fixed point implied decimal */
 
 
-/* Apple colour space data structures */
+/* Apple color space data structures */
 
 /* RGB Color Value
    A color value expressed in the RGB color space is composed of red, green,
@@ -306,7 +356,7 @@ typedef struct
   guint16       red;
   guint16       green;
   guint16       blue;
-}CMRGBColor;
+} CMRGBColor;
 
 /*  HSV Color Value
     A color value expressed in the HSV color space is composed of hue,
@@ -321,7 +371,7 @@ typedef struct
   guint16       hue;
   guint16       saturation;
   guint16       value;
-}CMHSVColor;
+} CMHSVColor;
 
 /* CMYK Color Value
   A color value expressed in the CMYK color space is composed of cyan, magenta,
@@ -336,7 +386,7 @@ typedef struct
   guint16       magenta;
   guint16       yellow;
   guint16       black;
-}CMCMYKColor;
+} CMCMYKColor;
 
 /* L*a*b* Color Value
    The first three values in the color data are, respectively, the colors
@@ -363,25 +413,33 @@ typedef struct
 } CMGrayColor ;
 
 /* The color union is defined by the CMColor type definition.
-*/
+ */
 typedef union
 {
-   CMRGBColor        rgb;
-   CMHSVColor        hsv;
-   CMLabColor        Lab;
-   CMCMYKColor       cmyk;
-   CMGrayColor       gray;
+  CMRGBColor        rgb;
+  CMHSVColor        hsv;
+  CMLabColor        Lab;
+  CMCMYKColor       cmyk;
+  CMGrayColor       gray;
 } CMColor;
 
+/* GIMP layer mode info */
+typedef struct
+{
+  GimpLayerMode          mode;
+  GimpLayerColorSpace    blend_space;
+  GimpLayerColorSpace    composite_space;
+  GimpLayerCompositeMode composite_mode;
+} LayerModeInfo;
 
 /* Image resolution data */
 typedef struct {
   Fixed         hRes;                   /* Horizontal resolution pixels/inch */
-  gint16        hResUnit;               /* Horizontal display resolution unit */
-  gint16        widthUnit;              /* Width unit ?? */
+  gint16        hResUnit;               /* Horizontal display resolution unit (1=pixels per inch, 2=pixels per cm) */
+  gint16        widthUnit;              /* Display width unit (1=inches; 2=cm; 3=points; 4=picas; 5=columns) */
   Fixed         vRes;                   /* Vertical resolution pixels/inch */
   gint16        vResUnit;               /* Vertical display resolution unit */
-  gint16        heightUnit;             /* Height unit ?? */
+  gint16        heightUnit;             /* Display height unit */
 } ResolutionInfo;
 
 /* Grid & guide header */
@@ -410,20 +468,28 @@ typedef struct {
   gint16        planes;                 /* Number of planes (always 1) */
 } ThumbnailInfo;
 
-/* Channel display info data */
+/* Channel display info data for Adobe Photoshop CS2 and lower */
 typedef struct {
-  gint16        colorSpace;             /* Colour space from  PSDColorSpace */
+  gint16        colorSpace;             /* Color space from PSDColorSpace */
   guint16       color[4];               /* 4 * 16 bit color components */
   gint16        opacity;                /* Opacity 0 to 100 */
   gchar         kind;                   /* Selected = 0, Protected = 1 */
   gchar         padding;                /* Padding */
 } DisplayInfo;
 
+/* Channel display info data for Adobe Photoshop CS3 and higher to support floating point colors */
+typedef struct {
+  gint16        colorSpace;             /* Color space from PSDColorSpace */
+  guint16       color[4];               /* 4 * 16 bit color components */
+  gint16        opacity;                /* Opacity 0 to 100 */
+  gchar         mode;                   /* Alpha = 0, Inverted alpha = 1, Spot = 2 */
+} DisplayInfoNew;
+
 /* PSD Channel length info data structure */
 typedef struct
 {
   gint16        channel_id;             /* Channel ID */
-  guint32       data_len;               /* Layer left */
+  guint64       data_len;               /* Layer left */
 } ChannelLengthInfo;
 
 /* PSD Layer flags */
@@ -441,9 +507,38 @@ typedef struct
 {
   gboolean      relative_pos;           /* Mask position recorded relative to layer */
   gboolean      disabled;               /* Mask disabled */
-  gboolean      invert;                 /* Invert mask on blending */
+  gboolean      invert;                 /* Invert mask on blending (obsolete according to online specs) */
+  gboolean      rendered;               /* User mask actually came from rendering other data */
+  gboolean      params_present;         /* User and/or vector masks have parameters applied to them */
 } MaskFlags;
 
+/* PSD Slices */
+typedef struct
+{
+  gint32        id;                     /* ID */
+  gint32        groupid;                /* Group ID */
+  gint32        origin;                 /* Origin */
+  gint32        associatedid;           /* Associated Layer ID */
+  gchar         *name;                  /* Name */
+  gint32        type;                   /* Type */
+  gint32        left;                   /* Position coordinates */
+  gint32        top;
+  gint32        right;
+  gint32        bottom;
+  gchar         *url;                   /* URL */
+  gchar         *target;                /* Target */
+  gchar         *message;               /* Message */
+  gchar         *alttag;                /* Alt Tag */
+  gchar         html;                   /* Boolean for if cell text is HTML */
+  gchar         *celltext;              /* Cell text */
+  gint32        horizontal;             /* Horizontal alignment */
+  gint32        vertical;               /* Vertical alignment */
+  gchar         alpha;                  /* Alpha */
+  gchar         red;                    /* Red */
+  gchar         green;                  /* Green */
+  gchar         blue;                   /* Blue */
+} PSDSlice;
+
 /* PSD Layer mask data (length 20) */
 typedef struct
 {
@@ -451,10 +546,11 @@ typedef struct
   gint32                left;                   /* Layer left */
   gint32                bottom;                 /* Layer bottom */
   gint32                right;                  /* Layer right */
-  guchar                def_color;              /* Default background colour */
+  guchar                def_color;              /* Default background color */
   guchar                flags;                  /* Layer flags */
-  guchar                extra_def_color;        /* Real default background colour */
+  guchar                mask_params;            /* Mask parameters. Only present if bit 4 of flags is set. */
   guchar                extra_flags;            /* Real layer flags */
+  guchar                extra_def_color;        /* Real user mask background */
   MaskFlags             mask_flags;             /* Flags */
 } LayerMask;
 
@@ -467,6 +563,18 @@ typedef struct
   gint32                right;                  /* Layer right */
 } LayerMaskExtra;
 
+/* PSD text reading */
+typedef struct
+{
+  gdouble               xx; /* Transform information */
+  gdouble               xy;
+  gdouble               yx;
+  gdouble               yy;
+  gdouble               tx;
+  gdouble               ty;
+  gchar                 *info; /* Text information */
+} PSDText;
+
 /* PSD Layer data structure */
 typedef struct
 {
@@ -481,16 +589,19 @@ typedef struct
   gchar                 blend_mode[4];          /* Blend mode */
   guchar                opacity;                /* Opacity - 0 = transparent ... 255 = opaque */
   guchar                clipping;               /* Clipping */
+  guchar                clipping_group_type;    /* Used to track group needed for clipping (1 = group start, 2 = group end) */
   guchar                flags;                  /* Layer flags */
   guchar                filler;                 /* Filler */
-  guint32               extra_len;              /* Extra data length */
+  guint64               extra_len;              /* Extra data length */
   gchar                *name;                   /* Layer name */
-  guint32               mask_len;               /* Layer mask data length */
+  guint64               mask_len;               /* Layer mask data length */
   LayerMask             layer_mask;             /* Layer mask data */
   LayerMaskExtra        layer_mask_extra;       /* Layer mask extra data */
   LayerFlags            layer_flags;            /* Layer flags */
+  PSDText               text;                   /* PSD text */
   guint32               id;                     /* Layer ID (Tattoo) */
   guchar                group_type;             /* 0 -> not a group; 1 -> open folder; 2 -> closed folder; 3 -> end of group */
+  guint16               color_tag[4];           /* 4 * 16 bit color components */
 } PSDlayer;
 
 /* PSD Channel data structure */
@@ -508,9 +619,10 @@ typedef struct
 {
   GimpRGB       gimp_color;             /* Gimp RGB color */
   gint16        opacity;                /* Opacity */
+  guchar        ps_mode;                /* PS mode flag */
   guchar        ps_kind;                /* PS type flag */
-  gint16        ps_cspace;              /* PS colour space */
-  CMColor       ps_color;               /* PS colour */
+  gint16        ps_cspace;              /* PS color space */
+  CMColor       ps_color;               /* PS color */
 } PSDchanneldata;
 
 /* PSD Image Resource data structure */
@@ -519,8 +631,8 @@ typedef struct
   gchar         type[4];                /* Image resource type */
   gint16        id;                     /* Image resource ID */
   gchar         name[256];              /* Image resource name (pascal string) */
-  gint32        data_start;             /* Image resource data start */
-  gint32        data_len;               /* Image resource data length */
+  guint64       data_start;             /* Image resource data start */
+  guint64       data_len;               /* Image resource data length */
 } PSDimageres;
 
 /* PSD Layer Resource data structure */
@@ -528,41 +640,49 @@ typedef struct
 {
   gchar         sig[4];                 /* Layer resource signature */
   gchar         key[4];                 /* Layer resource key */
-  gint32        data_start;             /* Layer resource data start */
-  gint32        data_len;               /* Layer resource data length */
+  guint64       data_start;             /* Layer resource data start */
+  guint64       data_len;               /* Layer resource data length */
 } PSDlayerres;
 
 /* PSD File data structures */
 typedef struct
 {
+  gboolean              merged_image_only;      /* Whether to load only the merged image data */
+
+  guint16               version;                /* Version 1 (PSD) or 2 (PSB) */
   guint16               channels;               /* Number of channels: 1- 56 */
   gboolean              transparency;           /* Image has merged transparency alpha channel */
   guint32               rows;                   /* Number of rows: 1 - 30000 */
   guint32               columns;                /* Number of columns: 1 - 30000 */
-  guint16               bps;                    /* Bits per channel: 1, 8 or 16 */
-  guint16               color_mode;             /* Image colour mode: {PSDColorMode} */
-  GimpImageBaseType     base_type;              /* Image base colour mode: (GIMP) */
+  guint16               bps;                    /* Bits per sample: 1, 8, 16, or 32 */
+  guint16               color_mode;             /* Image color mode: {PSDColorMode} */
+  GimpImageBaseType     base_type;              /* Image base color mode: (GIMP) */
   guint16               comp_mode;              /* Merged image compression mode */
-  guchar               *color_map;              /* Colour map data */
-  guint32               color_map_len;          /* Colour map data length */
-  guint32               color_map_entries;      /* Colour map number of entries */
-  guint32               image_res_start;        /* Image resource block start address */
-  guint32               image_res_len;          /* Image resource block length */
-  guint32               mask_layer_start;       /* Mask & layer block start address */
-  guint32               mask_layer_len;         /* Mask & layer block length */
+  guchar               *color_map;              /* Color map data */
+  guint32               color_map_len;          /* Color map data length */
+  guint32               color_map_entries;      /* Color map number of entries */
+  guint64               image_res_start;        /* Image resource block start address */
+  guint64               image_res_len;          /* Image resource block length */
+  guint64               mask_layer_start;       /* Mask & layer block start address */
+  guint64               mask_layer_len;         /* Mask & layer block length */
   gint16                num_layers;             /* Number of layers */
-  guint32               layer_data_start;       /* Layer pixel data start */
-  guint32               layer_data_len;         /* Layer pixel data length */
-  guint32               merged_image_start;     /* Merged image pixel data block start address */
-  guint32               merged_image_len;       /* Merged image pixel data block length */
+  guint64               layer_data_start;       /* Layer pixel data start */
+  guint64               layer_data_len;         /* Layer pixel data length */
+  guint64               merged_image_start;     /* Merged image pixel data block start address */
+  guint64               merged_image_len;       /* Merged image pixel data block length */
   gboolean              no_icc;                 /* Do not use ICC profile */
-  guint16               layer_state;            /* Active layer number counting from bottom up */
+  guint16               layer_state;            /* Active layer index counting from bottom up */
+  GList                *layer_selection;        /* Selected layer IDs (GIMP layer tattoos) */
   GPtrArray            *alpha_names;            /* Alpha channel names */
   PSDchanneldata      **alpha_display_info;     /* Alpha channel display info */
   guint16               alpha_display_count;    /* Number of alpha channel display info recs */
   guint32              *alpha_id;               /* Alpha channel ids (tattoos) */
   guint16               alpha_id_count;         /* Number of alpha channel id items */
   guint16               quick_mask_id;          /* Channel number containing quick mask */
+
+  GimpColorProfile     *cmyk_profile;
+  gpointer              cmyk_transform;
+  gpointer              cmyk_transform_alpha;
 } PSDimage;
 
 /* Public functions */
